Unit EditorPropertiesForm;

Interface

Uses
   Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   StdCtrls, ComCtrls, ExtCtrls, Spin, EditorMainForm, Global;

Type
  TFrmProperties = Class(TForm)
    PagProperties: TPageControl;
    Tab3DModel: TTabSheet;
    CboType: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    SpnWaterHeight: TSpinEdit;
    Label3: TLabel;
    SpnAlpha: TSpinEdit;
    Label4: TLabel;
    SpnFloorHeight: TSpinEdit;
    Shape1: TShape;
    SpnCeilingHeight0: TSpinEdit;
    SpnCeilingHeight1: TSpinEdit;
    SpnCeilingHeight3: TSpinEdit;
    SpnCeilingHeight2: TSpinEdit;
    Label5: TLabel;
    BtnApply: TButton;
    BtnOk: TButton;
    BtnCancel: TButton;
    TabTextures: TTabSheet;
    Label6: TLabel;
    CboTexFloor: TComboBox;
    CboTexCeiling: TComboBox;
    CboTexWall: TComboBox;
    CboTexPedestalFloor: TComboBox;
    CboTexPedestalCeiling: TComboBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    TabOffsets: TTabSheet;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    SpnOfsFloor: TSpinEdit;
    SpnOfsCeiling: TSpinEdit;
    SpnOfsWall: TSpinEdit;
    SpnOfsPedestalCeiling: TSpinEdit;
    SpnOfsPedestalFloor: TSpinEdit;
    Procedure BtnOkClick(Sender: TObject);
    Procedure BtnCancelClick(Sender: TObject);
    Procedure FormShow(Sender: TObject);
    Procedure BtnApplyClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
  Private
    { Private-Deklarationen }
  Public
    { Public-Deklarationen }
    ActBlock: ^TEditorBlock;
    CallBack: Procedure;
    Procedure UpdateTextureList(TextureList: TComboBox);
    Procedure UpdateTextureLists;
    Procedure SelectProperListItem(TextureNum: Integer; TextureList: TComboBox);
    Procedure UpdateProperties(Const Block: TEditorBlock; CallBackAdr: Pointer);
    Procedure Apply;
  End;

Var
   FrmProperties: TFrmProperties;

Implementation

{$R *.DFM}

Procedure TFrmProperties.BtnOkClick(Sender: TObject);
Begin
     Apply;
     Close;
End;

Procedure TFrmProperties.BtnCancelClick(Sender: TObject);
Begin
     Close;
End;

Procedure TFrmProperties.BtnApplyClick(Sender: TObject);
Begin
     Apply;
End;

Procedure TFrmProperties.UpdateTextureList;
Var
   i: Integer;
Begin
     Try
        TextureList.Items.BeginUpdate;
        TextureList.Items.Clear;
        TextureList.Items.AddObject('', TObject(NoTextureAssigned));
        For i:=0 To FrmMain.TextureNum - 1 Do Begin
            TextureList.Items.AddObject(FrmMain.Textures[i].Name, TObject(i));
        End;
     Finally
        TextureList.Items.EndUpdate;
     End;
End;

Procedure TFrmProperties.UpdateTextureLists;
Begin
     UpdateTextureList(CboTexFloor);
     UpdateTextureList(CboTexCeiling);
     UpdateTextureList(CboTexWall);
     UpdateTextureList(CboTexPedestalFloor);
     UpdateTextureList(CboTexPedestalCeiling);
End;

Procedure TFrmProperties.SelectProperListItem;
Var
   Index: Integer;
Begin
     Index:=TextureList.Items.IndexOfObject(TObject(TextureNum));
     If Index >= 0 Then TextureList.ItemIndex:=Index;
End;

Procedure TFrmProperties.UpdateProperties;
Begin
     ActBlock:=@Block;
     UpdateTextureLists;
     CallBack:=CallBackAdr;
     // Seite "3D-Modell"
     CboType.ItemIndex:=ActBlock^.BlockType;
     SpnWaterHeight.Value:=Round(ActBlock^.WaterHeight);
     SpnAlpha.Value:=ActBlock^.Opacity;
     SpnFloorHeight.Value:=Round(ActBlock^.FloorHeight);
     SpnCeilingHeight0.Value:=Round(ActBlock^.CeilingHeight[0]);
     SpnCeilingHeight1.Value:=Round(ActBlock^.CeilingHeight[1]);
     SpnCeilingHeight2.Value:=Round(ActBlock^.CeilingHeight[2]);
     SpnCeilingHeight3.Value:=Round(ActBlock^.CeilingHeight[3]);
     // Seite "Texturen"
     SelectProperListItem(ActBlock^.Textures[QW_TexFloor], CboTexFloor);
     SelectProperListItem(ActBlock^.Textures[QW_TexCeiling], CboTexCeiling);
     SelectProperListItem(ActBlock^.Textures[QW_TexWall], CboTexWall);
     SelectProperListItem(ActBlock^.Textures[QW_TexPedestalFloor], CboTexPedestalFloor);
     SelectProperListItem(ActBlock^.Textures[QW_TexPedestalCeiling], CboTexPedestalCeiling);
     // Seite "Offsets"
     SpnOfsFloor.Value:=ActBlock^.TextureOffsets[QW_TexFloor];
     SpnOfsCeiling.Value:=ActBlock^.TextureOffsets[QW_TexCeiling];
     SpnOfsWall.Value:=ActBlock^.TextureOffsets[QW_TexWall];
     SpnOfsPedestalFloor.Value:=ActBlock^.TextureOffsets[QW_TexPedestalFloor];
     SpnOfsPedestalCeiling.Value:=ActBlock^.TextureOffsets[QW_TexPedestalCeiling];
End;

Procedure TFrmProperties.FormShow(Sender: TObject);
Begin
     FrmMain.SpdProperties.Down:=True;
     PagProperties.ActivePage:=Tab3DModel;
     BtnOk.SetFocus;
End;

Procedure TFrmProperties.Apply;
Begin
     // Seite "3D-Modell"
     ActBlock^.BlockType:=CboType.ItemIndex;
     ActBlock^.WaterHeight:=SpnWaterHeight.Value;
     ActBlock^.Opacity:=SpnAlpha.Value;
     ActBlock^.FloorHeight:=SpnFloorHeight.Value;
     ActBlock^.CeilingHeight[0]:=SpnCeilingHeight0.Value;
     ActBlock^.CeilingHeight[1]:=SpnCeilingHeight1.Value;
     ActBlock^.CeilingHeight[2]:=SpnCeilingHeight2.Value;
     ActBlock^.CeilingHeight[3]:=SpnCeilingHeight3.Value;
     // Seite "Texturen"
     ActBlock^.Textures[QW_TexFloor]:=Word(CboTexFloor.Items.Objects[CboTexFloor.ItemIndex]);
     ActBlock^.Textures[QW_TexCeiling]:=Word(CboTexCeiling.Items.Objects[CboTexCeiling.ItemIndex]);
     ActBlock^.Textures[QW_TexWall]:=Word(CboTexWall.Items.Objects[CboTexWall.ItemIndex]);
     ActBlock^.Textures[QW_TexPedestalFloor]:=Word(CboTexPedestalFloor.Items.Objects[CboTexPedestalFloor.ItemIndex]);
     ActBlock^.Textures[QW_TexPedestalCeiling]:=Word(CboTexPedestalCeiling.Items.Objects[CboTexPedestalCeiling.ItemIndex]);
     // Seite "Offsets"
     ActBlock^.TextureOffsets[QW_TexFloor]:=SpnOfsFloor.Value;
     ActBlock^.TextureOffsets[QW_TexCeiling]:=SpnOfsCeiling.Value;
     ActBlock^.TextureOffsets[QW_TexWall]:=SpnOfsWall.Value;
     ActBlock^.TextureOffsets[QW_TexPedestalFloor]:=SpnOfsPedestalFloor.Value;
     ActBlock^.TextureOffsets[QW_TexPedestalCeiling]:=SpnOfsPedestalCeiling.Value;
     // ggf. CallBack-Prozedur ausfhren
     If Assigned(CallBack) Then CallBack;
End;

procedure TFrmProperties.FormHide(Sender: TObject);
begin
     FrmMain.SpdProperties.Down:=False;
end;

End.
