unit EditorTextureEditorForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, StdCtrls, ComCtrls, HiColor, Global, EditorPropertiesForm;

type
  TFrmTextureEditor = class(TForm)
    LVTextures: TListView;
    Label1: TLabel;
    BtnAdd: TButton;
    BtnDelete: TButton;
    GrpTexture: TGroupBox;
    LstFiles: TListBox;
    RdoAnimationType: TRadioGroup;
    PntTexture: TPaintBox;
    Bevel1: TBevel;
    LblSubTexture: TLabel;
    SpdAddFile: TSpeedButton;
    SpdRemoveFile: TSpeedButton;
    SpdMoveUp: TSpeedButton;
    SpdMoveDown: TSpeedButton;
    Bevel2: TBevel;
    BtnClose: TButton;
    Label2: TLabel;
    DlgOpen: TOpenDialog;
    procedure BtnCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure LVTexturesEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure LVTexturesKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LVTexturesClick(Sender: TObject);
    procedure RdoAnimationTypeClick(Sender: TObject);
    procedure SpdAddFileClick(Sender: TObject);
    procedure SpdRemoveFileClick(Sender: TObject);
    procedure SpdMoveUpClick(Sender: TObject);
    procedure SpdMoveDownClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure PntTexturePaint(Sender: TObject);
    procedure LstFilesClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  FrmTextureEditor: TFrmTextureEditor;

Procedure CreateStretchedSprite(Var DestSprite: HC_Sprite; Width, Height: Integer; Const SourceSprite: HC_Sprite);

implementation

Uses
    EditorMainForm;

{$R *.DFM}

Procedure ShowSubTexture(Texture, SubTexture: Integer);
Var
   x, y: Integer;
   r, g, b: Byte;
   Col: Integer;
Begin
     With FrmTextureEditor Do Begin
          If (Texture < 0) Or (SubTexture < 0) Then Begin
             PntTexture.Canvas.Brush.Color:=clBtnFace;
             PntTexture.Canvas.Brush.Style:=bsSolid;
             PntTexture.Canvas.FillRect(Rect(0, 0, BlockSize, BlockSize));
             LblSubTexture.Caption:='';
          End Else With FrmMain.Textures[Texture].SubTextures[SubTexture] Do Begin
             PntTexture.Canvas.Brush.Color:=clBtnFace;
             PntTexture.Canvas.Brush.Style:=bsSolid;
             PntTexture.Canvas.FillRect(Rect(0, 0, BlockSize, BlockSize));
             LblSubTexture.Caption:=Filename;
             For y:=0 To Data.Height-1 Do Begin
                 For x:=0 To Data.Width-1 Do Begin
                     HC_EncodeHiColor(HC_GetPixelSprite(x, y, Data), r, g, b);
                     Asm
                        pushad
                        mov  al, r
                        shl  al, 3
                        mov  ah, g
                        shl  ah, 3
                        mov  Word Ptr [Col], ax
                        mov  al, b
                        shl  al, 3
                        mov  ah, 0
                        mov  Word ptr [Col+2], ax
                        popad
                     End;
//                     Col:=Integer(b)+Integer(g)*255+Integer(r)*65536;
                     PntTexture.Canvas.Pixels[x, y]:=Col;
                 End;
             End;
          End;
     End;
End;

Procedure AllowTextureElements(Allow: Boolean);
Begin
     FrmTextureEditor.SpdMoveUp.Enabled:=Allow;
     FrmTextureEditor.SpdMoveDown.Enabled:=Allow;
     FrmTextureEditor.SpdAddFile.Enabled:=Allow;
     FrmTextureEditor.SpdRemoveFile.Enabled:=Allow;
     FrmTextureEditor.RdoAnimationType.Enabled:=Allow;
End;

Procedure SetTextureElements;
Var
   i: Integer;
Begin
     With FrmTextureEditor Do Begin
          LstFiles.Items.Clear;
          If LVTextures.ItemFocused = Nil Then Begin
             RdoAnimationType.ItemIndex:=0;
             AllowTextureElements(False);
          End Else Begin
             AllowtextureElements(True);
             With TEditorTexture(LVTextures.ItemFocused.Data^) Do Begin
                  For i:=0 To SubTextureNum-1 Do
                      LstFiles.Items.Add(Subtextures[i].FileName);
                  RdoAnimationType.ItemIndex:=AnimationType;
             End;
             If LstFiles.Items.Count > 0 then LstFiles.ItemIndex := 0;
             PntTexturePaint(Nil);
          End;
     End;
End;

Procedure UpdateLVTextures;
Var
   i: Integer;
   NewItem: TListItem;
Begin
     FrmTextureEditor.LVTextures.Items.Clear;
     For i:=0 To FrmMain.TextureNum-1 Do Begin
         NewItem:=FrmTextureEditor.LVTextures.Items.Add;
         NewItem.Caption:=FrmMain.Textures[i].Name;
         NewItem.Data:=@FrmMain.Textures[i];
     End;
End;

procedure TFrmTextureEditor.BtnCloseClick(Sender: TObject);
begin
     Close;
end;

procedure TFrmTextureEditor.FormShow(Sender: TObject);
begin
     FrmProperties.Close;
     UpdateLVTextures;
     If LVTextures.Items.Count > 0 Then Begin
        LVTextures.AlphaSort;
        LVTextures.Items[0].Selected:=True;
        LVTextures.ItemFocused:=LVTextures.Items[0];
        LVTextures.SetFocus;
     End;
     SetTextureElements;
End;

procedure TFrmTextureEditor.BtnAddClick(Sender: TObject);
Var
   NewItem: TListItem;
   i: Integer;
begin
     If FrmMain.TextureNum > MaxTextures Then
        MessageDlg('Die Anzahl der Texturen ist auf '+Stri(MaxTextures+1)+' begrenzt.', mtError, [mbOk], 0)
     Else Begin
        NewItem:=LVTextures.Items.Add;
        NewItem.Data:=@FrmMain.Textures[FrmMain.TextureNum];
        With TEditorTexture(NewItem.Data^) Do Begin
             Index:=FrmMain.TextureNum;
             Name:='Neue Textur';
             AnimationType:=animRepeat;
             SubTextureNum:=0;
             For i:=0 To MaxSubTextures Do Begin
                 SubTextures[i].Filename:='';
                 SubTextures[i].Data.Width:=0;
                 SubTextures[i].Data.Height:=0;
                 SubTextures[i].Data.Addr:=Nil;
             End;
        End;
        Inc(FrmMain.TextureNum);
        NewItem.Caption:='Neue Textur';
        LVTextures.SetFocus;
        NewItem.EditCaption;
        SetTextureElements;
     End;
end;

procedure TFrmTextureEditor.LVTexturesEdited(Sender: TObject;
  Item: TListItem; var S: String);
begin
     TEditorTexture(Item.Data^).Name:=s;
end;

procedure TFrmTextureEditor.LVTexturesKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     If Key In [VK_Up, VK_Down] Then LVTexturesClick(Nil);
end;

procedure TFrmTextureEditor.LVTexturesClick(Sender: TObject);
begin
     SetTextureElements;
end;

procedure TFrmTextureEditor.RdoAnimationTypeClick(Sender: TObject);
begin
     If LVTextures.ItemFocused <> Nil Then Begin
        TEditorTexture(LVTextures.ItemFocused.Data^).AnimationType:=RdoAnimationType.ItemIndex;
     End;
end;

procedure TFrmTextureEditor.SpdAddFileClick(Sender: TObject);
Var
   i: Integer;
   Dummy: HC_Sprite;
   NewWidth, NewHeight: Integer;
begin
     If LVTextures.ItemFocused <> Nil Then Begin
        If TEditorTexture(LVTextures.itemFocused.Data^).SubTextureNum > MaxSubTextures Then Begin
           MessageDlg('Mehr als '+Stri(MaxSubTextures+1)+' Dateien pro Textur sind nicht zulssig.', mtError, [mbOk], 0);
           Exit;
        End;
        DlgOpen.InitialDir:=FrmMain.AppPath+QW_TexturePath;
        If DlgOpen.Execute Then If DlgOpen.Files.Count > 0 Then Begin
           For i:=0 To DlgOpen.Files.Count-1 Do Begin
               If TEditorTexture(LVTextures.itemFocused.Data^).SubTextureNum > MaxSubTextures Then Begin
                  MessageDlg('Mehr als '+Stri(MaxSubTextures+1)+' Dateien pro Textur sind nicht zulssig.', mtError, [mbOk], 0);
                  Exit;
               End Else With TEditorTexture(LVTextures.ItemFocused.Data^) Do Begin
                  SubTextures[SubTextureNum].Filename:=ExtractFileName(DlgOpen.Files[i]);
                  // Sprite einladen und auf richtige Gre bringen (64x64)
                  HC_Result;
                  HC_LoadPCX8Bit(FrmMain.AppPath+QW_TexturePath+SubTextures[SubTextureNum].Filename, Dummy);
                  If HC_Result <> HC_NoError Then Begin
                     MessageDlg('Fehler beim Laden von "'+FrmMain.AppPath+QW_TexturePath+SubTextures[SubTextureNum].Filename+'".', mtError, [mbOk], 0);
                     Exit;
                  End;
                  If Dummy.Width >= Dummy.Height Then Begin
                     NewWidth:=BlockSize;
                     NewHeight:=Trunc((BlockSize / Dummy.Width) * Dummy.Height);
                  End Else Begin
                     NewWidth:=Trunc((BlockSize / Dummy.Height) * Dummy.Width);
                     NewHeight:=BlockSize;
                  End;
                  CreateStretchedSprite(SubTextures[SubTextureNum].Data, NewWidth,
                                        NewHeight, Dummy);
                  HC_FreeSprite(Dummy);
                  LstFiles.Items.Add(SubTextures[SubTextureNum].Filename);
                  Inc(SubTextureNum);
               End;
           End;
           PntTexturePaint(Nil);
        End;
     End;
end;

procedure TFrmTextureEditor.SpdRemoveFileClick(Sender: TObject);
Var
   i: Integer;
   Dummy: Integer;
begin
     If LVTextures.ItemFocused <> Nil Then Begin
        If LstFiles.ItemIndex >= 0 Then With TEditorTexture(LVTextures.ItemFocused.Data^) Do Begin
           If SubTextureNum > 0 Then Begin
              If Assigned(SubTextures[LstFiles.ItemIndex].Data.Addr) Then
                 FreeMem(SubTextures[LstFiles.ItemIndex].Data.Addr);
              Dec(SubTextureNum);
              For i:=LstFiles.ItemIndex To SubTextureNum-1 Do
                  SubTextures[i]:=SubTextures[i+1];
              Dummy:=LstFiles.ItemIndex;
              LstFiles.Items.Delete(Dummy);
              If LstFiles.Items.Count > 0 Then Begin
                 If LstFiles.Items.Count <= Dummy Then
                    LstFiles.ItemIndex:=Dummy-1
                 Else
                    LstFiles.ItemIndex:=Dummy;
              End;
              PntTexturePaint(Nil);
           End;
        End;
     End;
end;

procedure TFrmTextureEditor.SpdMoveUpClick(Sender: TObject);
Var
   Dummy: TEditorSubTexture;
   Dummy2: Integer;
begin
     If LVTextures.ItemFocused <> Nil Then With TEditorTexture(LVTextures.ItemFocused.Data^) Do Begin
        If LstFiles.ItemIndex > 0 Then Begin
           // Eintrge im Array vertauschen
           Dummy:=SubTextures[LstFiles.ItemIndex-1];
           SubTextures[LstFiles.ItemIndex-1]:=SubTextures[LstFiles.ItemIndex];
           SubTextures[LstFiles.ItemIndex]:=Dummy;
           // Eintrge im Listenfeld vertauschen
           Dummy2:=LstFiles.ItemIndex;
           LstFiles.Items.Exchange(Dummy2, Dummy2-1);
           LstFiles.ItemIndex:=Dummy2-1;
        End;
     End;
end;

procedure TFrmTextureEditor.SpdMoveDownClick(Sender: TObject);
Var
   Dummy: TEditorSubTexture;
   Dummy2: Integer;
begin
     If LVTextures.ItemFocused <> Nil Then With TEditorTexture(LVTextures.ItemFocused.Data^) Do Begin
        If LstFiles.ItemIndex < LstFiles.Items.Count-1 Then Begin
           // Eintrge im Array vertauschen
           Dummy:=SubTextures[LstFiles.ItemIndex+1];
           SubTextures[LstFiles.ItemIndex+1]:=SubTextures[LstFiles.ItemIndex];
           SubTextures[LstFiles.ItemIndex]:=Dummy;
           // Eintrge im Listenfeld vertauschen
           Dummy2:=LstFiles.ItemIndex;
           LstFiles.Items.Exchange(Dummy2, Dummy2+1);
           LstFiles.ItemIndex:=Dummy2+1;
        End;
     End;
end;

procedure TFrmTextureEditor.BtnDeleteClick(Sender: TObject);
Var
   i, m: Integer;
   x, y: Integer;
   Dummy: Integer;
begin
     If LVTextures.ItemFocused <> Nil Then With TEditorTexture(LVTextures.ItemFocused.Data^) Do Begin
        LVTextures.ItemFocused.CancelEdit;
        If FrmMain.TextureNum > 0 Then Begin
           // berprfen, ob die Textur noch benutzt wird
           For y:=0 To QW_MapHeight - 1 Do Begin
               For x:=0 To QW_MapWidth - 1 Do Begin
                   For i:=0 To QW_MaxTexturesPerBlock Do
                       If FrmMain.Map[x, y].Textures[i] = Index Then Begin
                          MessageDlg('Die Textur wird noch benutzt. Sie kann nicht gelscht werden.', mtError, [mbOk], 0);
                          Exit;
                       End;
               End;
           End;
           For i:=0 To MaxDefinedBlocks Do Begin
               For m:=0 To QW_MaxTexturesPerBlock Do
                   If FrmMain.DefinedBlocks[i].Properties.Textures[m] = Index Then Begin
                      MessageDlg('Die Textur wird noch benutzt. Sie kann nicht gelscht werden.', mtError, [mbOk], 0);
                      Exit;
                   End;
           End;
           // Die Textur-Indezies der Blcke ggf. berichtigen
           for y:=0 to QW_Mapheight-1 do begin
               for x:=0 to QW_MapWidth-1 do begin
                   for i:=0 to QW_MaxTexturesPerBlock do begin
                       if (frmMain.map[x, y].Textures[i] > Index) and (frmMain.map[x, y].Textures[i] < NoTextureAssigned) then
                          dec(frmMain.map[x, y].Textures[i]);
                   end;
               end;
           end;
           // Unter-Texturen freigeben
           For i:=0 To SubTextureNum-1 Do
               If Assigned(SubTextures[i].Data.Addr) Then
                  HC_FreeSprite(SubTextures[i].Data);
           // Aktuelle Textur mit nachfolgenden berschreiben
           Dec(FrmMain.TextureNum);
           For i:=Index To FrmMain.TextureNum-1 Do Begin
               FrmMain.Textures[i]:=FrmMain.Textures[i+1];
           End;
           // Indecies "berichtigen", weil durch das Lschen ein Sprung entstanden ist
           For i:=0 To FrmMain.TextureNum-1 Do
               FrmMain.Textures[i].Index:=i;
           Dummy:=LVTextures.ItemFocused.Index;
           UpdateLVTextures;
           If LVTextures.Items.Count > 0 Then Begin
              If LVTextures.Items.Count <= Dummy Then
                 LVTextures.ItemFocused:=LVTextures.Items[Dummy-1]
              Else
                 LVTextures.ItemFocused:=LVTextures.Items[Dummy];
              LVTextures.ItemFocused.Selected:=True;
              LVTextures.ItemFocused.MakeVisible(False);
           End;
           LVTextures.SetFocus;
           SetTextureElements;
        End;
     End;
end;

Procedure CreateStretchedSprite;
Var
   ActX, ActY: Integer;
   a, b: Integer;
   MinAdvanceX, MinAdvanceY: Integer;
   OddAdvanceX, OddAdvanceY: Integer;
   OddX, OddY: Integer;
   Color: Word;
   Tmp: Integer;
Begin
     HC_CreateSprite(Width, Height, DestSprite);
     MinAdvanceX:=SourceSprite.Width Div Width;
     OddAdvanceX:=SourceSprite.Width Mod Width;
     MinAdvanceY:=SourceSprite.Height Div Height;
     OddAdvanceY:=SourceSprite.Height Mod Height;
     ActY:=0;
     OddX:=0;
     OddY:=0;
     For b:=0 To Height - 1 Do Begin
         ActX:=0;
         Color:=HC_GetPixelSprite(ActX, ActY, SourceSprite);
         For a:=0 To Width - 1 Do Begin
             HC_PutPixelSprite(a, b, Color, DestSprite);
             Tmp:=ActX;
             Inc(ActX, MinAdvanceX);
             Inc(OddX, OddAdvanceX);
             If OddX >= Width Then Begin
                Dec(OddX, Width);
                Inc(ActX);
             End;
             If ActX <> Tmp Then Color:=HC_GetPixelSprite(ActX, ActY, SourceSprite);
         End;
         Inc(ActY, MinAdvanceY);
         Inc(OddY, OddAdvanceY);
         If OddY >= Height Then Begin
            Dec(OddY, Height);
            Inc(ActY);
         End;
     End;
End;

procedure TFrmTextureEditor.PntTexturePaint(Sender: TObject);
begin
     If (LVTextures.ItemFocused <> Nil) And (LstFiles.ItemIndex >= 0) Then Begin
        ShowSubTexture(TEditorTexture(LVTextures.ItemFocused.Data^).Index, LstFiles.ItemIndex);
     End Else Begin
        ShowSubTexture(-1, -1);
     End;
end;

procedure TFrmTextureEditor.LstFilesClick(Sender: TObject);
begin
     PntTexturePaint(Nil);
end;

end.
