Unit HiColor2;

{

   Diese Unit Stellt Routinen fr Spritegrafik per DirectX (ab v6.0) zur
   Verfgung.

   Copyright (c) by Jan Strter 2000.

   *******************************************************************

   *******************************************************************
   Beispiel fr eine typische Zeichenroutine:
   -------------------------------------------------------------------

           Am besten ist es, wenn alle Zeichenoperationen sich
           innerhalb EINER Routine befinden.
           Auerdem sollte immer berprft werden, ob die Unit
           berhaupt initialisiert wurde.
           z.B.:

           Procedure MyApp.DoOnIdle(Sender: TObject; Done: Boolean);
           Begin

                // Abfragen, ob die Unit initialisiert wurde...
                If HC_Initialized Then Begin

                   // Offscreen-Buffer locken und abfragen, ob der Zeiger
                   // auch gltig ist, man kann nie sicher genug sein!
                   If HC_Lock <> Nil Then Begin

                      HC_PutSprite(...          // Offscreen-Zeichnen...
                      HC_Line(...
                      ...

                      HC_ShowBuf;               // Ohne das hier sieht man nix!

                   End; // If (2)

                End; // If (1)

           End;  // Procedure


   *******************************************************************

   Die Unit sollte im OnShow-Ereignis des Hauptfensters initialisiert
   werden.
   Zum Auffrischen des Bildschirms sollte eine OnIdle-Prozedur oder
   ein Timer benutzt werden.
   Auch sollte der Mauszeiger abgeschaltet werden, da er nur die
   Anzeige verunstaltet.

   *******************************************************************

   *******************************************************************

}

Interface

Uses
    Windows, DirectDraw, Dialogs, Classes;

Type
    HC_ShadowPalType = Array[0..65535] Of Word;

    HC_Sprite = Record
       Width, Height: Longint;
       Addr: Pointer;
    End;

    HC_BJSCut = Packed Record
       Left, Top, Right, Bottom: Integer;
    End;
    HC_PBJSCut = ^HC_BJSCut;

    HC_BJSPos = Packed Record
       FileStart, FileLength: Integer;
    End;
    HC_PBJSPos = ^HC_BJSPos;

Function HC_Result: Integer;
{
  Funktioniert analog zu IOResult.
}
Procedure HC_ExtractRGBBitCount(pf: TDDPixelFormat);
{
  Liest die RGB-Bit-Anteile des angegebenen Pixelformats aus.
}
Procedure HC_Init(Width, Height: Word; Hndl: HDC);
{
  Initialisiert die Unit und erstellt ein neues WinG-Handle.
}
Procedure HC_Done;
{
  Gibt die durch WinG benutzten Ressourcen wieder frei.
}
Function  HC_Lock: Pointer;
{
  Ruft HC_Back.Lock auf, schreibt die Adresse in HC_BufPtr und gibt
  sie als Ergebnis zurck.
  HC_Lock MUSS aufgerufen werden, bevor Routinen aufgerufen werden,
  die HC_BufPtr benutzen!
  Jedem Aufruf von HC_Lock MUSS auch irgendwann ein Aufruf von HC_UnLock
  folgen, damit Windows den Speicherblock wieder ordnungsgem
  verwalten kann.
}
Procedure HC_UnLock;
{
  Gegenstck zu HC_Lock. Ruft HC_Back.UnLock auf und setzt
  HC_BufPtr auf Nil.
  HC_UnLock MUSS aufgerufen werden, wenn alle Zeichenoperationen
  beendet sind und das Programm wieder in die Warteschleife eintreten
  soll!
  HC_UnLock wird von HC_ShowBuf automatisch aufgerufen.
}
Procedure HC_ShowBuf;
{
  Kopiert den Video-Buffer (HC_Back) per HC_Primary.Flip in den Videospeicher.
}
Procedure HC_ClearBuf(Col: Word);
{
  Lscht den Video-Buffer mit der angegeben Farbe.
  (Sollte mit HC_MakeHiColor erstellt werden...)
}
Function HC_EncodeColor(r, g, b: Byte): Word;
{
  Erstellt einen 16-Bit-Wert aus den angegeben Teilen
  Rot, Grn und Blau. Bei R=0, G=0 und B=0 ist die Farbe
  schwarz, bei R=31, G=31 und B=31 ist sie wei.
  31 ist der hchste Wert fr jeden Farbanteil.
}
Procedure HC_DecodeColor(Col: Word; Var r, g, b: Byte);
{
  Schlsselt den 16-Bit-Farbcode COL in seine einzelnen
  Rot-, Grn- und Blau-Anteile auf.
}
Procedure HC_PutPixel(x, y: Longint; Col: Word);
{
  Zeichnet einen Pixel mit der Farbe Col an der Position
  (x/y) im Video-Buffer.
}
Procedure HC_PutPixelSave(x, y: Longint; Col: Word);
{
  Zeichnet einen Pixel mit der Farbe Col an der Position
  (x/y) im Video-Buffer, prft vorher, ob die Position gltig ist.
}
Function HC_GetPixel(x, y: Longint): Word;
{
  Gibt den Farbwert an Position (x/y) zurck.
  Ist die angegebene Position ungltig, wird 0 zurck gegeben.
}
Procedure HC_PutPixelSprite(x, y: Longint; Col: Word; DestSprite: HC_Sprite);
{
  Zeichnet einen Pixel mit der Farbe Col an der Position
  (x/y) in DESTSPRITE, prft vorher, ob die Position gltig ist.
}
Function HC_GetPixelSprite(x, y: Longint; SourceSprite: HC_Sprite): Word;
{
  Gibt den Farbwert an Position (x/y)  im Sprite SOURCESPRITE zurck.
  Ist die angegebene Position ungltig, wird 0 zurck gegeben.
}
Procedure HC_CreateSprite(Width, Height: Longint; Var DestSprite: HC_Sprite);
{
  Erstellt ein leeres Sprite mit den Ausmaen Width*Height.
}
Procedure HC_LoadPCX8Bit(FileName: String; Var DestSprite: HC_Sprite);
{
  Ldt eine 8-Bit PCX-Datei.
}
Procedure HC_LoadBJS8Bit(FileName: String; Var DestSprite: HC_Sprite; Cut: HC_PBJSCut; FPos: HC_PBJSPos);
{
  Ldt unser tolles Format.
}
Procedure HC_GetSprite(x, y, Width, Height: Longint; Var DestSprite: HC_Sprite);
{
  Erstellt ein Sprite aus dem Video-Puffer.
  Linke obere Ecke ist (x/y), rechte untere Ecke ist (x+Width/y+Height).
}
Procedure HC_FreeSprite(Var SourceSprite: HC_Sprite);
{
  Lscht das angegebene Sprite aus dem Speicher.
}
Procedure HC_SetWindow(x1, y1, x2, y2: Longint);
{
  Setzt das Grafikfenster.
}
Procedure HC_GetWindow(Var x1, y1, x2 ,y2: Longint);
{
  Gibt die Koordinaten des Grafikfensters zurck.
}
Procedure HC_ClearWindow(Col: Word);
{
  Lscht das Grafik-Fenster mit der Farbe COL.
}
Procedure HC_Line(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet eine Linie vom Punkt (x1/y1) bis (x2/y2).
}
Procedure HC_Rect(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet ein Rechteck von (x1/y1) bis (x2/y2) mit
  der Farbe COL.
}
Procedure HC_FillRect(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet ein geflltes Rechteck von (x1/y1) bis (x2/y2) mit
  der Farbe COL.
}
Procedure HC_Circle(x, y, r: Longint; Col: Word);
{
 Zeichnet einen Kreis mit der Farbe COL an X/Y mit dem Radius R.
}
Procedure HC_Ellipse(x, y, rx, ry: Longint; Col: Word);
{
 Zeichnet eine Ellipse mit der Farbe COL an X/Y.
 RX und RY geben die Ausdehnung der Ellipse in X- und Y-Richtung an.
}
Procedure HC_MakeShadowPal(r, g, b, Intensity: Longint);
{
 Berechnet die Schattenpalette mit den angegebenen Farbanteilen.
}
Procedure HC_FreeShadowPal;
{
 Gibt den durch die Schattenpalette belegten Speicher wieder frei.
}
Procedure HC_PutSprite(x, y: Longint; SourceSprite: HC_Sprite);
{
  Zeichnet das Sprite SOURCESPRITE an (x/y). Pixel, die einen
  Farbwert von 0 haben (R=0, G=0 und B=0), werden nicht gezeichnet.
  Falls das Sprite ganz oder teilweise auerhalb des Grafikfensters
  ist, wird es geclippt.
}
Procedure HC_PutSpriteBlock(x, y: Longint; SourceSprite: HC_Sprite);
{
  Wie HC_PUTSPRITE, kopiert aber DWords und bercksichtig keine
  Transparenten Stellen. (--> Schneller!)
}
Procedure HC_PutSpriteShadow(x, y: Longint; SourceSprite: HC_Sprite);
{
  Wie HC_PUTSPRITE, benutzt aber die Schattenpalette, um den Effekt
  zu erzielen, da das Sprite auf dem Hintergrund aussieht wie sein
  eigener Schatten.
}
Function HC_SpriteOverlapBlock(x1, y1: Longint; Sprite1: HC_Sprite; x2, y2: Longint; Sprite2: HC_Sprite): Boolean;
{
 Gibt TRUE zurck, falls sich Sprite1 und Sprite2 in ihren Koordinaten
 berschneiden, ansonsten FALSE.
}
Function HC_SpriteOverlap(x1, y1: Longint; Sprite1: HC_Sprite; x2, y2: Longint; Sprite2: HC_Sprite): Boolean;
{
 Wie HC_SPRITEOVERLAPBLOCK, aber auf Pixelebene.
}

Const
   { Fehler-Konstanten }
   HC_Error: Integer = 0;
   HC_NoError = 0;
   HC_DirectXError = 1;
   HC_OutOfMemError = 2;
   HC_IOError = 3;
   HC_WrongFormatError = 4;
   HC_AnyError = 5;

Var
   HC_DDrawObj: IDirectDraw4 = Nil;      { DirectDraw-Objekt }
   HC_Primary: IDirectDrawSurface4 = Nil;{ Primrer-Puffer (das, was man sieht) }
   HC_Back: IDirectDrawSurface4 = Nil;   { Offscreen-Puffer (darin wird gezeichnet) }
   HC_BufPtr: Pointer;                   { Zeiger auf den Offscreen-Puffer (fr direktes Ansprechen) }
   HC_Initialized: Boolean;              { True, wenn HC_Init erfolgreich }
   HC_Width, HC_Height: Longint;         { Hhe und Breite des Buffers }
   HC_Buffer: Array[0..256*256] Of Byte; { Allzweck-Buffer }
   HC_x1, HC_y1, HC_x2, HC_y2: Longint;  { Rechteck des Grafikfensters }
   HC_WinWidth, HC_WinHeight: Longint;   { Ausmae des Grafikfensters }
   HC_ShadowPal: ^HC_ShadowPalType;      { Zeiger auf die Schattenpalette }
   HC_Stride: DWord;                     { Anzahl Bytes pro Scanline }
   HC_WinHandle: THandle;                { Fensterhandle, wird fr SETCOOPERATIVELEVEL bentigt }

   // Variablen zum Handling der verschiedenen mglichen Pixelformate
   HC_RedBits, HC_GreenBits, HC_BlueBits: Integer;
   HC_RedBitLo, HC_RedBitHi: Integer;
   HC_GreenBitLo, HC_GreenBitHi: Integer;
   HC_BlueBitLo, HC_BlueBitHi: Integer;
   HC_RedBitMask, HC_GreenBitMask, HC_BlueBitMask: Cardinal;

Implementation

Type
    TPCXHeader = Record
         Manufacturer: byte;     { Always 10 for PCX file }

         Version: byte;          { 2 - old PCX - no palette (not used anymore),
                                   3 - no palette,
                                   4 - Microsoft Windows - no palette (only in
                                       old files, new Windows version uses 3),
                                   5 - with palette }

         Encoding: byte;         { 1 is PCX, it is possible that we may add
                                   additional encoding methods in the future }

         Bits_per_pixel: byte;   { Number of bits to represent a pixel
                                   (per plane) - 1, 2, 4, or 8 }

         Xmin: Word;          { Image window dimensions (inclusive) }
         Ymin: Word;          { Xmin, Ymin are usually zero (not always) }
         Xmax: Word;
         Ymax: Word;

         Hdpi: Word;          { Resolution of image (dots per inch) }
         Vdpi: Word;          { Set to scanner resolution - 300 is default }

         ColorMap: Array [0..15, 0..2] Of byte;
                                 { RGB palette data (16 colors or less)
                                   256 color palette is appended to end of file }

         Reserved: byte;         { (used to contain video mode)
                                   now it is ignored - just set to zero }

         Nplanes: byte;          { Number of planes }

         Bytes_per_line_per_plane: Word;   { Number of bytes to allocate
                                                for a scanline plane.
                                                MUST be an an EVEN number!
                                                Do NOT calculate from Xmax-Xmin! }

         PaletteInfo: Word;   { 1 = black & white or color image,
                                   2 = grayscale image - ignored in PB4, PB4+
                                   palette must also be set to shades of gray! }

         HscreenSize: Word;   { added for PC Paintbrush IV Plus ver 1.0,  }
         VscreenSize: Word;   { PC Paintbrush IV ver 1.02 (and later)     }
                                 { I know it is tempting to use these fields
                                   to determine what video mode should be used
                                   to display the image - but it is NOT
                                   recommended since the fields will probably
                                   just contain garbage. It is better to have
                                   the user install for the graphics mode he
                                   wants to use... }

         Filler: Array [74..127] Of byte;     { Just set to zeros }
    End;

    TBJSHeader = Packed Record
       id: Array[0..2] Of Char;
       Version: Byte;
       Width, Height: DWord;
       CutTop, CutLeft, CutBottom, CutRight: DWord;
       Encoding: DWord;
       Flags: Dword;
       Filler: Array[0..27] Of Byte;
       Palette: Array[0..767] Of Byte;
    End;

Function HC_Result: Integer;
Begin
     Result:=HC_Error;
     HC_Error:=HC_NoError;
End;

Procedure Swap(Var v1, v2: Longint);
Var
   Dummy: Longint;
Begin
     Dummy:=v1;
     v1:=v2;
     v2:=Dummy;
End;

Procedure HC_SetWindow;
Var
   Width, Height: Longint;
Begin
     If (x1 > x2) Then Swap(x1, x2);
     If (y1 > y2) Then Swap(y1, y2);
     Width:=Abs(x1-x2);
     Height:=Abs(y1-y2);
     If (Width <= 1) Or (Height <= 1) Then Begin
        HC_Error:=HC_AnyError;
        Exit;
     End;
     If (x1 < 0) Or (x1 >= HC_Width) Or (x2 < 0) Or (x2 >= HC_Width)
        Or (y1 < 0) Or (y1 >= HC_Height) Or (y2 < 0) Or (y2 >= HC_Height)
        Then Begin
        HC_Error:=HC_AnyError;
        Exit;
     End;
     HC_WinWidth:=Width;
     HC_WinHeight:=Height;
     HC_x1:=x1;
     HC_y1:=y1;
     HC_x2:=x2;
     HC_y2:=y2;
End;

Procedure HC_GetWindow;
Begin
     x1:=HC_x1;
     y1:=HC_y1;
     x2:=HC_x2;
     y2:=HC_y2;
End;

Procedure HC_ExtractRGBBitCount;
{Var
   pf: TDDPixelFormat;}

Function GetLoBit(mask: Cardinal): Integer;
Var
   BitPos: Integer;
   testmask: Cardinal;
Begin
     If mask <> 0 Then Begin
        BitPos:=0;
        testmask:=1;
        While (testmask And mask) = 0 Do Begin
              Inc(BitPos);
              testmask:=Cardinal(testmask Shl 1);
        End;
        Result:=BitPos;
     End Else Result:=-1;
End;

Function GetHiBit(mask: Cardinal): Integer;
Var
   BitPos: Integer;
   testmask: Cardinal;
Begin
     If mask <> 0 Then Begin
        BitPos:=31;
        testmask:=Cardinal(1 Shl 31);
        While (testmask And mask) = 0 Do Begin
              Dec(BitPos);
              testmask:=Cardinal(testmask Shr 1);
        End;
        Result:=BitPos;
     End Else Result:=-1;
End;

Begin
{     FillChar(pf, SizeOf(pf), #0);
     pf.dwSize:=SizeOf(pf);
}
{     If Failed(surface.GetPixelFormat(pf)) Then Begin
        HC_Error:=HC_DirectXError;
        Exit;
     End;}

     HC_RedBitMask:=pf.dwRBitMask;
     HC_GreenBitMask:=pf.dwGBitMask;
     HC_BlueBitMask:=pf.dwBBitMask;

     HC_RedBitLo:=GetLoBit(pf.dwRBitMask);
     HC_RedBitHi:=GethiBit(pf.dwRBitMask);
     HC_RedBits:=(HC_RedBitHi - HC_RedBitLo + 1);

     HC_GreenBitLo:=GetLoBit(pf.dwGBitMask);
     HC_GreenBitHi:=GethiBit(pf.dwGBitMask);
     HC_GreenBits:=(HC_GreenBitHi - HC_GreenBitLo + 1);

     HC_BlueBitLo:=GetLoBit(pf.dwBBitMask);
     HC_BlueBitHi:=GethiBit(pf.dwBBitMask);
     HC_BlueBits:=(HC_BlueBitHi - HC_BlueBitLo + 1);
End;

Procedure HC_Init(Width, Height: Word; Hndl: HDC);
Var
   Ret: Integer;
   ErrTxt: String;
   SD: TDDSurfaceDesc2;
   Caps: TDDSCaps2;
   DD1: IDirectDraw;
   pf: TDDPixelFormat;

Begin

     // DirectX initialisieren

     If Assigned(HC_DDrawObj) Then Exit;
     // DirectDraw-Objekt erzeugen
     If Succeeded(DirectDrawCreate(Nil, DD1, Nil)) Then Begin
        DD1.QueryInterface(IID_IDirectDraw4, HC_DDrawObj);
        // CooperativeLevel setzen
        HC_WinHandle:=Hndl;
        Ret:=HC_DDrawObj.SetCooperativeLevel(Hndl,
             DDSCL_FULLSCREEN + DDSCL_ALLOWREBOOT +
             DDSCL_ALLOWMODEX + DDSCL_EXCLUSIVE );
        If Ret <> DD_OK Then Begin
           Case Ret Of
              DDERR_EXCLUSIVEMODEALREADYSET: ErrTxt := 'Exclude mode already set';
              DDERR_HWNDALREADYSET:          ErrTxt := 'HWND already set';
              DDERR_HWNDSUBCLASSED:          ErrTxt := 'Hwnd Subclasses';
              DDERR_INVALIDOBJECT:           ErrTxt := 'Invalid Object';
              DDERR_INVALIDPARAMS:           ErrTxt := 'Invalid Params';
              DDERR_OUTOFMEMORY:             ErrTxt := 'Out of memory';
           Else
              ErrTxt := 'Unknown error';
           End;
           MessageDlg('SetCooperativeLevel failed: '+ErrTxt+'.',mtError,[mbOK],0);
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;
        // Bildschirmmodus umschalten
        If HC_DDrawObj.SetDisplayMode(Width, Height, 16, 0, 0) <> DD_OK Then Begin
           MessageDlg('Set Display Mode Failed',mtError,[mbOK],0);
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;
        // Primren Buffer allokieren
        FillChar(SD, SizeOf(SD), 0);
        With SD Do Begin
             SD.dwSize:=SizeOf(SD);
             SD.dwBackBufferCount:=1;
             SD.dwFlags:=DDSD_CAPS Or DDSD_BACKBUFFERCOUNT;
             SD.ddscaps.dwCaps:=DDSCAPS_PRIMARYSURFACE+DDSCAPS_FLIP+DDSCAPS_COMPLEX;
        End;
        If (Not Succeeded(HC_DDrawObj.CreateSurface(SD, HC_Primary, Nil))) Then Begin
           MessageDlg('Failed to create primary surface.',mtError,[mbOK],0);
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;
        // Sekundren Buffer allokieren
        Caps.dwCaps:=DDSCAPS_BACKBUFFER;
        If (Not Succeeded(HC_Primary.GetAttachedSurface(Caps, HC_Back))) Then Begin
           MessageDlg('Failed to create back-buffer.',mtError,[mbOK],0);
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;
{
        FillChar(SD, SizeOf(SD), 0);
        With SD Do Begin
             SD.dwSize:=SizeOf(SD);
             SD.dwFlags:=DDSD_CAPS;
             SD.ddscaps.dwCaps:=DDSCAPS_PRIMARYSURFACE;
        End;
        If (Not Succeeded(HC_DDrawObj.CreateSurface(SD, HC_Primary, Nil))) Then Begin
           MessageDlg('Failed to create primary surface.',mtError,[mbOK],0);
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;
        FillChar(SD, SizeOf(SD), 0);
        With SD Do Begin
             SD.dwSize:=SizeOf(SD);
             SD.dwFlags:=DDSD_CAPS+DDSD_WIDTH+DDSD_HEIGHT;
             SD.dwWidth:=Width;
             SD.dwHeight:=Height;
             SD.ddscaps.dwCaps:=DDSCAPS_OFFSCREENPLAIN+DDSCAPS_SYSTEMMEMORY;
        End;
        If (Not Succeeded(HC_DDrawObj.CreateSurface(SD, HC_Back, Nil))) Then Begin
           MessageDlg('Failed to create back-buffer.',mtError,[mbOK],0);
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;}

        HC_BufPtr:=Nil;
        If Failed(HC_Primary.GetPixelFormat(pf)) Then Begin
           HC_Error:=HC_DirectXError;
           HC_Done;
           Exit;
        End;
        HC_ExtractRGBBitCount(pf);

     End Else Begin
        MessageDlg('Failed to create DirectDraw object.',mtError,[mbOK],0);
        HC_Error:=HC_DirectXError;
        Exit;
     End;

     // Alles hat geklappt
     HC_Width:=Width;
     HC_Height:=Height;
     HC_Initialized:=True;
     HC_SetWindow(0,0,Width-1,Height-1);
End;

Procedure HC_Done;
Begin
     If HC_Initialized Then Begin

        // Resourcen wieder freiegeben
        If Assigned(HC_Back) Then HC_Back:=Nil;
        If Assigned(HC_Primary) Then HC_Primary:=Nil;
        If Assigned(HC_DDrawObj) Then Begin
           HC_DDrawObj.FlipToGDISurface;
           HC_DDrawObj.RestoreDisplayMode;
           HC_DDrawObj.SetCooperativeLevel(HC_WinHandle, DDSCL_NORMAL);
           HC_DDrawObj:=Nil;
        End;

        // Variablen wieder auf Anfangswerte setzen
        HC_Width:=0;
        HC_Height:=0;
        HC_BufPtr:=Nil;
        HC_FreeShadowPal;

        HC_Initialized:=False;

     End;
End;

Procedure HC_RestoreSurfaces;
Begin
     If Assigned(HC_Primary) Then
        If Bool(HC_Primary.IsLost) Then HC_Primary._Restore;
     If Assigned(HC_Back) Then
        If Bool(HC_Back.IsLost) Then HC_Back._Restore;
End;

Procedure HC_UnLock;
Begin
     HC_RestoreSurfaces;
     If Assigned(HC_Back) And (HC_BufPtr <> Nil) Then Begin
        If (Not Succeeded(HC_Back.UnLock(HC_BufPtr))) Then
           HC_Error:=HC_DirectXError;
        HC_BufPtr:=Nil;
     End;
End;

Function HC_Lock;
Var
   SD: TDDSurfaceDesc2;
Begin
     HC_UnLock;
     If Assigned(HC_Back) Then Begin
        FillChar(SD,SizeOf(SD),0);
        SD.dwSize:=SizeOf(SD);
        If (Not Succeeded(HC_Back.Lock(Nil, SD, DDLOCK_WAIT, 0))) Then
           HC_Error:=HC_DirectXError
        Else Begin
           HC_BufPtr:=SD.lpSurface;
           HC_Stride:=SD.lPitch;
        End;
        Result:=HC_BufPtr;
     End Else Result:=Nil;
End;

Procedure HC_ShowBuf;
Begin
     If Assigned(HC_Primary) Then Begin
        HC_UnLock;
        If (Not Succeeded(HC_Primary.Flip(Nil, DDFLIP_WAIT))) Then
           HC_Error:=HC_DirectXError;
//        HC_Primary.Blt(Nil, HC_Back, Nil, DDBLT_WAIT, Nil);
     End;
End;

Procedure HC_ClearBuf;
Var
   y: DWord;
Begin
     Asm
        pushad

        // Zieladresse laden
        mov     edi, HC_BufPtr

        // Anzahl der verbleibenden Bytes pro Scanline berechnen und in ESI speichern
        mov     esi, HC_Stride
        mov     eax, HC_Width
        shl     eax, 1
        sub     esi, eax

        // Anzahl der Scanlines nach EBX
        mov     eax, HC_Height
        mov     y, eax

     @YLoop:
        // Anzahl der DWords pro Scanline berechnen und
        // Anzahl DWords in ECX, EDX ist 1, wenn die Anzahl der Pixel ungerade ist
        xor     edx, edx
        mov     eax, HC_Width
        mov     ebx, 2
        div     ebx
        mov     ecx, eax

        // COL zweimal nach EAX laden
        xor     eax, eax
        mov     ax, Col
        shl     eax, 16
        mov     ax, Col

        // "Entrollte" Schleife, um noch ein bischen Geschwindigkeit rauszuquetschen
        // Ginge natrlich wesentlich eleganter, hab aber keine Lust
     @DWordLoop:
        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        je      @LastWord

        mov     [edi], eax
        add     edi, 4
        dec     ecx
        jne     @DWordLoop

        or      edx, edx
        je      @EvenScanLine

     @LastWord:
        mov     [edi], ax
        add     edi, 2

     @EvenScanLine:
        add     edi, esi
        dec     y
        jne     @YLoop

     @End:
        popad
     End;
End;

{Function HC_EncodeColor;
Begin
Asm
   pushad
   xor    ax,ax
   mov    al,b
   xor    bx,bx
   mov    bl,g
   shl    bx,5
   or     ax,bx
   xor    bx,bx
   mov    bl,r
   shl    bx,10
   or     ax,bx
   mov    Result,ax
   popad
End;
End;
}
Procedure HC_DecodeColor;
Begin
Asm
   pushad
   mov    dx,Col
   mov    ax,dx
   and    al,00011111b
   mov    ebx,b
   mov    [ebx],al
   and    ah,01111100b
   shr    ah,2
   mov    ebx,r
   mov    [ebx],ah
   and    dx,0000001111100000b
   shr    dx,5
   mov    ebx,g
   mov    [ebx],dl
   popad
End;
End;

Function HC_EncodeColor;
Begin
     If HC_RedBits <> 5 Then Begin
        If HC_RedBits > 5 Then
           r:=Byte(r Shl (HC_RedBits - 5))
        Else If HC_RedBits < 5 Then
           r:=Byte(r Shr (5 - HC_RedBits));
     End;

     If HC_GreenBits <> 5 Then Begin
        If HC_GreenBits > 5 Then
           g:=Byte(g Shl (HC_GreenBits - 5))
        Else If HC_GreenBits < 5 Then
           g:=Byte(g Shr (5 - HC_GreenBits));
     End;

     If HC_BlueBits <> 5 Then Begin
        If HC_BlueBits > 5 Then
           b:=Byte(b Shl (HC_BlueBits - 5))
        Else If HC_BlueBits < 5 Then
           b:=Byte(b Shr (5 - HC_BlueBits));
     End;

     Result:=Word(
                   Word(r Shl HC_RedBitLo) Or
                   Word(g Shl HC_GreenBitLo) Or
                   Word(b Shl HC_BlueBitLo)
                  );
End;

{Procedure HC_DecodeColor;
Begin
     r := Byte( (Cardinal(Col) And HC_RedBitMask) Shr HC_RedBitLo );
     g := Byte( (Cardinal(Col) And HC_GreenBitMask) Shr HC_GreenBitLo );
     b := Byte( (Cardinal(Col) And HC_BlueBitMask) Shr HC_BlueBitLo );

     If HC_RedBits <> 5 Then Begin
        If HC_RedBits > 5 Then
           r:=Byte(r Shl (HC_RedBits - 5))
        Else If HC_RedBits < 5 Then
           r:=Byte(r Shr (5 - HC_RedBits));
     End;

     If HC_GreenBits <> 5 Then Begin
        If HC_GreenBits > 5 Then
           g:=Byte(g Shl (HC_GreenBits - 5))
        Else If HC_GreenBits < 5 Then
           g:=Byte(g Shr (5 - HC_GreenBits));
     End;

     If HC_BlueBits <> 5 Then Begin
        If HC_BlueBits > 5 Then
           b:=Byte(b Shl (HC_BlueBits - 5))
        Else If HC_BlueBits < 5 Then
           b:=Byte(b Shr (5 - HC_BlueBits));
     End;
End;}

Procedure HC_PutPixel;
Begin
Asm
   pushad
   // Linke obere Ecke des Grafikfenster dazu addieren
   mov   eax,HC_x1
   add   x,eax
   mov   eax,HC_y1
   add   y,eax
   // Adresse berechnen
   mov   edi,HC_BufPtr
   mov   edx,0
   mov   eax, HC_Stride
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   mov   ax,Col
   // Pixel setzen
   mov   [edi],ax
   popad
End;
End;

Procedure HC_PutPixelSave;
Begin
Asm
   pushad
   // Linke obere Ecke des Grafikfenster dazu addieren
   mov   eax,HC_x1
   add   x,eax
   mov   eax,HC_y1
   add   y,eax
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,HC_x1
   jl    @End
   cmp   eax,HC_x2
   jg    @End
   mov   eax,y
   cmp   eax,HC_y1
   jl    @End
   cmp   eax,HC_y2
   jg    @End
   // Adresse berechnen
   mov   edi,HC_BufPtr
   mov   edx,0
   mov   eax, HC_Stride
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   mov   ax,Col
   // Pixel setzen
   mov   [edi],ax
@End:
   popad
End;
End;

Function HC_GetPixel;
Begin
Asm
   pushad
   // Linke obere Ecke des Grafikfenster dazu addieren
   mov   eax,HC_x1
   add   x,eax
   mov   eax,HC_y1
   add   y,eax
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,HC_x1
   jl    @Invalid
   cmp   eax,HC_x2
   jg    @Invalid
   mov   eax,y
   cmp   eax,HC_y1
   jl    @Invalid
   cmp   eax,HC_y2
   jg    @Invalid
   // Adresse berechnen
   mov   edi,HC_BufPtr
   mov   edx,0
   mov   eax, HC_Stride
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   // Pixel holen
   mov   ax,[edi]
   jmp   @End
@Invalid:
   mov   ax,0
@End:
   mov   Result,ax
   popad
End;
End;

Procedure HC_PutPixelSprite;
Begin
Asm
   pushad
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr DestSprite.Width
   jge   @End
   mov   eax,y
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr DestSprite.Height
   jge   @End
   // Adresse berechnen
   mov   edi,DWord Ptr DestSprite.Addr
   mov   edx,0
   mov   eax,Dword Ptr DestSprite.Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   mov   ax,Col
   // Pixel setzen
   mov   [edi],ax
@End:
   popad
End;
End;

Function HC_GetPixelSprite;
Begin
Asm
   pushad
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr SourceSprite.Width
   jge   @End
   mov   eax,y
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr SourceSprite.Height
   jge   @End
   // Adresse berechnen
   mov   edi,DWord Ptr SourceSprite.Addr
   mov   edx,0
   mov   eax,Dword Ptr SourceSprite.Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   // Pixel holen
   mov   ax,[edi]
   jmp   @End
@Invalid:
   mov   ax,0
@End:
   mov   Result,ax
   popad
End;
End;

Procedure HC_EllipseAndCircle(xm, ym: Longint; xr,  yr: Longint; DisplayColor: Word);
Var
   x, y: Longint;
   dx, dy,
   ySub, xAdd,
   r: Longint;
Begin
     If (xr = 0) Or (yr = 0) Then Exit;
     dx:=Longint(xr * 2) * Longint(xr);
     dy:=Longint(yr * 2) * Longint(yr);
     ySub:=0;
     xAdd:=Longint(xr) * dy;
     r:=xAdd Div 2;
     x:=xr;
     y:=0;
     Repeat
           HC_PutPixelSave(xm+x, ym+y, DisplayColor);
           HC_PutPixelSave(xm+x, ym-y, DisplayColor);
           HC_PutPixelSave(xm-x, ym-y, DisplayColor);
           HC_PutPixelSave(xm-x, ym+y, DisplayColor);
           If r >= 0 Then
           Begin
                Inc(y);
                Inc(ySub, dx);
                Dec(r, ySub);
           End;
           If r < 0 Then
           Begin
                Dec(x);
                Dec(xAdd, dy);
                Inc(r, xAdd);
           End;
     Until x < 0;
End;

Procedure HC_Circle;
Begin
     HC_EllipseAndCircle(x, y, r, r, Col);
End;

Procedure HC_Ellipse;
Begin
     HC_EllipseAndCircle(x, y, rx, ry, Col);
End;

Procedure HC_Line(x1, y1, x2, y2: Longint; Col: Word);
Var
   x, y, d_x, d_y, e_x, e_y, yadd, yunit: Longint;
Begin
Asm
   pushad
   mov   eax,x1                   // Linke obere Ecke des Fensters addieren
   add   eax,HC_x1
   mov   x1,eax
   mov   eax,y1
   add   eax,HC_y1
   mov   y1,eax
   mov   eax,x2
   add   eax,HC_x1
   mov   x2,eax
   mov   eax,y2
   add   eax,HC_y1
   mov   y2,eax
   mov   eax,x1
   cmp   eax,x2                   // x1 > x2 ?
   jle   @Nicht_Swappen
   xchg  eax,x2                   // Ja, x1 und x2 vertauschen
   mov   x1,eax
   mov   eax,y1
   xchg  eax,y2
   mov   y1,eax
@Nicht_Swappen:                   // Nein. Nicht verstauschen.
   mov   eax,x1
   sub   eax,x2
   mov   e_x,eax                  // e_x := x1 - x2
   mov   eax,y1
   sub   eax,y2
   cmp   eax,0
   jle   @Kein_Neg
   neg   eax
@Kein_Neg:
   mov   e_y,eax                  // e_y := -abs(y2-y1)
   mov   eax,e_x
   neg   eax
   shl   eax,1
   mov   d_x,eax                  // d_x := -(e_x) Shl 1
   mov   eax,e_y
   neg   eax
   shl   eax,1
   mov   d_y,eax                  // d_y := -(e_y) Shl 1
   mov   eax,y1
   cmp   eax,y2                   // y1 > y2 ?
   jle   @Y_Positiv
   mov   eax,0                   // Nein, von unten nach oben
   mov   eax,HC_Stride
   neg   eax
   mov   yadd,eax                // yadd := -HC__Width
   mov   yunit,-1                // yunit := -1
   jmp   @Weiter1
@Y_Positiv:                       // Ja, von oben nach unten
   mov   eax,0
   mov   eax,HC_Stride
   mov   yadd,eax                // yadd := HC__Width
   mov   yunit,1                 // yunit := 1
@Weiter1:
   mov   eax,0
   mov   ebx,0
   mov   edx,0
   mov   eax, HC_Stride
   mul   y1
   mov   ebx,x1
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_BufPtr           // eax := eax + Video-Puffer-Adresse
   mov   edi,eax                 // Start-Offset nach EDI laden
   mov   eax,x1
   mov   x,eax                    // x := x1
   mov   eax,y1
   mov   y,eax                    // y := y1
   mov   eax,d_x
   cmp   eax,d_y                  // d_x < d_y
   jl    @Y_Schleife_Anfang     // ja, dann Y-Schleife anspringen
   mov   ecx,x2                  // Nein, dann mit X-Schleife weitermachen
   sub   ecx,x1                  // ecx := x2 - x1
   cmp   ecx,0                   // Falls ecx negativ ist...
   jge   @Weiter3
   neg   ecx                     // ...Vorzeichen umkehren
@Weiter3:
   or    ecx,ecx                 // Wenn ecx Null ist...
   jne    @X_Schleife
   inc   ecx                     // ecx um 1 erhhen, damit der Rechner nicht abschmiert
@X_Schleife:
   mov   eax,HC_x2               //-------------------------------------
   cmp   x,eax                    //
   jg    @Nicht_Zeichnen1       // Prfen, ob sich x/y im Bildschirm
   mov   eax,HC_y2               // befindet. Wenn nicht, zum Label
   cmp   y,eax                    // @Nicht_Zeichnen1 springen
   jg    @Nicht_Zeichnen1       //--------------------------------------
   mov   eax,HC_x1
   cmp   x,eax
   jl    @Nicht_Zeichnen1
   mov   eax,HC_y1
   cmp   y,eax
   jl    @Nicht_Zeichnen1
   mov   ax,Col
   mov   [edi],ax             // Pixel nach [EDI] schreiben
@Nicht_Zeichnen1:
   mov   eax,d_y
   add   e_x,eax                  // Inc(e_x, d_y)
   cmp   e_x,0                   // e_x > 0 ?
   jle   @Y_Nicht_weiter        // Nein, dann diesen Teil berspringen
   mov   eax,d_x
   sub   e_x,eax                  // Dec(e_x, d_x)
   mov   eax,yunit
   add   y,eax                    // In nchste Zeile springen
   add   edi,yadd
@Y_Nicht_Weiter:
   inc   x                       // In nchste Spalte springen
   add   edi,2
   dec   ecx                      // Zhler dekrementieren
   jns    @X_Schleife           // Bis ecx = 0
   jmp   @Ende                  // Fertig, ans Ende springen
@Y_Schleife_Anfang:
   mov   ecx,y1
   sub   ecx,y2                   // ecx := y1 - y2
   cmp   ecx,0                    // Falls ecx negativ ist...
   jg    @Y_Schleife
   neg   ecx                      // ...Vorzeichen umkehren
@Y_Schleife:
   mov   eax,HC_x2               //---------------------------------------
   cmp   x,eax                    //
   jg    @Nicht_Zeichnen2       // Prfen, ob x/y im Bildschirm ist
   mov   eax,HC_y2               //
   cmp   y,eax                    //
   jg    @Nicht_Zeichnen2       //---------------------------------------
   mov   eax,HC_x1
   cmp   x,eax
   jl    @Nicht_Zeichnen2
   mov   eax,HC_y1
   cmp   y,eax
   jl    @Nicht_Zeichnen2
   mov   ax,Col
   mov   [edi],ax             // Pixel nach [EDI] schreiben
@Nicht_Zeichnen2:
   mov   eax,d_x
   add   e_y,eax                  // Inc(e_y, d_x)
   cmp   e_y,0                   // e_y > 0 ?
   jl    @X_Nicht_Weiter        // Nein, dann diesen Teil berspringen
   mov   eax,d_y
   sub   e_y,eax                  // Dec(e_y, d_y)
   inc   x
   add   edi,2                     // In nchste Spalte springen
@X_Nicht_Weiter:
   mov   eax,yunit
   add   y,eax                    // In nchste Zeile springen
   add   edi,yadd
   dec   ecx                      // Zhler dekrementieren
   jns   @Y_Schleife            // Bis ecx = 0
@Ende:
   popad
End;
End;

Procedure HC_Rect;
Begin
     HC_Line(x1,y1,x2,y1,Col);
     HC_Line(x1,y1,x1,y2,Col);
     HC_Line(x1,y2,x2,y2,Col);
     HC_Line(x2,y1,x2,y2,Col);
End;

Procedure HC_FillRect;
Var
   Skip: Longint;
Begin
Asm
   pushad
   // Obere linke Ecke des Grafik-Fensters dazu addieren
   mov   eax,HC_x1
   add   x1,eax
   add   x2,eax
   mov   eax,HC_y1
   add   y1,eax
   add   y2,eax
   // ggf. Koordinaten vertauschen, damit wir immer von links nach
   // rechts und von oben nach unten zeichnen knnen
   mov   eax,x1
   cmp   eax,x2
   jle   @NoXSwap
   mov   ebx,x2
   xchg  eax,ebx
   mov   x1,eax
   mov   x2,ebx
@NoXSwap:
   mov   eax,y1
   cmp   eax,y2
   jle   @NoYSwap
   mov   ebx,y2
   xchg  eax,ebx
   mov   y1,eax
   mov   y2,ebx
@NoYSwap:
   // Prfen, ob das Rechteck berhaupt im sichtbaren Bereich liegt
   mov   eax,x1
   cmp   eax,HC_x2
   jg    @End
   mov   eax,x2
   cmp   eax,HC_x1
   jl    @End
   mov   eax,y1
   cmp   eax,HC_y2
   jg    @End
   mov   eax,y2
   cmp   eax,HC_y1
   jl    @End
   // ggf. Clipping durchfhren
   mov   eax,x1
   cmp   eax,HC_x1
   jge   @NoX1Clip
   mov   eax,HC_x1
   mov   x1,eax
@NoX1Clip:
   mov   eax,x2
   cmp   eax,HC_x2
   jle   @NoX2Clip
   mov   eax,HC_x2
   mov   x2,eax
@NoX2Clip:
   mov   eax,y1
   cmp   eax,HC_y1
   jge   @NoY1Clip
   mov   eax,HC_y1
   mov   y1,eax
@NoY1Clip:
   mov   eax,y2
   cmp   eax,HC_y2
   jle   @NoY2Clip
   mov   eax,HC_y2
   mov   y2,eax
@NoY2Clip:
   // Startadresse berechnen
   mov   eax,HC_Stride
   mul   y1
   mov   ebx,x1
   shl   ebx,1
   add   eax,ebx
   mov   edi,HC_BufPtr
   add   edi,eax
   // Breite des zu zeichnenden Bereichs berechnen (in Bytes) und in EBX merken
   mov   ebx,x2
   sub   ebx,x1
   inc   ebx
   shl   ebx,1
   // Berechnen, wieviel am Ende jeder Zeile dazu addiert werden mu
   mov   eax,HC_Stride
   sub   eax,ebx
   mov   Skip,eax
   // In DWords und Words zerlegen und in EBX und EDX merken
   mov   eax,ebx
   mov   ecx,2
   mov   edx,0   // EDX lschen, damit kein Mll drin zurckbleibt
   div   ecx     // der Rest wird automatisch in EDX abgelegt
   mov   ebx,eax
   // Hhe des zu zeichnenden Bereichs berechnen und in ESI merken
   mov   eax,y2
   sub   eax,y1
   mov   esi,eax
   inc   esi
   // Jetzt kommt die Arbeit - den ganzen Kse mit COL ausfllen...
@YLoop:
   mov   ax,Col
   shl   eax,16
   mov   ax,Col
   mov   ecx,ebx
   or    ecx,ecx
   je    @WordCopy
@DWordLoop:
   mov   [edi],eax
   add   edi,4
   dec   ecx
   jne   @DWordLoop
@WordCopy:
   or    edx,edx
   je    @NoWord
   mov   [edi],ax
   add   edi,2
@NoWord:
   add   edi,Skip
   dec   esi
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_ClearWindow(Col: Word);
Begin
     HC_FillRect(0,0,HC_Width,HC_Height,Col);
End;

Procedure HC_CreateSprite;
Var
   Size: Longint;
Begin
     Size:=Width*Height*2;
     If Size <= 0 Then Begin
        HC_Error:=HC_WrongFormatError;
        Exit;
     End;
     GetMem(DestSprite.Addr,Size);
     DestSprite.Width:=Width;
     DestSprite.Height:=Height;
     FillChar(DestSprite.Addr^,Size,0);
End;

Procedure HC_LoadPCX8Bit;
Const
   Compress = $C0;
   DeCompress = $3F;
Var
   Palette: Array[0..767] Of Byte;
   Colors: Array[0..255] Of Word;
   Header: TPCXHeader;
   f: File;
   _Read: Integer;
   Width, Height: Integer;
   Size: Longint;
   i, OldPos: Longint;
   BufSize: Longint;
   Data, Counter: Byte;
   Offs: Longint;
   SourceOffs: Longint;
   ColAdr: Longint;
   ScanLineCount: Longint;

Function ReadNextByte: Byte;
Begin
     {$I-}
     If BufSize <= 0 Then Begin
        BlockRead(f,HC_Buffer,SizeOf(HC_Buffer),_Read);
        BufSize:=_Read;
        If (BufSize = 0) Or (IOResult <> 0) Then Begin
           HC_Error:=HC_IOError;
           ReadNextByte:=0;
           Exit;
        End Else Begin
           SourceOffs:=0;
           Result:=ReadNextByte;
        End;
     End Else Begin
        Result:=HC_Buffer[SourceOffs];
        Dec(BufSize);
        Inc(SourceOffs);
     End;
     {$I+}
End;

Begin
     {$I-}

     // Datei ffnen, abbrechen falls Fehler
     IOResult;
     AssignFile(f, FileName);
     Reset(f, 1);
     If IOResult <> 0 Then Begin
        HC_Error:=HC_IOError;
        Exit;
     End;

     // Header einlesen, abberechen falls Fehler
     BlockRead(f, Header, SizeOf(Header), _Read);
     If (_Read <> SizeOf(Header)) Or (IOResult <> 0) Then Begin
        CloseFile(f);
        IOResult;
        HC_Error:=HC_IOError;
        Exit;
     End;

     // Speicher berechnen und allokieren, abbrechen falls Fehler
     With Header Do Begin
          Width:=Abs(XMax-XMin)+1;
          Height:=Abs(YMax-YMin)+1;
          If (Bits_Per_Pixel <> 8) Or (NPLanes <> 1) Or (Width = 0)
             Or (Height = 0) Then Begin
             CloseFile(f);
             IOResult;
             HC_Error:=HC_WrongFormatError;
             Exit;
          End;
     End;
     Size:=Longint(Width)*Longint(Height);
     GetMem(DestSprite.Addr, Size*2);
     DestSprite.Width:=Width;
     DestSprite.Height:=Height;

     // Palette einlesen abbrechen, falls Fehler
     OldPos:=FilePos(f);
     Seek(f,FileSize(f)-SizeOf(Palette));
     BlockRead(f,Palette,SizeOf(Palette),_Read);
     If (IOResult <> 0) Or (_Read <> SizeOf(Palette)) Then Begin
        FreeMem(DestSprite.Addr,Size*2);
        CloseFile(f);
        IOResult;
        HC_Error:=HC_IOError;
        Exit;
     End;
     Seek(f,OldPos);

     // Palette in 15-Bit Farbwerte umrechnen
     For i:=0 To 255 Do Begin
         // Da die PCX-Palette als 8-Bit vorliegt, mssen die Farbwerte erst
         // 3 Bits nach rechts verschoben werden, da wir immer mit 5 Bit
         // Farbanteil rechnen
         Colors[i]:=HC_EncodeColor(Palette[i*3] Shr 3,
                                   Palette[i*3+1] Shr 3,
                                   Palette[i*3+2] Shr 3);
     End;
     // Variablen initialisieren
     i:=Size;
     BufSize:=0;
     Offs:=Longint(DestSprite.Addr);
     SourceOffs:=0;
     ColAdr:=Longint(@Colors);
     ScanLineCount:=DestSprite.Width;

     // Jetzt alle Bytes durchgehen und prfen, ob die beiden
     // obersten Bit gesetzt sind. Ist das der Fall, ist das Byte
     // ein Zhl-Byte
     Repeat
           Data:=ReadNextByte;
           If HC_Error = HC_IOError Then Begin
              FreeMem(DestSprite.Addr,Size*2);
              CloseFile(f);
              IOResult;
              Exit;
           End;
           If (Data And Compress) = Compress Then Begin
              Counter:=(Data And Decompress);
              Data:=ReadNextByte;
              If HC_Error = HC_IOError Then Begin
                 FreeMem(DestSprite.Addr,Size*2);
                 CloseFile(f);
                 IOResult;
                 Exit;
              End;
           End Else Counter:=1;
           Asm
              pushad
              // Farbwert aus Tabelle lesen
              mov     edi, ColAdr
              mov     eax, 0
              mov     al, Data
              shl     eax, 1
              add     edi, eax
              mov     ax, [edi]
              // Zieladresse und Zhler laden
              mov     edi, Offs
              mov     cl, Counter
              or      cl, cl
              je      @End
           @WriteLoop:
              // Eine Scanline mu ja immer "even" sein. Grummel, grummel!
              cmp     ScanLineCount, 0
              jne     @NoScanLineEnd
              mov     ebx, Width
              mov     ScanLineCount, ebx
              cmp     bx, Word Ptr Header.Bytes_Per_Line_Per_Plane
              jne     @OverridePixel
              // Pixel schreiben und Zieladdresse um 1 Word erhhen
           @NoScanLineEnd:
              mov     [edi], ax
              add     edi, 2
              add     Offs, 2
              dec     ScanLineCount
           @OverridePixel:
              dec     i
              dec     cl
              // Solange, bis CL = 0
              jne     @WriteLoop
           @End:
              popad
           End;
     Until i <= 0;

     // Ende - Datei schlieen
     CloseFile(f);
     IOResult;
     {$I+}

End;

Procedure HC_GetSprite;
Var
   Size, Skip: Longint;
   Adr: Pointer;
Begin
     If (x < 0) Or (x+Width > HC_Width) Or (y < 0) Or (y+Height > HC_Height) Then Begin
        HC_Error:=HC_WrongFormatError;
        Exit;
     End;
     Size:=Width*Height*2;
     If Size <= 0 Then Begin
        HC_Error:=HC_WrongFormatError;
        Exit;
     End;
     GetMem(DestSprite.Addr,Size);
     DestSprite.Width:=Width;
     DestSprite.Height:=Height;
     Skip:=Longint(HC_Stride) - (Width * 2);
     Adr:=DestSprite.Addr;
     Asm
        pushad
        mov    eax,HC_Stride
        mul    y
        mov    ebx,x
        shl    ebx,1
        add    eax,ebx
        mov    esi,HC_BufPtr
        add    esi,eax
        mov    edi,Adr
        mov    edx,Height
     @YLoop:
        mov    ecx,Width
     @XLoop:
        mov    ax,[esi]
        mov    [edi],ax
        add    esi,2
        add    edi,2
        dec    ecx
        jne    @XLoop
        add    esi,Skip
        dec    edx
        jne    @YLoop
        popad
     End;
End;

Procedure HC_FreeSprite;
Var
   Size: Longint;
Begin
     With SourceSprite Do Begin
          Size:=Width*Height*2;
          If Size > 0 Then Begin
             FreeMem(Addr,Size);
             Width:=0;
             Height:=0;
             Addr:=Nil;
          End;
     End;
End;

Procedure HC_MakeShadowPal;
Var
   i: Longint;
   r2, g2, b2: Byte;
Begin
     If HC_ShadowPal = Nil Then New(HC_ShadowPal);
     For i:=0 To 65535 Do Begin
         HC_DecodeColor(i, r2, g2, b2);
         HC_ShadowPal^[i]:=HC_EncodeColor(((r+r2) Div 2) * Intensity Div 100,
                                          ((g+g2) Div 2) * Intensity Div 100,
                                          ((b+b2) Div 2) * Intensity Div 100);
     End;
End;

procedure HC_FreeShadowPal;
Begin
     If HC_ShadowPal <> Nil Then Begin
        Dispose(HC_ShadowPal);
        HC_ShadowPal:=Nil;
     End;
End;

Procedure HC_PutSprite;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
Begin
Asm
   pushad
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax, HC_Stride
   mov   ebx, Dtx
   shl   ebx, 1
   sub   eax, ebx
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax, HC_Stride
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_BufPtr;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // Bitmap kopieren
   mov   edx,Dty
@YLoop:
   mov   ecx,Dtx
@XLoop:
   mov   ax,[esi]
   or    ax,ax
   je    @Null
   mov   [edi],ax
@Null:
   add   edi,2
   add   esi,2
   dec   ecx
   jne   @XLoop
   add   edi,SkipDest
   add   esi,SkipSource
   dec   edx
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_PutSpriteBlock;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
   DWords, Words: Longint;
Begin
Asm
   pushad
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax, HC_Stride
   mov   ebx, Dtx
   shl   ebx, 1
   sub   eax, ebx
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax, HC_Stride
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_BufPtr;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // In DWords und Words aufschlsseln
   xor   edx,edx
   mov   eax,Dtx
   shl   eax,1
   mov   ebx,4
   div   ebx
   mov   DWords,eax
   mov   Words,edx
   // Bitmap kopieren
   mov   edx,Dty
@YLoop:
   mov   ecx,DWords
   or    ecx,ecx
   je    @WordCopy
@DWordLoop:
   mov   eax,[esi]
   mov   [edi],eax
   add   edi,4
   add   esi,4
   dec   ecx
   jne   @DwordLoop
@WordCopy:
   cmp   Words,0
   je    @NoWords
   mov   ax,[esi]
   mov   [edi],ax
   add   esi,2
   add   edi,2
@NoWords:
   add   edi,SkipDest
   add   esi,SkipSource
   dec   edx
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_PutSpriteShadow;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
   YCount: Longint;
Begin
Asm
   pushad
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax, HC_Stride
   mov   ebx, Dtx
   shl   ebx, 1
   sub   eax, ebx
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax, HC_Stride
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_BufPtr;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // Bitmap kopieren
   mov   eax,Dty
   mov   YCount,eax
   mov   edx,HC_ShadowPal
@YLoop:
   mov   ecx,Dtx
   mov   eax,0
@XLoop:
   mov   ax,[esi]
   or    ax,ax
   je    @Null
   mov   ebx,edx
   mov   ax,[edi]
   shl   ax,1
   add   ebx,eax
   mov   ax,[ebx]
   mov   [edi],ax
@Null:
   add   edi,2
   add   esi,2
   dec   ecx
   jne   @XLoop
   add   edi,SkipDest
   add   esi,SkipSource
   dec   Ycount
   jne   @YLoop
@End:
   popad
End;
End;

Function HC_SpriteOverlapBlock;
Var
   x_overlap, y_overlap: boolean;
Begin
     If (x1 >= x2) And (x1 <= x2+sprite2.Width-1) Then x_overlap:=true Else
        If (x2 >= x1) And (x2 <= x1+sprite1.Width-1) Then x_overlap:=true Else
           x_overlap:=false;
     If (y1 >= y2) And (y1 <= y2+sprite2.Height-1) Then y_overlap:=true Else
        If (y2 >= y1) And (y2 <= y1+sprite1.Height-1) Then y_overlap:=true Else
           y_overlap:=false;
     Result:=x_overlap And y_overlap;
End;

Function HC_SpriteOverlap;
Var
   start_x_1, start_x_2, start_y_1, start_y_2: Longint;
   end_x_1, end_y_1: Longint;
   breite, hoehe: Longint;
   rest_x_1, rest_x_2: Longint;
Begin
     Result:=False;

     // Eigentlich berflssig, aber sonst hagelt's Warnungen...
     End_X_1:=0;
     End_Y_1:=0;

     { Haben wir berhaupt eine berschneidung? }
     If HC_SpriteOverlapBlock(x1,y1,sprite1,x2,y2,sprite2) = False Then Exit;

     { Ja, also bestimmen wir berschneidungsbreich }

     { Fall 1: Erstes Sprite befindet sich in X-Richtung im 2. Sprite (links)}
     If ((x1>=x2) And (x1<=x2+sprite2.Width-1)) Then Begin
        start_x_1:=x1;
        start_x_2:=x1;
     { Fall 2: 2. Sprite befindet sich in X-Richtung im 1. Sprite (links)}
     End Else If ((x2>=x1) And (x2<=x1+sprite1.Width-1)) Then Begin
        start_x_1:=x2;
        start_x_2:=x2;
     End;

     { Fall 1: Erstes Sprite befindet sich in X-Richtung im 2. Sprite (rechts)}
     If ((x1+sprite1.Width-1>=x2) And (x1+sprite1.Width-1<=x2+sprite2.Width-1)) Then Begin
        end_x_1:=x1+sprite1.Width-1;
     { Fall 2: 2. Sprite befindet sich in X-Richtung im 1. Sprite (rechts)}
     End Else If ((x2+sprite2.Width-1>=x1) And (x2+sprite2.Width-1<=x1+sprite1.Width-1)) Then Begin
        end_x_1:=x2+sprite2.Width-1;
     End;

     { Fall 1: 1. Sprite befindet sich in Y-Richtung im 2. Sprite (oben)}
     If ((y1>=y2) And (y1<=y2+sprite2.Height-1)) Then Begin
        start_y_1:=y1;
        start_y_2:=y1;
     { Fall 2: 2. Sprite befindet sich in Y-Richtung im 1. Sprite (oben)}
     End Else If ((y2>=y1) And (y2<=y1+sprite1.Height-1)) Then Begin
        start_y_1:=y2;
        start_y_2:=y2;
     End;

     { Fall 1: 1. Sprite befindet sich in Y-Richtung im 2. Sprite (unten)}
     If ((y1+sprite1.Height-1>=y2) And (y1+sprite1.Height-1<=y2+sprite2.Height-1)) then begin
        end_y_1:=y1+sprite1.Height-1;
     { Fall 2: 2. Sprite befindet sich in Y-Richtung im 1. Sprite (unten)}
     End Else If ((y2+sprite2.Height-1>=y1) And (y2+sprite2.Height-1<=y1+sprite1.Height-1)) Then Begin
        end_y_1:=y2+sprite2.Height-1;
     End;

     { Wir wollen keine absoluten Koordinaten, sondern die innerhalb der
       Sprites }
     Dec(start_x_1,x1);
     Dec(start_x_2,x2);
     Dec(start_y_1,y1);
     Dec(start_y_2,y2);
     Dec(end_x_1,x1);
     Dec(end_y_1,y1);

     { Breite und Hhe des berschneidungsbereiches bestimmen }
     breite:=end_x_1-start_x_1;
     hoehe:=end_y_1-start_y_1;

     { Um diese Werte mssen die Offsets der Sprites nach jeder Zeile
       erhht werden }
     rest_x_1:=(sprite1.Width-breite)*2;
     rest_x_2:=(sprite2.Width-breite)*2;

     { Wenn der Bereich kleiner-gleich Null ist, liest der Rechner irgendwelchen
       Mll aus dem Speicher, das mu verhindert werden! }
     If (breite < 1) Or (hoehe < 1) Then Exit;

     Asm
        pushad
        mov    Result,0
        // Startadresse Sprite1
        mov    eax,DWord Ptr Sprite1.Width
        shl    eax,1
        mul    Start_Y_1
        mov    ebx,Start_x_1
        shl    ebx,1
        add    eax,ebx
        mov    edi,DWord Ptr Sprite1.Addr
        add    edi,eax
        // Startadresse Sprite2
        mov    eax,DWord Ptr Sprite2.Width
        shl    eax,1
        mul    Start_Y_2
        mov    ebx,Start_x_2
        shl    ebx,1
        add    eax,ebx
        mov    esi,DWord Ptr Sprite2.Addr
        add    esi,eax
        // berschneidungsbereich abklappern
        mov    edx,Hoehe
     @YLoop:
        mov    ecx,Breite
     @XLoop:
        mov    ax,[edi]
        and    ax,[esi]
        jnz    @Gotcha
        add    edi,2
        add    esi,2
        dec    ecx
        jne    @XLoop
        add    edi,Rest_x_1
        add    esi,Rest_x_2
        dec    edx
        jne    @YLoop
        jmp    @End
     @Gotcha:
        mov    Result,1
     @End:
        popad
     End;
End;

Procedure HC_LoadBJS8Bit;
Var
   f: File;
   Header: TBJSHeader;
   i: Integer;
   BytesRead: Integer;
   Size: Integer;
   BufSize: Longint;
   Buffer: Array[0..65535] Of Byte;
   Colors: Array[0..255] Of Word;
   _Read: Integer;
   Data, Counter, ColorIndex: Byte;
   Offs: Longint;
   SourceOffs: Longint;
   PixelsLeft: Longint;
   SizeOfFile, BytesLeft: Integer;
   Color: Word;

Function ReadNextByte: Byte;
Var
   BytesToRead: Integer;
Begin
     {$I-}
     If BufSize <= 0 Then Begin
        If BytesLeft < SizeOf(Buffer) Then
           BytesToRead:=BytesLeft
        Else
           BytesToRead:=SizeOf(Buffer);
        BlockRead(f, Buffer, BytesToRead, _Read);
        If (_Read <> BytesToRead) Or (IOResult <> 0) Then Begin
           BufSize:=0;
           HC_Error:=HC_IOError;
           Result:=0;
           Exit;
        End Else Begin
           Dec(BytesLeft, _Read);
           BufSize:=_Read;
           SourceOffs:=0;
           Result:=ReadNextByte;
        End;
     End Else Begin
        Result:=Buffer[SourceOffs];
        Dec(BufSize);
        Inc(SourceOffs);
     End;
     {$I+}
End;

Begin
     {$I-}
     // Datei ffnen
     Assign(f, Filename);
     Reset(f, 1);
     If IOResult <> 0 Then Begin
        HC_Error:=HC_IOError;
        Exit;
     End;

     // ggf. Startposition verndern (z.B. wenn die Datei Teil einer Bibliothek ist)
     If FPos <> Nil Then Begin
        Seek(f, FPos^.FileStart);
     End;

     // Dateigre ermitteln
     If FPos = Nil Then
        SizeOfFile:=FileSize(f)
     Else
        SizeOfFile:=FPos^.FileLength;

     // Wenn noch nichtmal der Header reinpat, kanns ja nix geben
     If SizeOfFile < SizeOf(Header) Then Begin
        HC_Error:=HC_IOError;
        CloseFile(f);
        IOResult;
        Exit;
     End;

     // In BytesLeft wird mitgezhlt, wieviele Bytes der Datei noch verbleiben
     BytesLeft:=SizeOfFile;

     // Header einlesen
     BlockRead(f, Header, SizeOf(Header), BytesRead);
     If (IOResult <> 0) Or (BytesRead <> SizeOf(Header)) Then Begin
        HC_Error:=HC_IOError;
        CloseFile(f);
        IOResult;
        Exit;
     End;
     Dec(BytesLeft, SizeOf(Header));

     // Header berprfen
     With Header Do Begin
          If (id <> 'BJS') Or (Version <> 1) Or (Encoding <> 1) Then Begin
             HC_Error:=HC_WrongFormatError;
             CloseFile(f);
             IOResult;
             Exit;
          End;
     End;

     //ggf. Werte fr abgeschnittene Kanten eintragen
     If Cut <> Nil Then With Cut^ Do Begin
        Left:=Header.CutLeft;
        Top:=Header.CutTop;
        Right:=Header.CutRight;
        Bottom:=Header.CutBottom;
     End;

     // Speicher belegen
     Size:=Header.Width*Header.Height*2;
     With DestSprite Do Begin
          Width:=Header.Width;
          Height:=Header.Height;
          GetMem(Addr, Size);
     End;

     // Palette konvertieren
     For i:=0 To 255 Do Begin
         Colors[i]:=HC_EncodeColor(Header.Palette[i*3] Shr 3,
                                   Header.Palette[i*3+1] Shr 3,
                                   Header.Palette[i*3+2] Shr 3);
     End;

     // Daten lesen
     PixelsLeft:=Header.Width * Header.Height;
     Offs:=Integer(DestSprite.Addr);

     While PixelsLeft > 0 Do Begin

           // Datenbyte lesen
           Data:=ReadNextByte;

           // Wenn 255, dann gepackt, ansonsten einfaches Farb-Byte
           If Data = 255 Then Begin
              ColorIndex:=ReadNextByte;
              Counter:=ReadNextByte;
           End Else Begin
              ColorIndex:=Data;
              Counter:=1;
           End;

           // Gucken, ob was schiefgegangen ist
           If HC_Error = HC_IOError Then Begin
              FreeMem(DestSprite.Addr);
              CloseFile(f);
              Exit;
           End;

           // Farb-Index in "richtigen" Farbwert umwandeln
           Color:=Colors[ColorIndex];

           // "Counter"x Farb-Word schreiben
           Asm
              mov   ax, Color
              mov   cl, Counter
              mov   edx, Offs
           @WriteLoop:
              mov   [edx], ax
              add   edx, 2
              dec   PixelsLeft
              dec   cl
              jne   @WriteLoop
              mov   Offs, edx
           End;
     End;

     // Ende - Datei schlieen
     CloseFile(f);

End;

Initialization
     { Unit initialisieren }
     HC_Error:=HC_NoError;
     HC_Initialized:=False;
     HC_Width:=0;
     HC_Height:=0;
     HC_BufPtr:=Nil;
     HC_ShadowPal:=Nil;
     HC_DDrawObj:=Nil;
     HC_Primary:=Nil;
     HC_Back:=Nil;
     HC_Stride:=0;
Finalization
     { Aufrumen }
     HC_Done;
End.
