Unit HiColor;

{

   Diese Unit Stellt Routinen fr Spritegrafik per WinG-Library zur
   Verfgung.
   Die Farbtiefe betrgt 15 Bit (32.768 Farben).
   Jede beliebige Auflsung kann eingestellt werden.

   Copyright (c) by Jan Strter 1998.

}

Interface

Uses
    Windows, WinG;

Type
    HC_ShadowPalType = Array[0..32768] Of Word;

    HC_Sprite = Record
       Width, Height: Longint;
       Addr: Pointer;
    End;

Function HC_Result: Integer;
{
  Funktioniert analog zu IOResult.
}
Procedure HC_Init(Width, Height: Word; Hndl: HDC);
{
  Initialisiert die Unit und erstellt ein neues WinG-Handle.
}
Procedure HC_Done;
{
  Gibt die durch WinG benutzten Ressourcen wieder frei.
}
Procedure HC_ShowBuf(MulFac: Word);
{
  Kopiert den Video-Buffer per WinGBitBlt in den Bildschirm-Handle
}
Procedure HC_ClearBuf(Col: Word);
{
  Lscht den Video-Buffer mit der angegeben Farbe.
  (Sollte mit HC_MakeHiColor erstellt werden...)
}
Function HC_MakeHiColor(r, g, b: Byte): Word;
{
  Erstellt einen 16-Bit-Wert aus den angegeben Teilen
  Rot, Grn und Blau. Bei R=0, G=0 und B=0 ist die Farbe
  schwarz, bei R=31, G=31 und B=31 ist sie wei.
  31 ist der hchste Wert fr jeden Farbanteil, da
  fr jeden Anteil nur jeweils 5 Bit zur Verfgung stehen,
  das letzte Bit wird nicht verwendet.
}
Procedure HC_EncodeHiColor(col: Word; Var r, g, b: Byte);
{
  Schlsselt den 15-Bit-Farbcode COL in seine einzelnen
  Rot-, Grn- und Blau-Anteile auf.
}
Procedure HC_PutPixel(x, y: Longint; Col: Word);
{
  Zeichnet einen Pixel mit der Farbe Col an der Position
  (x/y) im Video-Buffer.
}
Procedure HC_PutPixelSave(x, y: Longint; Col: Word);
{
  Zeichnet einen Pixel mit der Farbe Col an der Position
  (x/y) im Video-Buffer, prft vorher, ob die Position gltig ist.
}
Function HC_GetPixel(x, y: Longint): Word;
{
  Gibt den Farbwert an Position (x/y) zurck.
  Ist die angegebene Position ungltig, wird 0 zurck gegeben.
}
Procedure HC_PutPixelSprite(x, y: Longint; Col: Word; DestSprite: HC_Sprite);
{
  Zeichnet einen Pixel mit der Farbe Col an der Position
  (x/y) in DESTSPRITE, prft vorher, ob die Position gltig ist.
}
Function HC_GetPixelSprite(x, y: Longint; SourceSprite: HC_Sprite): Word;
{
  Gibt den Farbwert an Position (x/y)  im Sprite SOURCESPRITE zurck.
  Ist die angegebene Position ungltig, wird 0 zurck gegeben.
}
Procedure HC_CreateSprite(Width, Height: Longint; Var DestSprite: HC_Sprite);
{
  Erstellt ein leeres Sprite mit den Ausmaen Width*Height.
}
Procedure HC_LoadPCX8Bit(FileName: String; Var DestSprite: HC_Sprite);
{
  Ich hab' keine Ahnung, wie das PCX-Format 16-Bit Grafiken speichert...
  ...scheie auch!
}
Procedure HC_GetSprite(x, y, Width, Height: Longint; Var DestSprite: HC_Sprite);
{
  Erstellt ein Sprite aus dem Video-Puffer.
  Linke obere Ecke ist (x/y), rechte untere Ecke ist (x+Width/y+Height).
}
Procedure HC_FreeSprite(Var SourceSprite: HC_Sprite);
{
  Lscht das angegebene Sprite aus dem Speicher.
}
Procedure HC_SetWindow(x1, y1, x2, y2: Longint);
{
  Setzt das Grafikfenster.
}
Procedure HC_GetWindow(Var x1, y1, x2 ,y2: Longint);
{
  Gibt die Koordinaten des Grafikfensters zurck.
}
Procedure HC_ClearWindow(Col: Word);
{
  Lscht das Grafik-Fenster mit der Farbe COL.
}
Procedure HC_Line(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet eine Linie vom Punkt (x1/y1) bis (x2/y2).
}
Procedure HC_Rect(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet ein Rechteck von (x1/y1) bis (x2/y2) mit
  der Farbe COL.
}
Procedure HC_FillRect(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet ein geflltes Rechteck von (x1/y1) bis (x2/y2) mit
  der Farbe COL.
}
Procedure HC_FillTrans(x1, y1, x2, y2: Longint; Col: Word);
{
  Zeichnet ein geflltes Rechteck von (x1/y1) bis (x2/y2) mit
  der Farbe COL, wobei es den Mittelwert aus COL und der schon
  bestehenden Farbe verwendet.
}
Procedure HC_Circle(x, y, r: Longint; Col: Word);
{
 Zeichnet einen Kreis mit der Farbe COL an X/Y mit dem Radius R.
}
Procedure HC_Ellipse(x, y, rx, ry: Longint; Col: Word);
{
 Zeichnet eine Ellipse mit der Farbe COL an X/Y.
 RX und RY geben die Ausdehnung der Ellipse in X- und Y-Richtung an.
}
Procedure HC_MakeShadowPal(r, g, b: Longint);
{
 Berechnet die Schattenpalette mit den angegebenen Farbanteilen.
}
Procedure HC_FreeShadowPal;
{
 Gibt den durch die Schattenpalette belegten Speicher wieder frei.
}
Procedure HC_PutSprite(x, y: Longint; SourceSprite: HC_Sprite);
{
  Zeichnet das Sprite SOURCESPRITE an (x/y). Pixel, die einen
  Farbwert von 0 haben (R=0, G=0 und B=0), werden nicht gezeichnet.
  Falls das Sprite ganz oder teilweise auerhalb des Grafikfensters
  ist, wird es geclippt.
}
Procedure HC_PutSpriteTrans(x, y: Longint; SourceSprite: HC_Sprite);
{
  Wie HC_PUTSPRITE, nur da der bestehende Hintergrund beim Zeichnen
  miteinbezogen wird. Ein neugezeichneter Pixel setzt sich zu
  gleichen Anteilen aus dem Pixel aus dem Hintergrund und dem Pixel
  aus dem Sprite zusammen, erscheint also transparent.
}
Procedure HC_PutSpriteBlock(x, y: Longint; SourceSprite: HC_Sprite);
{
  Wie HC_PUTSPRITE, kopiert aber DWords und bercksichtig keine
  Transparenten Stellen. (--> Schneller!)
}
Procedure HC_PutSpriteShadow(x, y: Longint; SourceSprite: HC_Sprite);
{
  Wie HC_PUTSPRITE, benutzt aber die Schattenpalette, um den Effekt
  zu erzielen, da das Sprite auf dem Hintergrund aussieht wie sein
  eigener Schatten.
}
Function HC_SpriteOverlapBlock(x1, y1: Longint; Sprite1: HC_Sprite; x2, y2: Longint; Sprite2: HC_Sprite): Boolean;
{
 Gibt TRUE zurck, falls sich Sprite1 und Sprite2 in ihren Koordinaten
 berschneiden, ansonsten FALSE.
}
Function HC_SpriteOverlap(x1, y1: Longint; Sprite1: HC_Sprite; x2, y2: Longint; Sprite2: HC_Sprite): Boolean;
{
 Wie HC_SPRITEOVERLAPBLOCK, aber auf Pixelebene.
}

Const
   { Fehler-Konstanten }
   HC_Error: Integer = 0;
   HC_NoError = 0;
   HC_WinGError = 1;
   HC_OutOfMemError = 2;
   HC_IOError = 3;
   HC_WrongFormatError = 4;
   HC_AnyError = 5;
   { PutSprite-Style-Konstanten }
   HC_Normal = 0;
   HC_Mirrored = 1;
   HC_TurnedLeft = 2;
   HC_TurnedRight = 3;

Var
   HC_WinGBi: TBitmapInfo;     {Bitmapinfo des Wing-Puffers}
   HC_HWinGDC: HDC;            {Puffer-DC}
   HC_HWinGOldB,               {alter Bitmap-Handle zur Restauration}
   HC_HWinGB: HBitmap;         {Bitmap-Handle}
   HC_PWinGB: Pointer;         {Zeiger auf Puffer}
   HC_Initialized: Boolean;    {True, wenn HC_Init erfolgreich}
   HC_ScreenHandle: HDC;       {Bildschirm-Handle}
   HC_Width,HC_Height: Longint;{Hhe und Breite des Buffers}
   HC_Buffer: Array[0..256*256] Of Byte;
   HC_x1, HC_y1, HC_x2, HC_y2: Longint;
   HC_WinWidth, HC_WinHeight: Longint;
   HC_WinSkip: Longint;
   HC_ShadowPal: ^HC_ShadowPalType;

Implementation

Type
   PCXHeader = Record
        Manufacturer: byte;     { Always 10 for PCX file }

        Version: byte;          { 2 - old PCX - no palette (not used anymore),
                                  3 - no palette,
                                  4 - Microsoft Windows - no palette (only in
                                      old files, new Windows version uses 3),
                                  5 - with palette }

        Encoding: byte;         { 1 is PCX, it is possible that we may add
                                  additional encoding methods in the future }

        Bits_per_pixel: byte;   { Number of bits to represent a pixel
                                  (per plane) - 1, 2, 4, or 8 }

        Xmin: Word;          { Image window dimensions (inclusive) }
        Ymin: Word;          { Xmin, Ymin are usually zero (not always) }
        Xmax: Word;
        Ymax: Word;

        Hdpi: Word;          { Resolution of image (dots per inch) }
        Vdpi: Word;          { Set to scanner resolution - 300 is default }

        ColorMap: Array [0..15, 0..2] Of byte;
                                { RGB palette data (16 colors or less)
                                  256 color palette is appended to end of file }

        Reserved: byte;         { (used to contain video mode)
                                  now it is ignored - just set to zero }

        Nplanes: byte;          { Number of planes }

        Bytes_per_line_per_plane: Word;   { Number of bytes to allocate
                                               for a scanline plane.
                                               MUST be an an EVEN number!
                                               Do NOT calculate from Xmax-Xmin! }

        PaletteInfo: Word;   { 1 = black & white or color image,
                                  2 = grayscale image - ignored in PB4, PB4+
                                  palette must also be set to shades of gray! }

        HscreenSize: Word;   { added for PC Paintbrush IV Plus ver 1.0,  }
        VscreenSize: Word;   { PC Paintbrush IV ver 1.02 (and later)     }
                                { I know it is tempting to use these fields
                                  to determine what video mode should be used
                                  to display the image - but it is NOT
                                  recommended since the fields will probably
                                  just contain garbage. It is better to have
                                  the user install for the graphics mode he
                                  wants to use... }

        Filler: Array [74..127] Of byte;     { Just set to zeros }
   End;

Function HC_Result: Integer;
Begin
     Result:=HC_Error;
     HC_Error:=HC_NoError;
End;

Procedure Swap(Var v1, v2: Longint);
Var
   Dummy: Longint;
Begin
     Dummy:=v1;
     v1:=v2;
     v2:=Dummy;
End;

Procedure HC_SetWindow;
Var
   Width, Height: Longint;
Begin
     If (x1 > x2) Then Swap(x1, x2);
     If (y1 > y2) Then Swap(y1, y2);
     Width:=Abs(x1-x2);
     Height:=Abs(y1-y2);
     If (Width < 1) Or (Height < 1) Then Begin
        HC_Error:=HC_AnyError;
        Exit;
     End;
     If (x1 < 0) Or (x1 >= HC_Width) Or (x2 < 0) Or (x2 >= HC_Width)
        Or (y1 < 0) Or (y1 >= HC_Height) Or (y2 < 0) Or (y2 >= HC_Height)
        Then Begin
        HC_Error:=HC_AnyError;
        Exit;
     End;
     HC_WinWidth:=Width;
     HC_WinHeight:=Height;
     HC_x1:=x1;
     HC_y1:=y1;
     HC_x2:=x2;
     HC_y2:=y2;
     HC_WinSkip:=HC_Width-HC_WinWidth;
End;

Procedure HC_GetWindow;
Begin
     x1:=HC_x1;
     y1:=HC_y1;
     x2:=HC_x2;
     y2:=HC_y2;
End;

Procedure HC_Init(Width, Height: Word; Hndl: HDC);
Begin
     If (Width Mod 2) <> 0 Then Dec(Width);
     { Empfohlenes Format laden, dann Hhe und Breite eintragen }
     FillChar(HC_WinGBi, SizeOf(TBitmapInfo), 0);
     WinGRecommendDIBFormat(@HC_WinGBi);
     With HC_WinGbi.bmiHeader Do Begin
          biCompression:=BI_RGB;
          biBitCount:=16;
          biWidth:=Width;
          { Negative Hhe, um Top-Down-Bitmap zu erzeugen }
          biHeight:=-Height;
     End;

     { Den WinG-Handle erzeugen }
     HC_HWinGDC:=WinGCreateDC;
     If HC_HWinGDC = 0 Then Begin
        HC_Error:=HC_WinGError;
        Exit;
     End;

     { Eine neue Bitmap erzeugen }
     HC_HWinGB:=WinGCreateBitmap(HC_HWinGDC, @HC_WinGBi, @HC_PWinGB);
     If HC_HWinGB = 0 Then Begin
        DeleteDC(HC_HWinGDC);
        HC_Error:=HC_WinGError;
        Exit;
     End;

     { Die erstellte Bitmap auswhlen und die alte merken }
     HC_HWinGOldB:=SelectObject(HC_HWinGDC, HC_HWinGB);

     { Den so erzeugten Video-Buffer lschen }
     FillChar(HC_PWinGB^,Longint(Width)*Longint(Height)*Longint(2),0);

     { Alles hat geklappt }
     HC_ScreenHandle:=Hndl;
     HC_Width:=Width;
     HC_Height:=Height;
     HC_Initialized:=True;
     HC_SetWindow(0,0,Width-1,Height-1);
End;

Procedure HC_Done;
Begin
     If HC_Initialized Then Begin

        { Resourcen wieder freiegeben }
        SelectObject(HC_HWinGDC, HC_HWinGOldB);
        DeleteObject(HC_HWinGB);
        DeleteDC(HC_HWinGDC);

        { Variablen wieder auf Anfangswerte setzen }
        HC_Initialized:=False;
        HC_Width:=0;
        HC_Height:=0;
        HC_PWinGB:=Nil;
        HC_ScreenHandle:=0;
        HC_FreeShadowPal;

     End;
End;

Procedure HC_ShowBuf;
Begin
     If MulFac = 1 Then Begin
        If WinGBitBlt(HC_ScreenHandle,0,0,HC_Width,HC_Height,HC_HWinGDC,0,0)
           = False Then
           HC_Error:=HC_WinGError;
     End Else Begin
        If WinGStretchBlt(HC_ScreenHandle,0,0,HC_Width*MulFac,HC_Height*MulFac,
        HC_HWinGDC,0,0,HC_Width,HC_Height) = False Then
           HC_Error:=HC_WinGError;
     End;
End;

Procedure HC_ClearBuf;
Begin
Asm
   pushad
   mov    edi,HC_PWinGB
   mov    eax,HC_Width
   mul    HC_Height
   shl    eax,1
   mov    ebx,4
   mov    edx,0
   div    ebx
   mov    ecx,eax
   mov    ax,Col
   shl    ax,16
   mov    ax,Col
@DWordLoop:
   mov    [edi],eax
   add    edi,4
   dec    ecx
   jne    @DWordLoop
   or     dx,dx
   je     @End
   mov    [edi],ax
@End:
   popad
End;
End;

Function HC_MakeHiColor;
Begin
Asm
   pushad
   xor    ax,ax
   mov    al,b
   xor    bx,bx
   mov    bl,g
   shl    bx,5
   or     ax,bx
   xor    bx,bx
   mov    bl,r
   shl    bx,10
   or     ax,bx
   mov    Result,ax
   popad
End;
End;

Procedure HC_EncodeHiColor;
Begin
Asm
   pushad
   mov    dx,Col
   mov    ax,dx
   and    al,00011111b
   mov    ebx,b
   mov    [ebx],al
   and    ah,01111100b
   shr    ah,2
   mov    ebx,r
   mov    [ebx],ah
   and    dx,0000001111100000b
   shr    dx,5
   mov    ebx,g
   mov    [ebx],dl
   popad
End;
End;

Procedure HC_PutPixel;
Begin
Asm
   pushad
   // Linke obere Ecke des Grafikfenster dazu addieren
   mov   eax,HC_x1
   add   x,eax
   mov   eax,HC_y1
   add   y,eax
   // Adresse berechnen
   mov   edi,HC_PWinGB
   mov   edx,0
   mov   eax,HC_Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   mov   ax,Col
   // Pixel setzen
   mov   [edi],ax
   popad
End;
End;

Procedure HC_PutPixelSave;
Begin
Asm
   pushad
   // Linke obere Ecke des Grafikfenster dazu addieren
   mov   eax,HC_x1
   add   x,eax
   mov   eax,HC_y1
   add   y,eax
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,HC_x1
   jl    @End
   cmp   eax,HC_x2
   jg    @End
   mov   eax,y
   cmp   eax,HC_y1
   jl    @End
   cmp   eax,HC_y2
   jg    @End
   // Adresse berechnen
   mov   edi,HC_PWinGB
   mov   edx,0
   mov   eax,HC_Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   mov   ax,Col
   // Pixel setzen
   mov   [edi],ax
@End:
   popad
End;
End;

Function HC_GetPixel;
Begin
Asm
   pushad
   // Linke obere Ecke des Grafikfenster dazu addieren
   mov   eax,HC_x1
   add   x,eax
   mov   eax,HC_y1
   add   y,eax
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,HC_x1
   jl    @Invalid
   cmp   eax,HC_x2
   jg    @Invalid
   mov   eax,y
   cmp   eax,HC_y1
   jl    @Invalid
   cmp   eax,HC_y2
   jg    @Invalid
   // Adresse berechnen
   mov   edi,HC_PWinGB
   mov   edx,0
   mov   eax,HC_Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   edi,eax
   // Pixel holen
   mov   ax,[edi]
   jmp   @End
@Invalid:
   mov   ax,0
@End:
   mov   Result,ax
   popad
End;
End;

Procedure HC_PutPixelSprite;
Begin
Asm
   pushad
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr DestSprite.Width
   jge   @End
   mov   eax,y
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr DestSprite.Height
   jge   @End
   // Adresse berechnen
   mov   ebx,DWord Ptr DestSprite.Addr
   mov   edx,0
   mov   eax,Dword Ptr DestSprite.Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   ebx,eax
   mov   ax,Col
   // Pixel setzen
   mov   [ebx],ax
@End:
   popad
End;
End;

Function HC_GetPixelSprite;
Begin
Asm
   pushad
   // Prfen, ob im sichtbaren Bereich
   mov   eax,x
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr SourceSprite.Width
   jge   @End
   mov   eax,y
   cmp   eax,0
   jl    @End
   cmp   eax,DWord Ptr SourceSprite.Height
   jge   @End
   // Adresse berechnen
   mov   ebx,DWord Ptr SourceSprite.Addr
   mov   edx,0
   mov   eax,Dword Ptr SourceSprite.Width
   shl   eax,1
   mul   y
   add   eax,x
   add   eax,x
   add   ebx,eax
   // Pixel holen
   mov   ax,[ebx]
   jmp   @End
@Invalid:
   mov   ax,0
@End:
   mov   Result,ax
   popad
End;
End;

Procedure HC_EllipseAndCircle(xm, ym: Longint; xr,  yr: Longint; DisplayColor: Word);
Var
   x, y: Longint;
   dx, dy,
   ySub, xAdd,
   r: Longint;
Begin
     If (xr = 0) Or (yr = 0) Then Exit;
     dx:=Longint(xr * 2) * Longint(xr);
     dy:=Longint(yr * 2) * Longint(yr);
     ySub:=0;
     xAdd:=Longint(xr) * dy;
     r:=xAdd Div 2;
     x:=xr;
     y:=0;
     Repeat
           HC_PutPixelSave(xm+x, ym+y, DisplayColor);
           HC_PutPixelSave(xm+x, ym-y, DisplayColor);
           HC_PutPixelSave(xm-x, ym-y, DisplayColor);
           HC_PutPixelSave(xm-x, ym+y, DisplayColor);
           If r >= 0 Then
           Begin
                Inc(y);
                Inc(ySub, dx);
                Dec(r, ySub);
           End;
           If r < 0 Then
           Begin
                Dec(x);
                Dec(xAdd, dy);
                Inc(r, xAdd);
           End;
     Until x < 0;
End;

Procedure HC_Circle;
Begin
     HC_EllipseAndCircle(x, y, r, r, Col);
End;

Procedure HC_Ellipse;
Begin
     HC_EllipseAndCircle(x, y, rx, ry, Col);
End;

Procedure HC_Line(x1, y1, x2, y2: Longint; Col: Word);
Var
   x, y, d_x, d_y, e_x, e_y, yadd, yunit: Longint;
Begin
Asm
   pushad
   mov   eax,x1                   // Linke obere Ecke des Fensters addieren
   add   eax,HC_x1
   mov   x1,eax
   mov   eax,y1
   add   eax,HC_y1
   mov   y1,eax
   mov   eax,x2
   add   eax,HC_x1
   mov   x2,eax
   mov   eax,y2
   add   eax,HC_y1
   mov   y2,eax
   mov   eax,x1
   cmp   eax,x2                   // x1 > x2 ?
   jle   @Nicht_Swappen
   xchg  eax,x2                   // Ja, x1 und x2 vertauschen
   mov   x1,eax
   mov   eax,y1
   xchg  eax,y2
   mov   y1,eax
@Nicht_Swappen:                   // Nein. Nicht verstauschen.
   mov   eax,x1
   sub   eax,x2
   mov   e_x,eax                  // e_x := x1 - x2
   mov   eax,y1
   sub   eax,y2
   cmp   eax,0
   jle   @Kein_Neg
   neg   eax
@Kein_Neg:
   mov   e_y,eax                  // e_y := -abs(y2-y1)
   mov   eax,e_x
   neg   eax
   shl   eax,1
   mov   d_x,eax                  // d_x := -(e_x) Shl 1
   mov   eax,e_y
   neg   eax
   shl   eax,1
   mov   d_y,eax                  // d_y := -(e_y) Shl 1
   mov   eax,y1
   cmp   eax,y2                   // y1 > y2 ?
   jle   @Y_Positiv
   mov   eax,0                   // Nein, von unten nach oben
   mov   eax,HC_Width
   shl   eax,1
   neg   eax
   mov   yadd,eax                // yadd := -HC__Width
   mov   yunit,-1                // yunit := -1
   jmp   @Weiter1
@Y_Positiv:                       // Ja, von oben nach unten
   mov   eax,0
   mov   eax,HC_Width
   shl   eax,1
   mov   yadd,eax                // yadd := HC__Width
   mov   yunit,1                 // yunit := 1
@Weiter1:
   mov   eax,0
   mov   ebx,0
   mov   edx,0
   mov   eax,HC_Width
   shl   eax,1
   mul   y1
   mov   ebx,x1
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_PWinGB           // eax := eax + Video-Puffer-Adresse
   mov   edi,eax                 // Start-Offset nach EDI laden
   mov   eax,x1
   mov   x,eax                    // x := x1
   mov   eax,y1
   mov   y,eax                    // y := y1
   mov   eax,d_x
   cmp   eax,d_y                  // d_x < d_y
   jl    @Y_Schleife_Anfang     // ja, dann Y-Schleife anspringen
   mov   ecx,x2                  // Nein, dann mit X-Schleife weitermachen
   sub   ecx,x1                  // ecx := x2 - x1
   cmp   ecx,0                   // Falls ecx negativ ist...
   jge   @Weiter3
   neg   ecx                     // ...Vorzeichen umkehren
@Weiter3:
   or    ecx,ecx                 // Wenn ecx Null ist...
   jne    @X_Schleife
   inc   ecx                     // ecx um 1 erhhen, damit der Rechner nicht abschmiert
@X_Schleife:
   mov   eax,HC_x2               //-------------------------------------
   cmp   x,eax                    //
   jg    @Nicht_Zeichnen1       // Prfen, ob sich x/y im Bildschirm
   mov   eax,HC_y2               // befindet. Wenn nicht, zum Label
   cmp   y,eax                    // @Nicht_Zeichnen1 springen
   jg    @Nicht_Zeichnen1       //--------------------------------------
   mov   eax,HC_x1
   cmp   x,eax
   jl    @Nicht_Zeichnen1
   mov   eax,HC_y1
   cmp   y,eax
   jl    @Nicht_Zeichnen1
   mov   ax,Col
   mov   [edi],ax             // Pixel nach [EDI] schreiben
@Nicht_Zeichnen1:
   mov   eax,d_y
   add   e_x,eax                  // Inc(e_x, d_y)
   cmp   e_x,0                   // e_x > 0 ?
   jle   @Y_Nicht_weiter        // Nein, dann diesen Teil berspringen
   mov   eax,d_x
   sub   e_x,eax                  // Dec(e_x, d_x)
   mov   eax,yunit
   add   y,eax                    // In nchste Zeile springen
   add   edi,yadd
@Y_Nicht_Weiter:
   inc   x                       // In nchste Spalte springen
   add   edi,2
   dec   ecx                      // Zhler dekrementieren
   jns    @X_Schleife           // Bis ecx = 0
   jmp   @Ende                  // Fertig, ans Ende springen
@Y_Schleife_Anfang:
   mov   ecx,y1
   sub   ecx,y2                   // ecx := y1 - y2
   cmp   ecx,0                    // Falls ecx negativ ist...
   jg    @Y_Schleife
   neg   ecx                      // ...Vorzeichen umkehren
@Y_Schleife:
   mov   eax,HC_x2               //---------------------------------------
   cmp   x,eax                    //
   jg    @Nicht_Zeichnen2       // Prfen, ob x/y im Bildschirm ist
   mov   eax,HC_y2               //
   cmp   y,eax                    //
   jg    @Nicht_Zeichnen2       //---------------------------------------
   mov   eax,HC_x1
   cmp   x,eax
   jl    @Nicht_Zeichnen2
   mov   eax,HC_y1
   cmp   y,eax
   jl    @Nicht_Zeichnen2
   mov   ax,Col
   mov   [edi],ax             // Pixel nach [EDI] schreiben
@Nicht_Zeichnen2:
   mov   eax,d_x
   add   e_y,eax                  // Inc(e_y, d_x)
   cmp   e_y,0                   // e_y > 0 ?
   jl    @X_Nicht_Weiter        // Nein, dann diesen Teil berspringen
   mov   eax,d_y
   sub   e_y,eax                  // Dec(e_y, d_y)
   inc   x
   add   edi,2                     // In nchste Spalte springen
@X_Nicht_Weiter:
   mov   eax,yunit
   add   y,eax                    // In nchste Zeile springen
   add   edi,yadd
   dec   ecx                      // Zhler dekrementieren
   jns   @Y_Schleife            // Bis ecx = 0
@Ende:
   popad
End;
End;

Procedure HC_Rect;
Begin
     HC_Line(x1,y1,x2,y1,Col);
     HC_Line(x1,y1,x1,y2,Col);
     HC_Line(x1,y2,x2,y2,Col);
     HC_Line(x2,y1,x2,y2,Col);
End;

Procedure HC_FillRect;
Var
   Skip: Longint;
Begin
Asm
   pushad
   // Obere linke Ecke des Grafik-Fensters dazu addieren
   mov   eax,HC_x1
   add   x1,eax
   add   x2,eax
   mov   eax,HC_y1
   add   y1,eax
   add   y2,eax
   // ggf. Koordinaten vertauschen, damit wir immer von links nach
   // rechts und von oben nach unten zeichnen knnen
   mov   eax,x1
   cmp   eax,x2
   jle   @NoXSwap
   mov   ebx,x2
   xchg  eax,ebx
   mov   x1,eax
   mov   x2,ebx
@NoXSwap:
   mov   eax,y1
   cmp   eax,y2
   jle   @NoYSwap
   mov   ebx,y2
   xchg  eax,ebx
   mov   y1,eax
   mov   y2,ebx
@NoYSwap:
   // Prfen, ob das Rechteck berhaupt im sichtbaren Bereich liegt
   mov   eax,x1
   cmp   eax,HC_x2
   jg    @End
   mov   eax,x2
   cmp   eax,HC_x1
   jl    @End
   mov   eax,y1
   cmp   eax,HC_y2
   jg    @End
   mov   eax,y2
   cmp   eax,HC_y1
   jl    @End
   // ggf. Clipping durchfhren
   mov   eax,x1
   cmp   eax,HC_x1
   jge   @NoX1Clip
   mov   eax,HC_x1
   mov   x1,eax
@NoX1Clip:
   mov   eax,x2
   cmp   eax,HC_x2
   jle   @NoX2Clip
   mov   eax,HC_x2
   mov   x2,eax
@NoX2Clip:
   mov   eax,y1
   cmp   eax,HC_y1
   jge   @NoY1Clip
   mov   eax,HC_y1
   mov   y1,eax
@NoY1Clip:
   mov   eax,y2
   cmp   eax,HC_y2
   jle   @NoY2Clip
   mov   eax,HC_y2
   mov   y2,eax
@NoY2Clip:
   // Startadresse berechnen
   mov   eax,HC_Width
   shl   eax,1
   mul   y1
   mov   ebx,x1
   shl   ebx,1
   add   eax,ebx
   mov   edi,HC_PWinGB
   add   edi,eax
   // Breite des zu zeichnenden Bereichs berechnen und in EBX merken
   mov   ebx,x2
   sub   ebx,x1
   inc   ebx
   // Berechnen, wieviel am Ende jeder Zeile dazu addiert werden mu
   mov   eax,HC_Width
   sub   eax,ebx
   shl   eax,1
   mov   Skip,eax
   // In DWords und Words zerlegen und in EBX und EDX merken
   mov   eax,ebx
   mov   ecx,2
   mov   edx,0   // EDX lschen, damit kein Mll drin zurckbleibt
   div   ecx     // der Rest wird automatisch in EDX abgelegt
   mov   ebx,eax
   // Hhe des zu zeichnenden Bereichs berechnen und in ESI merken
   mov   eax,y2
   sub   eax,y1
   mov   esi,eax
   inc   esi
   // Jetzt kommt die Arbeit - den ganzen Kse mit COL ausfllen...
@YLoop:
   mov   ax,Col
   shl   eax,16
   mov   ax,Col
   mov   ecx,ebx
   or    ecx,ecx
   je    @WordCopy
@DWordLoop:
   mov   [edi],eax
   add   edi,4
   dec   ecx
   jne   @DWordLoop
@WordCopy:
   or    edx,edx
   je    @NoWord
   mov   [edi],ax
   add   edi,2
@NoWord:
   add   edi,Skip
   dec   esi
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_ClearWindow(Col: Word);
Begin
     HC_FillRect(0,0,HC_Width,HC_Height,Col);
End;

Procedure HC_FillTrans;
Var
   Skip, Width, YCount: Longint;
Begin
Asm
   pushad
   // Obere linke Ecke des Grafik-Fensters dazu addieren
   mov   eax,HC_x1
   add   x1,eax
   add   x2,eax
   mov   eax,HC_y1
   add   y1,eax
   add   y2,eax
   // ggf. Koordinaten vertauschen, damit wir immer von links nach
   // rechts und von oben nach unten zeichnen knnen
   mov   eax,x1
   cmp   eax,x2
   jle   @NoXSwap
   mov   ebx,x2
   xchg  eax,ebx
   mov   x1,eax
   mov   x2,ebx
@NoXSwap:
   mov   eax,y1
   cmp   eax,y2
   jle   @NoYSwap
   mov   ebx,y2
   xchg  eax,ebx
   mov   y1,eax
   mov   y2,ebx
@NoYSwap:
   // Prfen, ob das Rechteck berhaupt im sichtbaren Bereich liegt
   mov   eax,x1
   cmp   eax,HC_x2
   jg    @End
   mov   eax,x2
   cmp   eax,HC_x1
   jl    @End
   mov   eax,y1
   cmp   eax,HC_y2
   jg    @End
   mov   eax,y2
   cmp   eax,HC_y1
   jl    @End
   // ggf. Clipping durchfhren
   mov   eax,x1
   cmp   eax,HC_x1
   jge   @NoX1Clip
   mov   eax,HC_x1
   mov   x1,eax
@NoX1Clip:
   mov   eax,x2
   cmp   eax,HC_x2
   jle   @NoX2Clip
   mov   eax,HC_x2
   mov   x2,eax
@NoX2Clip:
   mov   eax,y1
   cmp   eax,HC_y1
   jge   @NoY1Clip
   mov   eax,HC_y1
   mov   y1,eax
@NoY1Clip:
   mov   eax,y2
   cmp   eax,HC_y2
   jle   @NoY2Clip
   mov   eax,HC_y2
   mov   y2,eax
@NoY2Clip:
   // Startadresse berechnen
   mov   eax,HC_Width
   shl   eax,1
   mul   y1
   mov   ebx,x1
   shl   ebx,1
   add   eax,ebx
   mov   edi,HC_PWinGB
   add   edi,eax
   // Breite des zu zeichnenden Bereichs berechnen und in WIDTH merken
   mov   ebx,x2
   sub   ebx,x1
   inc   ebx
   mov   Width,ebx
   // Berechnen, wieviel am Ende jeder Zeile dazu addiert werden mu
   mov   eax,HC_Width
   sub   eax,ebx
   shl   eax,1
   mov   Skip,eax
   // Hhe des zu zeichnenden Bereichs berechnen und in YCount merken
   mov   eax,y2
   sub   eax,y1
   mov   YCount,eax
   inc   YCount
   // Jetzt kommt die Arbeit - den ganzen Kse mit Mittelwerten ausfllen...
@YLoop:
   mov   esi,Width
@XLoop:
   mov   dx,[edi]
   mov   bx,dx
   and   bx,0111110000000000b
   mov   ax,Col
   and   ax,0111110000000000b
   add   bx,ax
   shr   bx,1
   and   bx,0111110000000000b
   mov   cx,bx
   mov   bx,dx
   and   bx,0000001111100000b
   mov   ax,Col
   and   ax,0000001111100000b
   add   bx,ax
   shr   bx,1
   and   bx,0000001111100000b
   or    cx,bx
   mov   bx,dx
   and   bx,0000000000011111b
   mov   ax,Col
   and   ax,0000000000011111b
   add   bx,ax
   shr   bx,1
   and   bx,0000000000011111b
   or    cx,bx
   mov   [edi],cx
   add   edi,2
   dec   esi
   jne   @XLoop
   add   edi,Skip
   dec   YCount
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_CreateSprite;
Var
   Size: Longint;
Begin
     Size:=Width*Height*2;
     If Size <= 0 Then Begin
        HC_Error:=HC_WrongFormatError;
        Exit;
     End;
     GetMem(DestSprite.Addr,Size);
     DestSprite.Width:=Width;
     DestSprite.Height:=Height;
     FillChar(DestSprite.Addr^,Size,0);
End;

Procedure HC_LoadPCX8Bit;
Const
   Compress = $C0;
   DeCompress = $3F;
Var
   Buffer: Array[0..65535] Of Byte;
   Palette: Array[0..767] Of Byte;
   Colors: Array[0..255] Of Word;
   Header: PCXHeader;
   f: File;
   _Read: Integer;
   Width, Height: Integer;
   Size: Longint;
   i, OldPos: Longint;
   BufSize: Longint;
   Data, Counter: Byte;
   Offs: Longint;
   SourceOffs: Longint;
   ColAdr: Longint;
   ScanLineCount: Longint;

Function ReadNextByte: Byte;
Begin
     {$I-}
     If BufSize <= 0 Then Begin
        BlockRead(f,Buffer,SizeOf(Buffer),_Read);
        BufSize:=_Read;
        If (BufSize = 0) Or (IOResult <> 0) Then Begin
           HC_Error:=HC_IOError;
           Result:=0;
           Exit;
        End Else Begin
           SourceOffs:=0;
           Result:=ReadNextByte;
        End;
     End Else Begin
        Result:=Buffer[SourceOffs];
        Dec(BufSize);
        Inc(SourceOffs);
     End;
     {$I+}
End;

Begin
     {$I-}

     // Datei ffnen, abbrechen falls Fehler
     IOResult;
     AssignFile(f, FileName);
     Reset(f, 1);
     If IOResult <> 0 Then Begin
        HC_Error:=HC_IOError;
        Exit;
     End;

     // Header einlesen, abberechen falls Fehler
     BlockRead(f, Header, SizeOf(Header), _Read);
     If (_Read <> SizeOf(Header)) Or (IOResult <> 0) Then Begin
        CloseFile(f);
        IOResult;
        HC_Error:=HC_IOError;
        Exit;
     End;

     // Speicher berechnen und allokieren, abbrechen falls Fehler
     With Header Do Begin
          Width:=Abs(XMax-XMin)+1;
          Height:=Abs(YMax-YMin)+1;
          If (Bits_Per_Pixel <> 8) Or (NPLanes <> 1) Or (Width = 0)
             Or (Height = 0) Then Begin
             CloseFile(f);
             IOResult;
             HC_Error:=HC_WrongFormatError;
             Exit;
          End;
     End;
     Size:=Longint(Width)*Longint(Height);
     GetMem(DestSprite.Addr, Size*2);
     DestSprite.Width:=Width;
     DestSprite.Height:=Height;

     // Palette einlesen abbrechen, falls Fehler
     OldPos:=FilePos(f);
     Seek(f,FileSize(f)-SizeOf(Palette));
     BlockRead(f,Palette,SizeOf(Palette),_Read);
     If (IOResult <> 0) Or (_Read <> SizeOf(Palette)) Then Begin
        FreeMem(DestSprite.Addr,Size*2);
        CloseFile(f);
        IOResult;
        HC_Error:=HC_IOError;
        Exit;
     End;
     Seek(f,OldPos);

     // Palette in 16-Bit Farbwerte umrechnen
     For i:=0 To 255 Do Begin
         Colors[i]:=HC_MakeHiColor(Palette[i*3] Shr 3,
                                  Palette[i*3+1] Shr 3,
                                  Palette[i*3+2] Shr 3);
     End;
     // Variablen initialisieren
     i:=Size;
     BufSize:=0;
     Offs:=Longint(DestSprite.Addr);
     SourceOffs:=0;
     ColAdr:=Longint(@Colors);
     ScanLineCount:=DestSprite.Width;

     // Jetzt alle Bytes durchgehen und prfen, ob die beiden
     // obersten Bit gesetzt sind. Ist das der Fall, ist das Byte
     // ein Zhl-Byte
     Repeat
           Data:=ReadNextByte;
           If HC_Error = HC_IOError Then Begin
              FreeMem(DestSprite.Addr,Size*2);
              CloseFile(f);
              IOResult;
              Exit;
           End;
           If (Data And Compress) = Compress Then Begin
              Counter:=(Data And Decompress);
              Data:=ReadNextByte;
              If HC_Error = HC_IOError Then Begin
                 FreeMem(DestSprite.Addr,Size*2);
                 CloseFile(f);
                 IOResult;
                 Exit;
              End;
           End Else Counter:=1;
           Asm
              pushad
              // Farbwert aus Tabelle lesen
              mov     edi, ColAdr
              mov     eax, 0
              mov     al, Data
              shl     eax, 1
              add     edi, eax
              mov     ax, [edi]
              // Zieladresse und Zhler laden
              mov     edi, Offs
              mov     cl, Counter
              or      cl, cl
              je      @End
           @WriteLoop:
              // Eine Scanline mu ja immer "even" sein. Grummel, grummel!
              cmp     ScanLineCount, 0
              jne     @NoScanLineEnd
              mov     ebx, Width
              mov     ScanLineCount, ebx
              cmp     bx, Word Ptr Header.Bytes_Per_Line_Per_Plane
              jne     @OverridePixel
              // Pixel schreiben und Zieladdresse um 1 Word erhhen
           @NoScanLineEnd:
              mov     [edi], ax
              add     edi, 2
              add     Offs, 2
              dec     ScanLineCount
           @OverridePixel:
              dec     i
              dec     cl
              // Solange, bis CL = 0
              jne     @WriteLoop
           @End:
              popad
           End;
     Until i <= 0;

     // Ende - Datei schlieen
     CloseFile(f);
     IOResult;
     {$I+}

End;

Procedure HC_GetSprite;
Var
   Size, Skip: Longint;
   Adr: Pointer;
Begin
     If (x < 0) Or (x+Width > HC_Width) Or (y < 0) Or (y+Height > HC_Height) Then Begin
        HC_Error:=HC_WrongFormatError;
        Exit;
     End;
     Size:=Width*Height*2;
     If Size <= 0 Then Begin
        HC_Error:=HC_WrongFormatError;
        Exit;
     End;
     GetMem(DestSprite.Addr,Size);
     DestSprite.Width:=Width;
     DestSprite.Height:=Height;
     Skip:=(HC_Width-Width)*2;
     Adr:=DestSprite.Addr;
     Asm
        pushad
        mov    eax,HC_Width
        shl    eax,1
        mul    y
        mov    ebx,x
        shl    ebx,1
        add    eax,ebx
        mov    esi,HC_PWinGB
        add    esi,eax
        mov    edi,Adr
        mov    edx,Height
     @YLoop:
        mov    ecx,Width
     @XLoop:
        mov    ax,[esi]
        mov    [edi],ax
        add    esi,2
        add    edi,2
        dec    ecx
        jne    @XLoop
        add    esi,Skip
        dec    edx
        jne    @YLoop
        popad
     End;
End;

Procedure HC_FreeSprite;
Var
   Size: Longint;
Begin
     With SourceSprite Do Begin
          Size:=Width*Height*2;
          If Size > 0 Then Begin
             FreeMem(Addr,Size);
             Width:=0;
             Height:=0;
             Addr:=Nil;
          End;
     End;
End;

Procedure HC_MakeShadowPal;
Var
   i: Longint;
   r2, g2, b2: Byte;
Begin
     If HC_ShadowPal = Nil Then New(HC_ShadowPal);
     For i:=0 To 32768 Do Begin
         HC_EncodeHiColor(i,r2,g2,b2);
         HC_ShadowPal^[i]:=HC_MakeHiColor(r*r2 Div 100, g*g2 Div 100, b*b2 Div 100);
     End;
End;

procedure HC_FreeShadowPal;
Begin
     If HC_ShadowPal <> Nil Then Begin
        Dispose(HC_ShadowPal);
        HC_ShadowPal:=Nil;
     End;
End;

Procedure HC_PutSprite;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
Begin
Asm
   pushad
   // Obere/Linke Ecke dess Clipping-Fensters dazuaddieren
   mov   eax, HC_x1
   add   x, eax
   mov   eax, HC_y1
   add   y, eax
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax,HC_Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax,HC_Width
   shl   eax,1
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_PWinGB;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // Bitmap kopieren
   mov   edx,Dty
@YLoop:
   mov   ecx,Dtx
@XLoop:
   mov   ax,[esi]
   or    ax,ax
   je    @Null
   mov   [edi],ax
@Null:
   add   edi,2
   add   esi,2
   dec   ecx
   jne   @XLoop
   add   edi,SkipDest
   add   esi,SkipSource
   dec   edx
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_PutSpriteTrans;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
   XCount, YCount: Longint;
   Pixel: Word;
Begin
Asm
   pushad
   // Obere/Linke Ecke dess Clipping-Fensters dazuaddieren
   mov   eax, HC_x1
   add   x, eax
   mov   eax, HC_y1
   add   y, eax
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax,HC_Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax,HC_Width
   shl   eax,1
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_PWinGB;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // Bitmap kopieren
   mov   eax,Dty
   mov   YCount,eax
@YLoop:
   mov   eax,Dtx
   mov   XCount,eax
@XLoop:
   mov   cx,[esi]
   or    cx,cx
   je    @Null
   // Mittelwert bilden
   mov   dx,[edi]
   mov   ax,cx
   and   al,00011111b
   mov   bx,dx
   and   bl,00011111b
   add   al,bl
   shr   al,1
   mov   Byte Ptr Pixel,al
   and   ah,011111000b
   and   bh,011111000b
   add   ah,bh
   shr   ah,1
   and   ah,011111000b
   mov   Byte Ptr Pixel+1,ah
   and   cx,0000001111100000b
   and   dx,0000001111100000b
   add   cx,dx
   shr   cx,1
   and   cx,0000001111100000b
   or    Pixel,cx
   mov   ax,Pixel
   mov   [edi],ax
@Null:
   add   edi,2
   add   esi,2
   dec   XCount
   jne   @XLoop
   add   edi,SkipDest
   add   esi,SkipSource
   dec   YCount
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_PutSpriteBlock;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
   DWords, Words: Longint;
Begin
Asm
   pushad
   // Obere/Linke Ecke dess Clipping-Fensters dazuaddieren
   mov   eax, HC_x1
   add   x, eax
   mov   eax, HC_y1
   add   y, eax
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax,HC_Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax,HC_Width
   shl   eax,1
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_PWinGB;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // In DWords und Words aufschlsseln
   xor   edx,edx
   mov   eax,Dtx
   shl   eax,1
   mov   ebx,4
   div   ebx
   mov   DWords,eax
   mov   Words,edx
   // Bitmap kopieren
   mov   edx,Dty
@YLoop:
   mov   ecx,DWords
   or    ecx,ecx
   je    @WordCopy
@DWordLoop:
   mov   eax,[esi]
   mov   [edi],eax
   add   edi,4
   add   esi,4
   dec   ecx
   jne   @DwordLoop
@WordCopy:
   cmp   Words,0
   je    @NoWords
   mov   ax,[esi]
   mov   [edi],ax
   add   esi,2
   add   edi,2
@NoWords:
   add   edi,SkipDest
   add   esi,SkipSource
   dec   edx
   jne   @YLoop
@End:
   popad
End;
End;

Procedure HC_PutSpriteShadow;
Var
   ClipUp, ClipDn, ClipLt, ClipRt: Longint;
   SkipDest, SkipSource: Longint;
   Dtx, Dty: Longint;
   StartX, StartY: Longint;
   YCount: Longint;
Begin
Asm
   pushad
   // Obere/Linke Ecke dess Clipping-Fensters dazuaddieren
   mov   eax, HC_x1
   add   x, eax
   mov   eax, HC_y1
   add   y, eax
   // Prfen, ob Sprite berhaupt im Bild, wenn nein -> Raus!
   mov   eax,x
   cmp   eax,HC_x2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x1
   jle   @End
   mov   eax,y
   cmp   eax,HC_y2
   jge   @End
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y1
   jle   @End
   // Clipping-Variablen mit Standardwerten laden
   mov   eax,x
   mov   StartX,eax
   mov   eax,y
   mov   StartY,eax
   mov   eax,0
   mov   ClipUp,eax
   mov   ClipDn,eax
   mov   ClipRt,eax
   mov   ClipLt,eax
   // Clipping links
   mov   eax,x
   cmp   eax,HC_x1
   jge   @ClipRight
   mov   eax,HC_x1
   sub   eax,x
   mov   ClipLt,eax
   mov   eax,HC_x1
   mov   StartX,eax
@ClipRight:
   // Clipping rechts
   mov   eax,x
   add   eax,DWord Ptr SourceSprite.Width
   cmp   eax,HC_x2
   jle   @ClipUp
   sub   eax,HC_x2
   dec   eax
   mov   ClipRt,eax
@ClipUp:
   // Clipping oben
   mov   eax,y
   cmp   eax,HC_y1
   jge   @ClipDn
   mov   eax,HC_y1
   sub   eax,y
   mov   ClipUp,eax
   mov   eax,HC_y1
   mov   StartY,eax
@ClipDn:
   // Clipping unten
   mov   eax,y
   add   eax,DWord Ptr SourceSprite.Height
   cmp   eax,HC_y2
   jle   @NoClipDn
   sub   eax,HC_y2
   dec   eax
   mov   ClipDn,eax
@NoClipDn:
   // Breite und Hhe des zu zeichnenden Bereichs berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,ClipLt
   sub   eax,ClipRt
   mov   Dtx,eax
   mov   eax,DWord Ptr SourceSprite.Height
   sub   eax,ClipUp
   sub   eax,ClipDn
   mov   Dty,eax
   // Berechnen, wieviel jede Zeile dazu addiert werden mu
   mov   eax,HC_Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipDest,eax
   mov   eax,DWord Ptr SourceSprite.Width
   sub   eax,Dtx
   shl   eax,1
   mov   SkipSource,eax
   // Ziel-Anfangs-Adresse berechnen
   mov   eax,HC_Width
   shl   eax,1
   mul   StartY
   mov   ebx,StartX
   shl   ebx,1
   add   eax,ebx
   add   eax,HC_PWinGB;
   mov   edi,eax
   // Quell-Anfangs-Adresse berechnen
   mov   eax,DWord Ptr SourceSprite.Width
   shl   eax,1
   mul   ClipUp
   mov   ebx,ClipLt
   shl   ebx,1
   add   eax,ebx
   add   eax,DWord Ptr SourceSprite.Addr
   mov   esi,eax
   // Bitmap kopieren
   mov   eax,Dty
   mov   YCount,eax
   mov   edx,HC_ShadowPal
@YLoop:
   mov   ecx,Dtx
   mov   eax,0
@XLoop:
   mov   ax,[esi]
   or    ax,ax
   je    @Null
   mov   ebx,edx
   mov   ax,[edi]
   shl   ax,1
   add   ebx,eax
   mov   ax,[ebx]
   mov   [edi],ax
@Null:
   add   edi,2
   add   esi,2
   dec   ecx
   jne   @XLoop
   add   edi,SkipDest
   add   esi,SkipSource
   dec   Ycount
   jne   @YLoop
@End:
   popad
End;
End;

Function HC_SpriteOverlapBlock;
Var
   x_overlap, y_overlap: boolean;
Begin
     Result:=false;
     If (x1 >= x2) And (x1 <= x2+sprite2.Width-1) Then x_overlap:=true Else
        If (x2 >= x1) And (x2 <= x1+sprite1.Width-1) Then x_overlap:=true Else
           x_overlap:=false;
     If (y1 >= y2) And (y1 <= y2+sprite2.Height-1) Then y_overlap:=true Else
        If (y2 >= y1) And (y2 <= y1+sprite1.Height-1) Then y_overlap:=true Else
           y_overlap:=false;
     Result:=x_overlap And y_overlap;
End;

Function HC_SpriteOverlap;
Var
   start_x_1, start_x_2, start_y_1, start_y_2: Longint;
   end_x_1, end_x_2, end_y_1, end_y_2: Longint;
   breite, hoehe: Longint;
   rest_x_1, rest_x_2: Longint;
Begin
     Result:=False;

     { Haben wir berhaupt eine berschneidung? }
     If HC_SpriteOverlapBlock(x1,y1,sprite1,x2,y2,sprite2) = False Then Exit;

     { Ja, also bestimmen wir berschneidungsbreich }

     { Fall 1: Erstes Sprite befindet sich in X-Richtung im 2. Sprite (links)}
     If ((x1>=x2) And (x1<=x2+sprite2.Width-1)) Then Begin
        start_x_1:=x1;
        start_x_2:=x1;
     { Fall 2: 2. Sprite befindet sich in X-Richtung im 1. Sprite (links)}
     End Else If ((x2>=x1) And (x2<=x1+sprite1.Width-1)) Then Begin
        start_x_1:=x2;
        start_x_2:=x2;
     End;

     { Fall 1: Erstes Sprite befindet sich in X-Richtung im 2. Sprite (rechts)}
     If ((x1+sprite1.Width-1>=x2) And (x1+sprite1.Width-1<=x2+sprite2.Width-1)) Then Begin
        end_x_1:=x1+sprite1.Width-1;
        end_x_2:=x1+sprite1.Width-1;
     { Fall 2: 2. Sprite befindet sich in X-Richtung im 1. Sprite (rechts)}
     End Else If ((x2+sprite2.Width-1>=x1) And (x2+sprite2.Width-1<=x1+sprite1.Width-1)) Then Begin
        end_x_1:=x2+sprite2.Width-1;
        end_x_2:=x2+sprite2.Width-1;
     End;

     { Fall 1: 1. Sprite befindet sich in Y-Richtung im 2. Sprite (oben)}
     If ((y1>=y2) And (y1<=y2+sprite2.Height-1)) Then Begin
        start_y_1:=y1;
        start_y_2:=y1;
     { Fall 2: 2. Sprite befindet sich in Y-Richtung im 1. Sprite (oben)}
     End Else If ((y2>=y1) And (y2<=y1+sprite1.Height-1)) Then Begin
        start_y_1:=y2;
        start_y_2:=y2;
     End;

     { Fall 1: 1. Sprite befindet sich in Y-Richtung im 2. Sprite (unten)}
     If ((y1+sprite1.Height-1>=y2) And (y1+sprite1.Height-1<=y2+sprite2.Height-1)) then begin
        end_y_1:=y1+sprite1.Height-1;
        end_y_2:=y1+sprite1.Height-1;
     { Fall 2: 2. Sprite befindet sich in Y-Richtung im 1. Sprite (unten)}
     End Else If ((y2+sprite2.Height-1>=y1) And (y2+sprite2.Height-1<=y1+sprite1.Height-1)) Then Begin
        end_y_1:=y2+sprite2.Height-1;
        end_y_2:=y2+sprite2.Height-1;
     End;

     { Wir wollen keine absoluten Koordinaten, sondern die innerhalb der
       Sprites }
     Dec(start_x_1,x1);
     Dec(start_x_2,x2);
     Dec(start_y_1,y1);
     Dec(start_y_2,y2);
     Dec(end_x_1,x1);
     Dec(end_x_2,x2);
     Dec(end_y_1,y1);
     Dec(end_y_2,y2);

     { Breite und Hhe des berschneidungsbereiches bestimmen }
     breite:=end_x_1-start_x_1;
     hoehe:=end_y_1-start_y_1;

     { Um diese Werte mssen die Offsets der Sprites nach jeder Zeile
       erhht werden }
     rest_x_1:=(sprite1.Width-breite)*2;
     rest_x_2:=(sprite2.Width-breite)*2;

     { Wenn der Bereich kleiner Null ist, liest der Rechner irgendwelchen
       Mll aus dem Speicher, das mu verhindert werden! }
     If (breite < 1) Or (hoehe < 1) Then Exit;

     Asm
        pushad
        mov    Result,0
        // Startadresse Sprite1
        mov    eax,DWord Ptr Sprite1.Width
        shl    eax,1
        mul    Start_Y_1
        mov    ebx,Start_x_1
        shl    ebx,1
        add    eax,ebx
        mov    edi,DWord Ptr Sprite1.Addr
        add    edi,eax
        // Startadresse Sprite2
        mov    eax,DWord Ptr Sprite2.Width
        shl    eax,1
        mul    Start_Y_2
        mov    ebx,Start_x_2
        shl    ebx,1
        add    eax,ebx
        mov    esi,DWord Ptr Sprite2.Addr
        add    esi,eax
        // berschneidungsbereich abklappern
        mov    edx,Hoehe
     @YLoop:
        mov    ecx,Breite
     @XLoop:
        mov    ax,[edi]
        and    ax,[esi]
        jnz    @Gotcha
        add    edi,2
        add    esi,2
        dec    ecx
        jne    @XLoop
        add    edi,Rest_x_1
        add    esi,Rest_x_2
        dec    edx
        jne    @YLoop
        jmp    @End
     @Gotcha:
        mov    Result,1
     @End:
        popad
     End;
End;

Initialization
     { Unit initialisieren }
     HC_Error:=HC_NoError;
     HC_Initialized:=False;
     HC_Width:=0;
     HC_Height:=0;
     HC_PWinGB:=Nil;
     HC_ScreenHandle:=0;
     HC_ShadowPal:=Nil;
Finalization
     { Aufrumen }
     HC_Done;
End.
