unit EditorMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus, HiColor, StdCtrls, Global, Buttons;

Const
  Title = 'QuadWorld Editor - ';
  StdFilename = 'Unbenannt.qwm';
  animRepeat = 0;
  animPingPong = 1;
  BlockSize = 64;
  MaxDefinedBlocks = 3;
  ToolsWidth = BlockSize+8;
  MaxTextures = 255;
  MaxSubTextures = 7;
  NoTextureAssigned = 65535;
  emSelect = 0;
  emDrop = 1;
  QWM_Str = 'QWM1';
  columnFaceNum = 12;

type
  TQWMHeader = Packed Record
     Str: Packed Array[0..3] Of Char;
     StartX, StartY: Integer;
     NumberOfTextures: Integer;
  End;

  TEditorBlock = Packed Record
     BlockType: Byte;
     Opacity: Byte;
     Flags: Word;
     FloorHeight: Single;
     CeilingHeight: Packed Array[0..3] Of Single;
     WaterHeight: Single;
     Textures: Packed Array[0..QW_MaxTexturesPerBlock] Of Word;
     TextureOffsets: Packed Array[0..QW_MaxTexturesPerBlock] Of smallInt;
  End;

  TEditorSubTexture = Packed Record
     FileName: String[12];
     Data: HC_Sprite;
  End;

  TEditorTexture = Packed Record
     Index: Integer; // Um den Umgang mit Listenfeldern zu erleichtern (wg. alphab. Sort.)
     Name: String[255];
     AnimationType: Integer;
     SubTextureNum: Integer;
     SubTextures: Array[0..MaxSubTextures] Of TEditorSubTexture;
  End;

  TClipBoard = Array[0..QW_MapWidth*QW_MapHeight-1] Of TEditorBlock;

  TFrmMain = class(TForm)
    MnuMain: TMainMenu;
    MnuFile: TMenuItem;
    FileNew: TMenuItem;
    FileOpen: TMenuItem;
    FileSave: TMenuItem;
    FileSaveAs: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    FileCompile: TMenuItem;
    FileExit: TMenuItem;
    MnuEdit: TMenuItem;
    EditCopy: TMenuItem;
    EditPaste: TMenuItem;
    MnuView: TMenuItem;
    ViewTextureEditor: TMenuItem;
    ViewProperties: TMenuItem;
    N3: TMenuItem;
    ViewFloor: TMenuItem;
    ViewCeiling: TMenuItem;
    PnlToolBar: TPanel;
    Bevel1: TBevel;
    ViewPort: TPaintBox;
    N4: TMenuItem;
    EditFill: TMenuItem;
    ScrHorizontal: TScrollBar;
    ScrVertical: TScrollBar;
    SpdOpen: TSpeedButton;
    SpdSave: TSpeedButton;
    SpdFill: TSpeedButton;
    SpdTextureEditor: TSpeedButton;
    SpdProperties: TSpeedButton;
    ViewWall: TMenuItem;
    ViewPedestalFloor: TMenuItem;
    ViewPedestalCeiling: TMenuItem;
    SpdSelect: TSpeedButton;
    Bevel2: TBevel;
    LblPos: TLabel;
    N5: TMenuItem;
    EditSetStartPos: TMenuItem;
    DlgSaveAs: TSaveDialog;
    DlgOpen: TOpenDialog;
    procedure FileExitClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ViewPortPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ViewPortMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ViewTextureEditorClick(Sender: TObject);
    procedure SpdTextureEditorClick(Sender: TObject);
    procedure ScrHorizontalChange(Sender: TObject);
    procedure ScrVerticalChange(Sender: TObject);
    procedure SpdSelectClick(Sender: TObject);
    procedure ViewPedestalCeilingClick(Sender: TObject);
    procedure ViewPropertiesClick(Sender: TObject);
    procedure SpdPropertiesClick(Sender: TObject);
    procedure MnuViewClick(Sender: TObject);
    procedure ViewPortMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure EditFillClick(Sender: TObject);
    procedure SpdFillClick(Sender: TObject);
    procedure EditCopyClick(Sender: TObject);
    procedure EditPasteClick(Sender: TObject);
    procedure MnuEditClick(Sender: TObject);
    procedure EditSetStartPosClick(Sender: TObject);
    procedure FileSaveAsClick(Sender: TObject);
    procedure FileSaveClick(Sender: TObject);
    procedure FileNewClick(Sender: TObject);
    procedure FileOpenClick(Sender: TObject);
    procedure FileCompileClick(Sender: TObject);
  private
    { Private-Deklarationen }
    fEditMode: Integer;
    fModified: Boolean;
    Procedure SetEditMode(Value: Integer);
  public
    { Public-Deklarationen }
    PlayerStartPos: TPoint;
    Map: Array[0..QW_MapWidth-1, 0..QW_MapHeight-1] Of TEditorBlock;
    Textures: Array[0..MaxTextures] Of TEditorTexture;
    TextureNum: Integer;
    DefinedBlocks: Array[0..MaxDefinedBlocks] Of Record
       Rect: TRect;
       Properties: TEditorBlock;
    End;
    BlockSprites: Array[0..QW_MaxDifferentBlocks] Of HC_Sprite;
    ActDefinedBlock: Integer;
    StartBlock: TPoint;
    SelectedBlocks: TRect;
    AppPath: String;
    Filename: String;
    ShowTexture: Integer;
    EndX, EndY: Integer;
    PreviousBlockAtMouse, BlockAtMouse: TPoint;
    ClipBoard: ^TClipBoard;
    ClipBoardSize: TPoint;
    Vertecies: Packed Array[0..QW_MaxVertex] Of TVertex;
    Polygons: Packed Array[0..QW_MaxPolygon] Of TPolygon;
    TexCoords: Packed Array[0..QW_MaxTexCoord] Of TTexCoord;
    TexCoordIndecies: Packed Array[0..QW_MaxIndex] Of Word;
    VertexIndecies: Packed Array[0..QW_MaxIndex] Of Word;
    VertexNum, PolygonNum, TexCoordNum, IndexNum: Integer;
    columnFace_XZ_U: packed array[0..columnFaceNum] of packed record
       x, z, u: single;
    end;
    Property EditMode: Integer Read fEditMode Write SetEditMode;
    Property Modified: Boolean Read fModified Write fModified;
    Procedure EraseBlock(Var Block: TEditorBlock);
    Procedure DrawBlock(x, y: Integer; Const Block: TEditorBlock);
    Procedure DrawViewPort;
    Function WriteToFile(Filename: String): Boolean;
    Function ReadFromFile(Filename: String): Boolean;
    Procedure EraseMap(EraseTextures: Boolean);
    Function Save: Boolean;
    Function SaveAs: Boolean;
    Function Open: Boolean;
    Procedure Compile;
    procedure calculateColumnFace_XZ_V(radius: single);
  end;

  Function Stri(i: Longint): String;
  Procedure Swap(Var i1, i2: Integer);
  Procedure UpdateAfterApplying;
  Procedure UpdateAfterApplyingToMap;

var
  FrmMain: TFrmMain;

implementation

uses EditorTextureEditorForm, EditorPropertiesForm, EditorSetStartPosForm,
  EditorNewForm, EditorCompileForm;

{$R *.DFM}

Function Stri(i: Longint): String;
Begin
     Str(i, Result);
End;

Procedure Swap;
Var
   Dummy: Integer;
Begin
     Dummy:=i1;
     i1:=i2;
     i2:=Dummy;
End;

procedure TFrmMain.calculateColumnFace_XZ_V;
var
   i: integer;
   angle, angleInc, u, uInc: single;
begin
     angleInc := (2*pi) / columnFaceNum;
     uInc := QW_Blocksize / columnFaceNum;
     angle := 0;
     u := QW_Blocksize;
     for i:=0 to columnFaceNum do begin
         columnFace_XZ_U[i].x := cos(angle) * radius + QW_Blocksize / 2;
         columnFace_XZ_U[i].z := sin(angle) * radius + QW_Blocksize / 2;
         columnFace_XZ_U[i].u := u;
         angle := angle + angleInc;
         u := u - uInc;
     end;
end;

Procedure TFrmMain.SetEditMode;
Begin
     fEditMode:=Value;
     SpdSelect.Down:=(fEditMode = emSelect);
     DrawViewPort;
End;

Procedure TFrmMain.EraseBlock;
Var
   i: Integer;
Begin
     With Block Do Begin
          BlockType:=QW_BlockFloorCeiling;
          Opacity:=128;
          Flags:=0;
          FloorHeight:=0;
          CeilingHeight[0]:=QW_BlockSize;
          CeilingHeight[1]:=QW_BlockSize;
          CeilingHeight[2]:=QW_BlockSize;
          CeilingHeight[3]:=QW_BlockSize;
          WaterHeight:=0;
          For i:=0 To QW_MaxTexturesPerBlock Do Block.Textures[i]:=NoTextureAssigned;
          For i:=0 To QW_MaxTexturesPerBlock Do Block.TextureOffsets[i]:=0;
     End;
End;

Procedure TFrmMain.EraseMap;
Var
   x, y: Integer;
   i, m: Integer;
Begin
     For y:=0 To QW_MapHeight-1 Do Begin
         For x:=0 To QW_MapWidth-1 Do EraseBlock(Map[x, y]);
     End;
     For i:=0 To MaxDefinedBlocks Do With DefinedBlocks[i] Do Begin
         Rect.Left:=3;
         Rect.Right:=Rect.Left+BlockSize-1;
         Rect.Top:=2+(i*(BlockSize+8));
         Rect.Bottom:=Rect.Top+BlockSize-1;
         EraseBlock(Properties);
     End;
     PlayerStartPos.x:=0;
     PlayerStartPos.y:=0;
     ActDefinedBlock:=0;
     StartBlock.x:=0;
     StartBlock.y:=0;
     SelectedBlocks.Left:=0;
     SelectedBlocks.Top:=0;
     SelectedBlocks.Right:=0;
     SelectedBlocks.Bottom:=0;
     If EraseTextures Then Begin
        For i:=0 To TextureNum - 1 Do Begin
            For m:=0 To Textures[i].SubTextureNum - 1 Do
                If Assigned(Textures[i].SubTextures[m].Data.Addr) Then
                   FreeMem(Textures[i].SubTextures[m].Data.Addr);
        End;
        TextureNum:=0;
        FillChar(Textures, SizeOf(Textures), #0);
     End;
     Filename:=StdFilename;
     Caption:=Title+StdFilename;
     fModified:=False;
End;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
     // Variablen initialisieren
     AppPath:=ExtractFilePath(ParamStr(0));
     FillChar(Textures, SizeOf(Textures), #0);
     TextureNum:=0;
     ScrHorizontal.Max:=QW_MapWidth-1;
     ScrVertical.Max:=QW_MapHeight-1;
     ShowTexture:=QW_TexWall;
     ViewWall.Checked:=True;
     fEditMode:=emSelect;
     SpdSelect.Down:=True;
     PreviousBlockAtMouse.x:=-1;
     PreviousBlockAtMouse.y:=-1;
     BlockAtMouse.x:=-1;
     BlockAtMouse.y:=-1;
     LblPos.Caption:='';
     ClipBoard:=Nil;
     ClipBoardSize.x:=0;
     ClipBoardSize.y:=0;
     EraseMap(False);
     calculateColumnFace_XZ_V(QW_Blocksize / (2 * pi)); // Damit der Umfang der Sule genau QW_Blocksize ist
     // Symbole einladen
     HC_Result;
     HC_CreateSprite(32, 32, BlockSprites[QW_BlockFloorCeiling]);
     HC_LoadPCX8Bit(AppPath+'QWEGfx\TypeWall.pcx', BlockSprites[QW_BlockWall]);
     HC_LoadPCX8Bit(AppPath+'QWEGfx\TypeColumn.pcx', BlockSprites[QW_BlockColumn]);
     HC_LoadPCX8Bit(AppPath+'QWEGfx\TypeWater.pcx', BlockSprites[QW_BlockWater]);
     HC_LoadPCX8Bit(AppPath+'QWEGfx\TypeWindowHoriz.pcx', BlockSprites[QW_BlockWindowHoriz]);
     HC_LoadPCX8Bit(AppPath+'QWEGfx\TypeWindowVert.pcx', BlockSprites[QW_BlockWindowVert]);
     If HC_Result <> HC_NoError Then Begin
        MessageDlg('Konnte ein oder mehrere Symbole nicht laden.', mtError, [mbOk], 0);
     End;
end;

procedure TFrmMain.FileExitClick(Sender: TObject);
begin
     Close;
end;

procedure TFrmMain.FormResize(Sender: TObject);
begin
     ScrHorizontal.Left:=ToolsWidth+2;
     ScrHorizontal.Top:=ClientHeight-ScrHorizontal.Height-2;
     ScrHorizontal.Width:=ClientWidth-ToolsWidth-4-ScrVertical.Width;
     ScrVertical.Left:=ClientWidth-ScrVertical.Width-2;
     ScrVertical.Top:=ViewPort.Top+2;
     ScrVertical.Height:=ClientHeight-ViewPort.Top-ScrHorizontal.Height-4;
     ScrHorizontal.LargeChange:=(ClientWidth-ToolsWidth-6) Div BlockSize - 1;
     ScrVertical.LargeChange:=(ClientHeight-ViewPort.Top-6) Div BlockSize - 1;
     HC_Done;
     HC_Init(ViewPort.Width, ViewPort.Height, ViewPort.Canvas.Handle);
     DrawViewPort;
end;

procedure TFrmMain.ViewPortPaint(Sender: TObject);
begin
     If HC_Initialized Then DrawViewPort;
end;

Procedure Draw3DLineTop(x1, x2, y: Integer);
Begin
     HC_Line(x1, y, x2, y, HC_MakeHiColor(15, 15, 15));
     HC_Line(x1+1, y+1, x2-1, y+1, 0);
End;

Procedure Draw3DLineBottom(x1, x2, y: Integer);
Begin
     HC_Line(x1+1, y, x2-1, y, HC_MakeHiColor(23, 23, 23));
     HC_Line(x1, y+1, x2, y+1, HC_MakeHiColor(31, 31, 31));
End;

Procedure Draw3DLineLeft(x, y1, y2: Integer);
Begin
     HC_Line(x, y1, x, y2, HC_MakeHiColor(15, 15, 15));
     HC_Line(x+1, y1+1, x+1, y2-1, 0);
End;

Procedure Draw3DLineRight(x, y1, y2: Integer);
Begin
     HC_Line(x+1, y1, x+1, y2, HC_MakeHiColor(31, 31, 31));
     HC_Line(x, y1+1, x, y2-1, HC_MakeHiColor(23, 23, 23));
End;

Procedure TFrmMain.DrawBlock;
Begin
     HC_FillRect(x, y, x+BlockSize-1, y+BlockSize-1, 0);
     If Block.Textures[ShowTexture] <> NoTextureAssigned Then With Textures[Block.Textures[ShowTexture]] Do Begin
        If SubTextureNum > 0 Then
           HC_PutSprite(x, y, SubTextures[0].Data);
     End;
     HC_PutSprite(x, y, BlockSprites[Block.BlockType]);
End;

Procedure TFrmMain.DrawViewPort;
Var
   i, m: Integer;
   x, y: Integer;
   SelX1, SelY1, SelX2, SelY2: Integer;
Begin
     HC_ScreenHandle:=ViewPort.Canvas.Handle;
     HC_ClearBuf(0);

     HC_SetWindow(0, 0, HC_Width-1, HC_Height-1);

     // Leiste mit Definierten Blcken zeichnen
     HC_FillRect(0, 0, ToolsWidth-1, HC_y2, HC_MakeHiColor(23, 23, 23));
     For i:=0 To MaxDefinedBlocks Do With DefinedBlocks[i] Do Begin
         DrawBlock(Rect.Left, Rect.Top, Properties);
         Draw3DLineTop(Rect.Left-2, Rect.Right+2, Rect.Top-2);
         Draw3DLineBottom(Rect.Left-2, Rect.Right+2, Rect.Bottom+2);
         Draw3DLineLeft(Rect.Left-2, Rect.Top-2, Rect.Bottom+2);
         Draw3DLineRight(Rect.Right+2, Rect.Top-2, Rect.Bottom+2);
         If i = ActDefinedBlock Then Begin
            HC_Rect(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom, HC_MakeHiColor(31, 0, 0));
            If EditMode = emDrop Then HC_Rect(Rect.Left+2, Rect.Top+2, Rect.Right-2, Rect.Bottom-2, HC_MakeHiColor(31, 0, 0));
         End;
     End;

     // Rand der Arbeitsflche zeichnen
     HC_FillRect(HC_x2-ScrVertical.Width-1, HC_y2-ScrHorizontal.Height-1, HC_x2, HC_y2, HC_MakeHiColor(23, 23, 23));
     Draw3DLineTop(ToolsWidth, HC_x2, 0);
     Draw3DLineBottom(ToolsWidth, HC_x2, HC_y2-1);
     Draw3DLineLeft(ToolsWidth, 0, HC_y2);
     Draw3DLineRight(HC_x2-1, 0, HC_y2);

     HC_SetWindow(ToolsWidth+1, 2, ViewPort.Width - ScrVertical.Width-3, ViewPort.Height - ScrHorizontal.Height - 3);

     // Arbeitsflche zeichnen
     EndX:=HC_WinWidth Div (BlockSize+1) + 1 + StartBlock.x;
     If EndX >= QW_MapWidth Then EndX:=QW_MapWidth - 1;
     EndY:=HC_WinHeight Div (BlockSize+1) + 1 + StartBlock.y;
     If EndY >= QW_MapHeight Then EndY:=QW_MapHeight - 1;
     y:=0;
     For m:=StartBlock.y To EndY Do Begin
         x:=0;
         For i:=StartBlock.x To EndX Do Begin
             HC_Rect(x * (BlockSize + 1), y * (BlockSize + 1), (x+1) * (BlockSize + 1), (y+1) * (BlockSize + 1), $FFFF);
             DrawBlock(x * (BlockSize + 1) + 1, y * (BlockSize + 1) + 1, Map[i, m]);
             Inc(x);
         End;
         Inc(y);
     End;

     // Ggf. Auswahlrahmen zeichnen
     SelX1:=SelectedBlocks.Left-StartBlock.x;
     SelY1:=SelectedBlocks.Top-StartBlock.y;
     SelX2:=SelectedBlocks.Right-StartBlock.x;
     SelY2:=SelectedBlocks.Bottom-StartBlock.y;
     If SelX1 > SelX2 Then Swap(SelX1, SelX2);
     If SelY1 > SelY2 Then Swap(SelY1, SelY2);

     Inc(SelX2);
     Inc(SelY2);

     HC_Rect(SelX1*(BlockSize+1), SelY1*(BlockSize+1),
            SelX2*(BlockSize+1), SelY2*(BlockSize+1), HC_MakeHiColor(31, 0, 0));
     If EditMode = emSelect Then HC_Rect(SelX1*(BlockSize+1)+2, SelY1*(BlockSize+1)+2,
                                         SelX2*(BlockSize+1)-2, SelY2*(BlockSize+1)-2,
                                         HC_MakeHiColor(31, 0, 0));

     HC_ShowBuf(1);
End;

procedure TFrmMain.ViewPortMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
Var
   i: Integer;
   a, b: Integer;
   aAbs, bAbs: Integer;
begin
     If (mbLeft In [Button]) Or (mbRight In [Button]) Then Begin
        For i:=0 To MaxDefinedBlocks Do Begin
            If (x >= DefinedBlocks[i].Rect.Left) And
               (y >= DefinedBlocks[i].Rect.Top) And
               (x <= DefinedBlocks[i].Rect.Right) And
               (y <= DefinedBlocks[i].Rect.Bottom) Then Begin
                  ActDefinedBlock:=i;
                  EditMode:=emDrop;
                  FrmProperties.UpdateProperties(DefinedBlocks[i].Properties, @UpdateAfterApplying);
                  If mbRight In [Button] Then FrmProperties.Show;
                  Exit;
               End;
        End;
     End;

     If (EditMode = emDrop) And (mbRight In [Button]) Then Begin
        If (x >= HC_x1) And
           (y >= HC_y1) And
           (x <= HC_x2) And
           (y <= HC_y2) Then
        Begin
           EditMode:=emSelect;
           DrawViewPort;
           Exit;
        End;
     End;

     b:=0;
     For bAbs:=StartBlock.y To EndY Do Begin
         a:=0;
         For aAbs:=StartBlock.x To EndX Do Begin
             If (x >= a*(BlockSize+1)+HC_x1) And
                (y >= b*(BlockSize+1)+HC_y1) And
                (x < (a+1)*(BlockSize+1)+HC_x1) And
                (y < (b+1)*(BlockSize+1)+HC_y1) Then
             Begin
                If (mbLeft In [Button]) And (ssShift In Shift) Then Begin
                   DefinedBlocks[ActDefinedBlock].Properties:=Map[aAbs, bAbs];
                   FrmProperties.UpdateProperties(Map[aAbs, bAbs], @UpdateAfterApplyingToMap);
                End Else If (EditMode = emDrop) And (mbLeft In [Button]) Then Begin
                   Map[aAbs, bAbs] := DefinedBlocks[ActDefinedBlock].Properties;
                   Modified:=True;
                   FrmProperties.UpdateProperties(Map[aAbs, bAbs], @UpdateAfterApplyingToMap);
                End Else If (EditMode = emSelect) And (mbLeft In [Button]) Then Begin
                   SelectedBlocks.Left:=aAbs;
                   SelectedBlocks.Top:=bAbs;
                   SelectedBlocks.Right:=aAbs;
                   SelectedBlocks.Bottom:=bAbs;
                   FrmProperties.UpdateProperties(Map[aAbs, bAbs], @UpdateAfterApplyingToMap);
                End Else If (EditMode = emSelect) And (mbRight In [Button]) Then Begin
                   SelectedBlocks.Left:=aAbs;
                   SelectedBlocks.Top:=bAbs;
                   SelectedBlocks.Right:=aAbs;
                   SelectedBlocks.Bottom:=bAbs;
                   FrmProperties.UpdateProperties(Map[aAbs, bAbs], @UpdateAfterApplyingToMap);
                   FrmProperties.Show;
                End;
                DrawViewPort;
                Exit;
             End;
             Inc(a);
         End;
         Inc(b);
     End;
end;

procedure TFrmMain.ViewPortMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
Var
   a, b: Integer;
begin
     a:=(x - HC_x1) Div (BlockSize + 1) + StartBlock.x;
     b:=(y - HC_y1) Div (BlockSize + 1) + StartBlock.y;
     If (a <> BlockAtMouse.x) Or (b <> BlockAtMouse.y) Then Begin
        PreviousBlockAtMouse:=BlockAtMouse;
        BlockAtMouse.x:=a;
        BlockAtMouse.y:=b;
        If (x >= HC_x1) And (y >= HC_y1) And (x <= HC_x2) And (y <= HC_y2) Then
           LblPos.Caption:=Stri(a)+': '+Stri(b)
        Else
           LblPos.Caption:='';
        If (EditMode = emSelect) And (ssLeft In Shift) Then Begin
           SelectedBlocks.Right:=a;
           SelectedBlocks.Bottom:=b;
           DrawViewPort;
        End;
     End;
end;

procedure TFrmMain.ViewTextureEditorClick(Sender: TObject);
begin
     FrmTextureEditor.ShowModal;
end;

procedure TFrmMain.SpdTextureEditorClick(Sender: TObject);
begin
     ViewTextureEditorClick(Nil);
end;

procedure TFrmMain.ScrHorizontalChange(Sender: TObject);
begin
     StartBlock.x:=ScrHorizontal.Position;
     DrawViewPort;
end;

procedure TFrmMain.ScrVerticalChange(Sender: TObject);
begin
     StartBlock.y:=ScrVertical.Position;
     DrawViewPort;
end;

Procedure UpdateAfterApplying;
Begin
     FrmMain.DrawViewPort;
End;

Procedure UpdateAfterApplyingToMap;
Begin
     FrmMain.Modified:=True;
     FrmMain.DrawViewPort;
End;

procedure TFrmMain.SpdSelectClick(Sender: TObject);
begin
     If SpdSelect.Down Then EditMode:=emSelect
     Else EditMode:=emDrop;
end;

procedure TFrmMain.ViewPedestalCeilingClick(Sender: TObject);
begin
     (Sender As TMenuItem).Checked:=True;
     ShowTexture:=(Sender As TMenuItem).Tag;
     DrawViewPort;
end;

procedure TFrmMain.ViewPropertiesClick(Sender: TObject);
begin
     If FrmProperties.Visible Then
        FrmProperties.Close
     Else
        FrmProperties.Show;
end;

procedure TFrmMain.SpdPropertiesClick(Sender: TObject);
begin
     If SpdProperties.Down Then
        FrmProperties.Show
     Else
        FrmProperties.Close;

end;

procedure TFrmMain.MnuViewClick(Sender: TObject);
begin
     ViewProperties.Checked:=FrmProperties.Visible;
end;

procedure TFrmMain.EditFillClick(Sender: TObject);
Var
   x, y: Integer;
   StartX, StartY, EndX, EndY: Integer;
begin
     StartX:=SelectedBlocks.Left;
     StartY:=SelectedBlocks.Top;
     EndX:=SelectedBlocks.Right;
     EndY:=SelectedBlocks.Bottom;
     If StartX > EndX Then Swap(StartX, EndX);
     If StartY > EndY Then Swap(StartY, EndY);
     For x:=StartX To EndX Do Begin
         For y:=StartY To EndY Do Begin
             Map[x, y]:=DefinedBlocks[ActDefinedBlock].Properties;
         End;
     End;
     Modified:=True;
     DrawViewPort;
end;

procedure TFrmMain.SpdFillClick(Sender: TObject);
begin
     EditFillClick(Nil);
end;

procedure TFrmMain.EditCopyClick(Sender: TObject);
Var
   x, y: Integer;
   StartX, StartY, EndX, EndY: Integer;
   i: Integer;
begin
     If Assigned(ClipBoard) Then FreeMem(ClipBoard);
     StartX:=SelectedBlocks.Left;
     StartY:=SelectedBlocks.Top;
     EndX:=SelectedBlocks.Right;
     EndY:=SelectedBlocks.Bottom;
     If StartX > EndX Then Swap(StartX, EndX);
     If StartY > EndY Then Swap(StartY, EndY);
     ClipBoardSize.x:=EndX-StartX+1;
     ClipBoardSize.y:=EndY-StartY+1;
     GetMem(ClipBoard, ClipBoardSize.x * ClipBoardSize.y * SizeOf(TEditorBlock));
     i:=0;
     For x:=StartX To EndX Do Begin
         For y:=StartY To EndY Do Begin
             ClipBoard^[i]:=Map[x, y];
             Inc(i);
         End;
     End;
end;

procedure TFrmMain.EditPasteClick(Sender: TObject);
Var
   x, y, i: Integer;
   StartX, StartY, EndX, EndY: Integer;
begin
     If Not Assigned(ClipBoard) Then Exit;
     If (ClipBoardSize.x <= 0) Or (ClipBoardSize.y <= 0) Then Exit;
     StartX:=SelectedBlocks.Left;
     StartY:=SelectedBlocks.Top;
     EndX:=SelectedBlocks.Right;
     EndY:=SelectedBlocks.Bottom;
     If StartX > EndX Then Swap(StartX, EndX);
     If StartY > EndY Then Swap(StartY, EndY);
     EndX:=StartX+ClipBoardSize.x-1;
     EndY:=StartY+ClipBoardSize.y-1;
     If EndX >= QW_MapWidth Then EndX:=QW_MapWidth-1;
     If EndY >= QW_MapHeight Then EndY:=QW_MapHeight-1;
     i:=0;
     For x:=StartX To EndX Do Begin
         For y:=StartY To EndY Do Begin
             Map[x, y]:=ClipBoard^[i];
             Inc(i);
         End;
     End;
     Modified:=True;
     SelectedBlocks.Left:=StartX;
     SelectedBlocks.Top:=StartY;
     SelectedBlocks.Right:=EndX;
     SelectedBlocks.Bottom:=EndY;
     DrawViewPort;
end;

procedure TFrmMain.MnuEditClick(Sender: TObject);
begin
     EditPaste.Enabled:=Assigned(ClipBoard);
end;

procedure TFrmMain.EditSetStartPosClick(Sender: TObject);
begin
     FrmSetStartPos.SpnPosX.Value:=PlayerStartPos.x;
     FrmSetStartPos.SpnPosY.Value:=PlayerStartPos.y;
     If FrmSetStartPos.ShowModal = mrOk Then Begin
        PlayerStartPos.x:=FrmSetStartPos.SpnPosX.Value;
        PlayerStartPos.y:=FrmSetStartPos.SpnPosY.Value;
     End;
end;

Function TFrmMain.WriteToFile;
Var
   f: File;
   i, m: Integer;
   Header: TQWMHeader;
Begin
     // Header vorbereiten
     For i:=0 To 3 Do Header.Str[i]:=QWM_Str[i];
     Header.StartX:=PlayerStartPos.x;
     Header.StartY:=PlayerStartPos.y;
     Header.NumberOfTextures:=TextureNum;

     // Datei erstellen
     IOResult;
     System.Assign(f, Filename);
     ReWrite(f, 1);
     If IOResult <> 0 Then Begin
        MessageDlg('Konnte '+Filename+' nicht schreiben.', mtError, [mbOk], 0);
        Result:=False;
        Exit;
     End;

     // Header schreiben
     BlockWrite(f, Header, SizeOf(Header));

     // Texturen schreiben
     For i:=0 To TextureNum - 1 Do Begin
         BlockWrite(f, Textures[i].Name, SizeOf(Textures[i].Name));
         BlockWrite(f, Textures[i].AnimationType, SizeOf(Textures[i].AnimationType));
         BlockWrite(f, Textures[i].SubTextureNum, SizeOf(Textures[i].SubTextureNum));
         For m:=0 To Textures[i].SubTextureNum - 1 Do
             BlockWrite(f, Textures[i].SubTextures[m].FileName, SizeOf(Textures[i].SubTextures[m].Filename));
     End;

     // Map-Daten schreiben
     BlockWrite(f, Map, SizeOf(Map));

     // Datei schlieen
     System.Close(f);
     If IOResult <> 0 Then Begin
        MessageDlg('Fehler beim Schreiben von '+Filename+'.', mtError, [mbOk], 0);
        Result:=False;
        Exit;
     End;

     // Wenn er hier ankommt, sollte alles geklappt haben
     Result:=True;
     Modified:=False;
End;

procedure TFrmMain.FileSaveAsClick(Sender: TObject);
begin
     SaveAs;
end;

Function TFrmMain.SaveAs;
Begin
     DlgSaveAs.InitialDir:=AppPath+QW_MapPath;
     If DlgSaveAs.Execute Then Begin
        Result:=WriteToFile(DlgSaveAs.Filename);
        If Result Then Begin
           Filename:=DlgSaveAs.Filename;
           Caption:=Title+Filename;
        End;
     End Else Result:=False;
End;

Function TFrmMain.Save;
Begin
     If Filename = StdFilename Then
        Result:=SaveAs
     Else
        Result:=WriteToFile(Filename);
End;

procedure TFrmMain.FileSaveClick(Sender: TObject);
begin
     Save;
end;

procedure TFrmMain.FileNewClick(Sender: TObject);
Var
   Res: Integer;
begin
     If FrmNew.ShowModal = mrOK Then Begin
        If Modified Then Begin
           Res:=MessageDlg('Die letzten nderungen wurden noch nicht gespeichert. Jetzt sichern?', mtWarning, [mbYes, mbNo, mbCancel], 0);
           Case Res Of
                mrYes: If Not Save Then Exit;
                mrCancel: Exit;
           End;
        End;
        If FrmNew.RdoEraseAll.Checked Then
           EraseMap(True)
        Else If FrmNew.RdoTakeOverTextures.Checked Then
           EraseMap(False);
        FrmProperties.Hide;
        DrawViewPort;
     End;
end;

Function TFrmMain.Open;
Var
   Res: Integer;
Begin
     DlgOpen.InitialDir:=AppPath+QW_MapPath;
     If DlgOpen.Execute Then Begin
        Result:=False;
        If Modified Then Begin
           Res:=MessageDlg('Die letzten nderungen wurden noch nicht gespeichert. Jetzt sichern?', mtWarning, [mbYes, mbNo, mbCancel], 0);
           Case Res Of
                mrYes: If Not Save Then Exit;
                mrCancel: Exit;
           End;
        End;
        Result:=ReadFromFile(DlgOpen.Filename);
        If Result Then Begin
           Caption:=Title+DlgOpen.Filename;
           Filename:=DlgOpen.Filename;
        End Else Begin
           Caption:=Title+StdFilename;
           Filename:=StdFilename;
        End;
     End Else Result:=False;
End;

procedure TFrmMain.FileOpenClick(Sender: TObject);
begin
     Open;
end;

Function TFrmMain.ReadFromFile;
Var
   f: File;
   Header: TQWMHeader;
   i, m: Integer;
   Dummy: HC_Sprite;
   NewWidth, NewHeight: Integer;
Begin
     // Alte Daten lschen
     FrmProperties.Hide;
     EraseMap(True);
     DrawViewPort;

     // Datei ffnen
     IOResult;
     System.Assign(f, Filename);
     Reset(f, 1);
     If IOResult <> 0 Then Begin
        Result:=False;
        Exit;
     End;

     // Header einlesen
     FillChar(Header, SizeOf(Header), #0);
     BlockRead(f, Header, SizeOf(Header));
     For i:=0 To 3 Do If Header.Str[i] <> QWM_Str[i] Then Begin
         MessageDlg(Filename+' ist keine QWM-Datei.', mtError, [mbOk], 0);
         System.Close(f);
         Result:=False;
         Exit;
     End;

     PlayerStartPos.x := header.StartX;
     PlayerStartPos.y := header.StartY;

     // Texturen einlesen
     For i:=0 To Header.NumberOfTextures - 1 Do Begin
         BlockRead(f, Textures[i].Name, SizeOf(Textures[i].Name));
         BlockRead(f, Textures[i].AnimationType, SizeOf(Textures[i].AnimationType));
         BlockRead(f, Textures[i].SubTextureNum, SizeOf(Textures[i].SubTextureNum));
         Textures[i].Index:=i;

         // Untertexturen lesen
         For m:=0 To Textures[i].SubTextureNum - 1 Do Begin
             BlockRead(f, Textures[i].SubTextures[m].Filename, SizeOf(Textures[i].SubTextures[m].Filename));
             HC_Result;
             HC_LoadPCX8Bit(AppPath+QW_TexturePath+Textures[i].SubTextures[m].Filename, Dummy);
             If HC_Result <> HC_NoError Then Begin
                MessageDlg('Fehler beim Laden von "'+AppPath+QW_TexturePath+Textures[i].SubTextures[m].Filename+'".', mtError, [mbOk], 0);
                EraseMap(True);
                Result:=False;
                System.Close(f);
                IOResult;
                Exit;
             End;
             If Dummy.Width >= Dummy.Height Then Begin
                NewWidth:=BlockSize;
                NewHeight:=Trunc((BlockSize / Dummy.Width) * Dummy.Height);
             End Else Begin
                NewWidth:=Trunc((BlockSize / Dummy.Height) * Dummy.Width);
                NewHeight:=BlockSize;
             End;
             CreateStretchedSprite(Textures[i].SubTextures[m].Data, NewWidth,
                                   NewHeight, Dummy);
             HC_FreeSprite(Dummy);
         End;
     End;
     TextureNum:=Header.NumberOfTextures;

     // Map-Daten einlesen
     BlockRead(f, Map, SizeOf(Map));
     If IOResult <> 0 Then Begin
        MessageDlg('Fehler beim Einlesen von '+Filename+'.', mtError, [mbOk], 0);
        EraseMap(True);
        Result:=False;
        System.Close(f);
        IOResult;
        Exit;
     End;

     // Datei schlieen
     System.Close(f);
     Result:=True;
     DrawViewPort;
     Modified:=False;
End;

procedure TFrmMain.FileCompileClick(Sender: TObject);
begin
     Compile;
end;

Procedure TFrmMain.Compile;
Type
  TVert = Record
     x, y, z: Single;
     u, v: Single;
  End;
Var
   OutPutFile: String;
   f: File;
   Header: TQCMHeader;
   CompiledMap: Packed Array[0..QW_MapWidth-1, 0..QW_MapHeight-1] Of TBlock;
   xStart, zStart: Integer;
   a, b, i, m: Integer;
   Dummy: String;
   Vert: Array[0..11] Of TVert;

Function IsValidBlock(a, b: Integer): Boolean;
Begin
     Result:=((a >= 0) And (a < QW_MapWidth) And (b >= 0) And (a < QW_Mapheight));
End;

Function AddVertex(x, y, z: Single; a, b: Integer): Integer;
Var
   i, m: Integer;
Begin
     If IsValidBlock(a-1, b-1) Then Begin
        For i:=CompiledMap[a-1, b-1].FirstPolygon To
               CompiledMap[a-1, b-1].FirstPolygon +
               CompiledMap[a-1, b-1].PolygonNum-1 Do
        Begin
             For m:=Polygons[i].FirstIndex To Polygons[i].FirstIndex+Polygons[i].IndexNum-1 Do Begin
                 If (Vertecies[VertexIndecies[m]].x = x) And
                    (Vertecies[VertexIndecies[m]].y = y) And
                    (Vertecies[VertexIndecies[m]].z = z) {And
                    (Vertecies[VertexIndecies[m]].Alpha = Alpha)} Then Begin
                       Result:=VertexIndecies[m];
                       Exit;
                    End;
             End;
        End;
     End;
     If IsValidBlock(a, b-1) Then Begin
        For i:=CompiledMap[a, b-1].FirstPolygon To
               CompiledMap[a, b-1].FirstPolygon +
               CompiledMap[a, b-1].PolygonNum-1 Do
        Begin
             For m:=Polygons[i].FirstIndex To Polygons[i].FirstIndex+Polygons[i].IndexNum-1 Do Begin
                 If (Vertecies[VertexIndecies[m]].x = x) And
                    (Vertecies[VertexIndecies[m]].y = y) And
                    (Vertecies[VertexIndecies[m]].z = z) {And
                    (Vertecies[VertexIndecies[m]].Alpha = Alpha)} Then Begin
                       Result:=VertexIndecies[m];
                       Exit;
                    End;
             End;
        End;
     End;
     If IsValidBlock(a+1, b-1) Then Begin
        For i:=CompiledMap[a+1, b-1].FirstPolygon To
               CompiledMap[a+1, b-1].FirstPolygon +
               CompiledMap[a+1, b-1].PolygonNum-1 Do
        Begin
             For m:=Polygons[i].FirstIndex To Polygons[i].FirstIndex+Polygons[i].IndexNum-1 Do Begin
                 If (Vertecies[VertexIndecies[m]].x = x) And
                    (Vertecies[VertexIndecies[m]].y = y) And
                    (Vertecies[VertexIndecies[m]].z = z){ And
                    (Vertecies[VertexIndecies[m]].Alpha = Alpha)} Then Begin
                       Result:=VertexIndecies[m];
                       Exit;
                    End;
             End;
        End;
     End;
     If IsValidBlock(a-1, b) Then Begin
        For i:=CompiledMap[a-1, b].FirstPolygon To
               CompiledMap[a-1, b].FirstPolygon +
               CompiledMap[a-1, b].PolygonNum-1 Do
        Begin
             For m:=Polygons[i].FirstIndex To Polygons[i].FirstIndex+Polygons[i].IndexNum-1 Do Begin
                 If (Vertecies[VertexIndecies[m]].x = x) And
                    (Vertecies[VertexIndecies[m]].y = y) And
                    (Vertecies[VertexIndecies[m]].z = z) {And
                    (Vertecies[VertexIndecies[m]].Alpha = Alpha)} Then Begin
                       Result:=VertexIndecies[m];
                       Exit;
                    End;
             End;
        End;
     End;
     If IsValidBlock(a, b) Then Begin
        For i:=CompiledMap[a, b].FirstPolygon To
               CompiledMap[a, b].FirstPolygon +
               CompiledMap[a, b].PolygonNum-1 Do
        Begin
             For m:=Polygons[i].FirstIndex To Polygons[i].FirstIndex+Polygons[i].IndexNum-1 Do Begin
                 If (Vertecies[VertexIndecies[m]].x = x) And
                    (Vertecies[VertexIndecies[m]].y = y) And
                    (Vertecies[VertexIndecies[m]].z = z) { And
                    (Vertecies[VertexIndecies[m]].Alpha = Alpha)} Then Begin
                       Result:=VertexIndecies[m];
                       Exit;
                    End;
             End;
        End;
     End;

     Vertecies[VertexNum].x:=x;
     Vertecies[VertexNum].y:=y;
     Vertecies[VertexNum].z:=z;
//     Vertecies[VertexNum].Alpha:=Alpha;

     // Das hier ist nur vorlufig...
     Vertecies[VertexNum].r:=255;
     Vertecies[VertexNum].g:=255;
     Vertecies[VertexNum].b:=255;
     Vertecies[VertexNum].Overall:=255;
//     FillChar(Vertecies[VertexNum].Normal, SizeOf(TNormal), #0);

     Result:=VertexNum;
     Inc(VertexNum);
End;

Function AddTexCoord(u, v: Single): Integer;
Var
   i: Integer;
Begin
     For i:=0 To TexCoordNum - 1 Do Begin
         If (TexCoords[i].u = u) And (TexCoords[i].v = v) Then Begin
            Result:=i;
            Exit;
         End;
     End;
     TexCoords[TexCoordNum].u:=u;
     TexCoords[TexCoordNum].v:=v;
     Result:=TexCoordNum;
     Inc(TexCoordNum);
End;

Function AddPolygon(Vert: Array Of TVert; Texture, VertNum, a, b: Integer; Opacity: Byte): Integer;
Var
   i: Integer;
Begin
     If VertNum <= 0 Then Begin
        Result:=0;
        Exit;
     End;
     Polygons[PolygonNum].FirstIndex:=IndexNum;
     Polygons[PolygonNum].IndexNum:=0;
     Polygons[PolygonNum].Texture:=Texture;
     Polygons[PolygonNum].Opacity:=Opacity;
     If Opacity <> 255 then Polygons[PolygonNum].IsUsed:=1 else Polygons[PolygonNum].IsUsed:=0;
     For i:=0 To VertNum - 1 Do Begin
         VertexIndecies[IndexNum]:=AddVertex(Vert[i].x, Vert[i].y, Vert[i].z, a, b);
         TexCoordIndecies[IndexNum]:=AddTexCoord(Vert[i].u, Vert[i].v);
         Inc(IndexNum);
     End;
     Polygons[PolygonNum].IndexNum:=VertNum;
     if (compiledMap[a, b].FirstPolygon = 0) and (polygonNum > 0) then compiledMap[a, b].FirstPolygon := PolygonNum;
     Result:=PolygonNum;
     Inc(PolygonNum);
End;

function wallHitTest(a, b: integer): boolean;
begin
     result := FALSE;
     if ((a < 0) or (b < 0) or (a > QW_MapWidth-1) or (b > QW_MapWidth-1)) then exit;
     if map[a, b].BlockType = QW_BlockWall then exit;
     if map[a, b].Textures[QW_TexFloor] = NoTextureAssigned then exit;
     result := TRUE;
end;

function floorHitTest(a, b, deltaA, deltaB: integer): boolean;
begin
     result := FALSE;
     if ((a+deltaA < 0) or (b+deltaB < 0) or (a+deltaA > QW_MapWidth-1) or (b+deltaB > QW_MapWidth-1)) then exit;
     if map[a+deltaA, b+deltaB].BlockType = QW_BlockWall then exit;
     if map[a+deltaA, b+deltaB].FloorHeight >= map[a, b].floorHeight then exit;
     if map[a+deltaA, b+deltaB].Textures[QW_TexFloor] = NoTextureAssigned then exit;
     result := TRUE;
end;

function ceilingHitTest(a, b, deltaA, deltaB: integer): boolean;
var
   x, y: integer;
begin
     result := FALSE;

     x := a + deltaA;
     y := b + deltaB;

     if ((x < 0) or (y < 0) or (x > QW_MapWidth-1) or (y > QW_MapWidth-1)) then exit;
     if map[a+deltaA, b+deltaB].BlockType = QW_BlockWall then exit;

     if ((deltaA = -1) and (deltaB = 0)) then begin
        if (map[x, y].ceilingHeight[1] <= map[a, b].ceilingHeight[0]) and (map[x, y].ceilingHeight[2] <= map[a, b].ceilingHeight[3]) then exit;

     end else if ((deltaA = 1) and (deltaB = 0)) then begin
        if (map[x, y].ceilingHeight[0] <= map[a, b].ceilingHeight[1]) and (map[x, y].ceilingHeight[3] <= map[a, b].ceilingHeight[2]) then exit;

     end else if ((deltaA = 0) and (deltaB = -1)) then begin
        if (map[x, y].ceilingHeight[3] <= map[a, b].ceilingHeight[0]) and (map[x, y].ceilingHeight[2] <= map[a, b].ceilingHeight[1]) then exit;

     end else if ((deltaA = 0) and (deltaB = 1)) then begin
        if (map[x, y].ceilingHeight[0] <= map[a, b].ceilingHeight[3]) and (map[x, y].ceilingHeight[1] <= map[a, b].ceilingHeight[2]) then exit;

     end else exit;

     result := TRUE;
end;

function calculateTextureV(y: single; a, b: integer): single;
begin
{     if (round(y) mod QW_BlockSize) = 0 then
        result:=(y / QW_BlockSize) * QW_Blocksize - y + map[a, b].textureOffsets[QW_TexWall]
     else}
        result:={(trunc(y / QW_BlockSize)+1) * QW_Blocksize -} - y + map[a, b].textureOffsets[QW_TexWall];
//        result := round(result) mod QW_Blocksize;
end;

Begin
     // Datei speichern
     If Not Save Then Begin
        MessageDlg('Die Compilierung kann nicht durchgefhrt werden, bevor die Daten gespeichert wurden.', mtError, [mbOk], 0);
        Exit;
     End;

     // Dateiendung durch "qcm" ersetzen
     For i:=Length(Filename) DownTo 1 Do Begin
         If Filename[i] = '\' Then Begin
            OutputFile:=Filename+'.qcm';
            Break;
         End Else If Filename[i] = '.' Then Begin
            OutputFile:=Copy(Filename, 1, i)+'qcm';
            Break;
         End;
     End;

     // Statusfenster anzeigen
     FrmCompile.Clear;
     FrmCompile.Show;
     FrmCompile.Add('QuadWorld Compiler v1.0');
     FrmCompile.Add('');

     Try
        // Anzeige aktualisieren
        Screen.Cursor:=crHourGlass;
        FrmCompile.Add('Erstelle 3D-Modell...');

        // Varibalen initialisieren
        FillChar(CompiledMap, SizeOf(CompiledMap), #0);
        VertexNum:=0;
        PolygonNum:=0;
        TexCoordNum:=0;
        IndexNum:=0;

        // Alle Blcke abarbeiten
        zStart:=0;
        For b:=0 To QW_MapHeight - 1 Do Begin
            xStart:=QW_MapWidth*QW_BlockSize;
            For a:=0 To QW_MapWidth - 1 Do Begin

                // Werte fr den aktuellen (compilierten) Block setzen
                CompiledMap[a, b].BlockType:=Map[a, b].BlockType;
                CompiledMap[a, b].Flags:=Map[a, b].Flags;
                CompiledMap[a, b].Opacity:=Map[a, b].Opacity;
                CompiledMap[a, b].FirstPolygon:=0;
                CompiledMap[a, b].PolygonNum:=0;
                CompiledMap[a, b].FloorHeight:=Map[a, b].FloorHeight;
                CompiledMap[a, b].WaterHeight:=Map[a, b].WaterHeight;
                CompiledMap[a, b].MinCeilingHeight:=Map[a, b].CeilingHeight[0];
                For i:=1 To 3 Do
                    If Map[a, b].CeilingHeight[i] < CompiledMap[a, b].MinCeilingHeight Then
                       CompiledMap[a, b].MinCeilingHeight:=Map[a, b].CeilingHeight[i];

                // 3D-Daten fr den Block erstellen
                //
                // Eckpunkte im Uhrzeigersinn (vom Betrachter aus gesehen)
                //
                // 0    1
                //   []
                // 3    2
                //
                // "Left-Handed"-Koordinatensystem
                //
                //   Y^
                //    |     ^ Z
                //    |    /
                //    |  /
                //    |/
                //    ------->X
                if map[a, b].BlockType = QW_BlockWall then begin
                   If Map[a, b].Textures[QW_TexWall] <> NoTextureAssigned Then Begin
                      // zum Betrachter
                      if wallHitTest(a, b-1) then begin
                         Vert[0].x:=xStart-QW_BlockSize;
                         Vert[0].y:=map[a, b-1].CeilingHeight[2];
                         Vert[0].z:=zStart;
                         Vert[0].u:=0;
//                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];
                         Vert[0].v:=calculateTextureV(Vert[0].y, a, b);

                         Vert[1].x:=xStart;
                         Vert[1].y:=Map[a, b-1].CeilingHeight[3];
                         Vert[1].z:=zStart;
                         Vert[1].u:=QW_BlockSize;
//                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;
                         Vert[1].v:=calculateTextureV(Vert[1].y, a, b);

                         Vert[2].x:=xStart;
                         Vert[2].y:=Map[a, b-1].FloorHeight;
                         Vert[2].z:=zStart;
                         Vert[2].u:=QW_BlockSize;
//                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart-QW_BlockSize;
                         Vert[3].y:=Map[a, b-1].FloorHeight;
                         Vert[3].z:=zStart;
                         Vert[3].u:=0;
  //                       Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         Vert[2].v:=vert[1].v + (vert[1].y - vert[2].y);
                         Vert[3].v:=vert[0].v + (vert[0].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // vom Betrachter weg
                      if wallHitTest(a, b+1) then begin
                         Vert[0].x:=xStart;
                         Vert[0].y:=Map[a, b+1].CeilingHeight[0];
                         Vert[0].z:=zStart+QW_BlockSize;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart-QW_BlockSize;
                         Vert[1].y:=map[a, b+1].CeilingHeight[1];
                         Vert[1].z:=zStart+QW_BlockSize;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart-QW_BlockSize;
                         Vert[2].y:=Map[a, b+1].FloorHeight;
                         Vert[2].z:=zStart+QW_BlockSize;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart;
                         Vert[3].y:=Map[a, b+1].FloorHeight;
                         Vert[3].z:=zStart+QW_BlockSize;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // rechts vom Betrachter
                      if wallHitTest(a-1, b) then begin
                         Vert[0].x:=xStart;
                         Vert[0].y:=Map[a-1, b].CeilingHeight[1];
                         Vert[0].z:=zStart;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart;
                         Vert[1].y:=map[a-1, b].CeilingHeight[2];
                         Vert[1].z:=zStart+QW_BlockSize;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart;
                         Vert[2].y:=Map[a-1, b].FloorHeight;
                         Vert[2].z:=zStart+QW_BlockSize;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart;
                         Vert[3].y:=Map[a-1, b].FloorHeight;
                         Vert[3].z:=zStart;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // links vom Betrachter
                      if wallHitTest(a+1, b) then begin
                         Vert[0].x:=xStart-QW_Blocksize;
                         Vert[0].y:=map[a+1, b].CeilingHeight[3];
                         Vert[0].z:=zStart+QW_BlockSize;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart-QW_Blocksize;
                         Vert[1].y:=Map[a+1, b].CeilingHeight[0];
                         Vert[1].z:=zStart;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart-QW_Blocksize;
                         Vert[2].y:=Map[a+1, b].FloorHeight;
                         Vert[2].z:=zStart;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart-QW_Blocksize;
                         Vert[3].y:=Map[a+1, b].FloorHeight;
                         Vert[3].z:=zStart+QW_BlockSize;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                   End;
                end else begin
                   // Boden
                   If Map[a, b].Textures[QW_TexFloor] <> NoTextureAssigned Then Begin
                      Vert[0].x:=xStart-QW_BlockSize;
                      Vert[0].y:=Map[a, b].FloorHeight;
                      Vert[0].z:=zStart+QW_BlockSize;
                      Vert[0].u:=QW_BlockSize;
                      Vert[0].v:=QW_BlockSize;

                      Vert[1].x:=xStart;
                      Vert[1].y:=Map[a, b].FloorHeight;
                      Vert[1].z:=zStart+QW_BlockSize;
                      Vert[1].u:=0;
                      Vert[1].v:=QW_BlockSize;

                      Vert[2].x:=xStart;
                      Vert[2].y:=Map[a, b].FloorHeight;
                      Vert[2].z:=zStart;
                      Vert[2].u:=0;
                      Vert[2].v:=0;

                      Vert[3].x:=xStart-QW_BlockSize;
                      Vert[3].y:=Map[a, b].FloorHeight;
                      Vert[3].z:=zStart;
                      Vert[3].u:=QW_BlockSize;
                      Vert[3].v:=0;

                      AddPolygon(Vert, Map[a, b].Textures[QW_TexFloor], 4, a, b, 255);
                      Inc(CompiledMap[a, b].PolygonNum);
                   End;
                   // Decke
                   If Map[a, b].Textures[QW_TexCeiling] <> NoTextureAssigned Then Begin
                      Vert[0].x:=xStart-QW_BlockSize;
                      Vert[0].y:=Map[a, b].CeilingHeight[2];
                      Vert[0].z:=zStart+QW_BlockSize;
                      Vert[0].u:=QW_BlockSize;
                      Vert[0].v:=0;

                      Vert[1].x:=xStart-QW_BlockSize;
                      Vert[1].y:=Map[a, b].CeilingHeight[1];
                      Vert[1].z:=zStart;
                      Vert[1].u:=QW_BlockSize;
                      Vert[1].v:=QW_BlockSize;

                      Vert[2].x:=xStart;
                      Vert[2].y:=Map[a, b].CeilingHeight[0];
                      Vert[2].z:=zStart;
                      Vert[2].u:=0;
                      Vert[2].v:=QW_BlockSize;

                      Vert[3].x:=xStart;
                      Vert[3].y:=Map[a, b].CeilingHeight[3];
                      Vert[3].z:=zStart+QW_BlockSize;
                      Vert[3].u:=0;
                      Vert[3].v:=0;

                      AddPolygon(Vert, Map[a, b].Textures[QW_TexCeiling], 4, a, b, 255);
                      Inc(CompiledMap[a, b].PolygonNum);
                   End;
                   // Sockel unten
                   If Map[a, b].Textures[QW_TexPedestalFloor] <> NoTextureAssigned Then Begin
                      // zum Betrachter
                      if floorHitTest(a, b, 0, -1) then begin
                         Vert[0].x:=xStart-QW_BlockSize;
                         Vert[0].y:=map[a, b].floorHeight;
                         Vert[0].z:=zStart;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart;
                         Vert[1].y:=map[a, b].floorHeight;
                         Vert[1].z:=zStart;
                         Vert[1].u:=QW_BlockSize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart;
                         Vert[2].y:=Map[a, b-1].FloorHeight;
                         Vert[2].z:=zStart;
                         Vert[2].u:=QW_BlockSize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart-QW_BlockSize;
                         Vert[3].y:=Map[a, b-1].FloorHeight;
                         Vert[3].z:=zStart;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalFloor], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // vom Betrachter weg
                      if floorHitTest(a, b, 0, 1) then begin
                         Vert[0].x:=xStart;
                         Vert[0].y:=map[a, b].floorHeight;
                         Vert[0].z:=zStart+QW_BlockSize;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart-QW_BlockSize;
                         Vert[1].y:=map[a, b].floorHeight;
                         Vert[1].z:=zStart+QW_BlockSize;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart-QW_BlockSize;
                         Vert[2].y:=Map[a, b+1].FloorHeight;
                         Vert[2].z:=zStart+QW_BlockSize;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart;
                         Vert[3].y:=Map[a, b+1].FloorHeight;
                         Vert[3].z:=zStart+QW_BlockSize;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalFloor], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // rechts vom Betrachter
                      if floorHitTest(a, b, -1, 0) then begin
                         Vert[0].x:=xStart;
                         Vert[0].y:=map[a, b].floorHeight;
                         Vert[0].z:=zStart;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart;
                         Vert[1].y:=map[a, b].floorHeight;
                         Vert[1].z:=zStart+QW_BlockSize;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart;
                         Vert[2].y:=Map[a-1, b].FloorHeight;
                         Vert[2].z:=zStart+QW_BlockSize;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart;
                         Vert[3].y:=Map[a-1, b].FloorHeight;
                         Vert[3].z:=zStart;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalFloor], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // links vom Betrachter
                      if floorHitTest(a, b, 1, 0) then begin
                         Vert[0].x:=xStart-QW_Blocksize;
                         Vert[0].y:=map[a, b].floorHeight;
                         Vert[0].z:=zStart+QW_BlockSize;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart-QW_Blocksize;
                         Vert[1].y:=map[a, b].floorHeight;
                         Vert[1].z:=zStart;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart-QW_Blocksize;
                         Vert[2].y:=Map[a+1, b].FloorHeight;
                         Vert[2].z:=zStart;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart-QW_Blocksize;
                         Vert[3].y:=Map[a+1, b].FloorHeight;
                         Vert[3].z:=zStart+QW_BlockSize;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalFloor], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                   end;
                   // Sockel oben
                   If Map[a, b].Textures[QW_TexPedestalCeiling] <> NoTextureAssigned Then Begin
                      // zum Betrachter
                      if ceilingHitTest(a, b, 0, -1) then begin
                         Vert[0].x:=xStart-QW_BlockSize;
                         Vert[0].y:=map[a, b-1].ceilingHeight[2];
                         Vert[0].z:=zStart;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart;
                         Vert[1].y:=map[a, b-1].ceilingHeight[3];
                         Vert[1].z:=zStart;
                         Vert[1].u:=QW_BlockSize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart;
                         Vert[2].y:=Map[a, b].ceilingHeight[0];
                         Vert[2].z:=zStart;
                         Vert[2].u:=QW_BlockSize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart-QW_BlockSize;
                         Vert[3].y:=Map[a, b].ceilingHeight[1];
                         Vert[3].z:=zStart;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalCeiling], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // vom Betrachter weg
                      if ceilingHitTest(a, b, 0, 1) then begin
                         Vert[0].x:=xStart;
                         Vert[0].y:=map[a, b+1].ceilingHeight[0];
                         Vert[0].z:=zStart+QW_BlockSize;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart-QW_BlockSize;
                         Vert[1].y:=map[a, b+1].ceilingHeight[1];
                         Vert[1].z:=zStart+QW_BlockSize;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart-QW_BlockSize;
                         Vert[2].y:=Map[a, b].ceilingHeight[2];
                         Vert[2].z:=zStart+QW_BlockSize;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart;
                         Vert[3].y:=Map[a, b].ceilingHeight[3];
                         Vert[3].z:=zStart+QW_BlockSize;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalCeiling], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // rechts vom Betrachter
                      if ceilingHitTest(a, b, -1, 0) then begin
                         Vert[0].x:=xStart;
                         Vert[0].y:=map[a-1, b].ceilingHeight[1];
                         Vert[0].z:=zStart;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart;
                         Vert[1].y:=map[a-1, b].ceilingHeight[2];
                         Vert[1].z:=zStart+QW_BlockSize;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart;
                         Vert[2].y:=Map[a, b].ceilingHeight[3];
                         Vert[2].z:=zStart+QW_BlockSize;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart;
                         Vert[3].y:=Map[a, b].ceilingHeight[0];
                         Vert[3].z:=zStart;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalCeiling], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                      // links vom Betrachter
                      if ceilingHitTest(a, b, 1, 0) then begin
                         Vert[0].x:=xStart-QW_Blocksize;
                         Vert[0].y:=map[a+1, b].ceilingHeight[3];
                         Vert[0].z:=zStart+QW_BlockSize;
                         Vert[0].u:=0;
                         Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                         Vert[1].x:=xStart-QW_Blocksize;
                         Vert[1].y:=map[a+1, b].ceilingHeight[0];
                         Vert[1].z:=zStart;
                         Vert[1].u:=QW_Blocksize;
                         Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                         Vert[2].x:=xStart-QW_Blocksize;
                         Vert[2].y:=Map[a, b].ceilingHeight[1];
                         Vert[2].z:=zStart;
                         Vert[2].u:=QW_Blocksize;
                         Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                         Vert[3].x:=xStart-QW_Blocksize;
                         Vert[3].y:=Map[a, b].ceilingHeight[3];
                         Vert[3].z:=zStart+QW_BlockSize;
                         Vert[3].u:=0;
                         Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                         AddPolygon(Vert, Map[a, b].Textures[QW_TexPedestalCeiling], 4, a, b, 255);
                         Inc(CompiledMap[a, b].PolygonNum);
                      end;
                   End;

                   Case Map[a, b].BlockType Of
                        QW_BlockWater:
                           Begin
                              // Wasseroberflche
                              If Map[a, b].Textures[QW_TexWall] <> NoTextureAssigned Then Begin
                                 Vert[0].x:=xStart-QW_BlockSize;
                                 Vert[0].y:=Map[a, b].waterHeight;
                                 Vert[0].z:=zStart+QW_BlockSize;
                                 Vert[0].u:=QW_BlockSize;
                                 Vert[0].v:=QW_BlockSize;

                                 Vert[1].x:=xStart;
                                 Vert[1].y:=Map[a, b].waterHeight;
                                 Vert[1].z:=zStart+QW_BlockSize;
                                 Vert[1].u:=0;
                                 Vert[1].v:=QW_BlockSize;

                                 Vert[2].x:=xStart;
                                 Vert[2].y:=Map[a, b].waterHeight;
                                 Vert[2].z:=zStart;
                                 Vert[2].u:=0;
                                 Vert[2].v:=0;

                                 Vert[3].x:=xStart-QW_BlockSize;
                                 Vert[3].y:=Map[a, b].waterHeight;
                                 Vert[3].z:=zStart;
                                 Vert[3].u:=QW_BlockSize;
                                 Vert[3].v:=0;

                                 AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, map[a, b].Opacity);
                                 Inc(CompiledMap[a, b].PolygonNum);
                              End;
                           End;
                        QW_BlockWindowHoriz:
                           Begin
                              If Map[a, b].Textures[QW_TexWall] <> NoTextureAssigned Then Begin
                                 // zum Betrachter
                                 Vert[0].x:=xStart-QW_BlockSize;
                                 Vert[0].y:=map[a, b].CeilingHeight[0];
                                 Vert[0].z:=zStart+QW_BLockSize / 2;
                                 Vert[0].u:=0;
                                 Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                                 Vert[1].x:=xStart;
                                 Vert[1].y:=Map[a, b].CeilingHeight[0];
                                 Vert[1].z:=zStart+QW_BLockSize / 2;;
                                 Vert[1].u:=QW_BlockSize;
                                 Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                                 Vert[2].x:=xStart;
                                 Vert[2].y:=Map[a, b].FloorHeight;
                                 Vert[2].z:=zStart+QW_BLockSize / 2;;
                                 Vert[2].u:=QW_BlockSize;
                                 Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                                 Vert[3].x:=xStart-QW_BlockSize;
                                 Vert[3].y:=Map[a, b].FloorHeight;
                                 Vert[3].z:=zStart+QW_BLockSize / 2;;
                                 Vert[3].u:=0;
                                 Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                                 AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, map[a, b].Opacity);
                                 Inc(CompiledMap[a, b].PolygonNum);
                              end;
                           End;
                        QW_BlockWindowVert:
                           Begin
                              If Map[a, b].Textures[QW_TexWall] <> NoTextureAssigned Then Begin
                                 Vert[0].x:=xStart-QW_Blocksize / 2;
                                 Vert[0].y:=map[a, b].CeilingHeight[0];
                                 Vert[0].z:=zStart+QW_BlockSize;
                                 Vert[0].u:=0;
                                 Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                                 Vert[1].x:=xStart-QW_Blocksize / 2;
                                 Vert[1].y:=Map[a, b].CeilingHeight[0];
                                 Vert[1].z:=zStart;
                                 Vert[1].u:=QW_Blocksize;
                                 Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                                 Vert[2].x:=xStart-QW_Blocksize / 2;
                                 Vert[2].y:=Map[a, b].FloorHeight;
                                 Vert[2].z:=zStart;
                                 Vert[2].u:=QW_Blocksize;
                                 Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                                 Vert[3].x:=xStart-QW_Blocksize / 2;
                                 Vert[3].y:=Map[a, b].FloorHeight;
                                 Vert[3].z:=zStart+QW_BlockSize;
                                 Vert[3].u:=0;
                                 Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                                 AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, map[a, b].Opacity);
                                 Inc(CompiledMap[a, b].PolygonNum);
                              end;
                           End;
                        QW_BlockColumn:
                           Begin
                              If Map[a, b].Textures[QW_TexWall] <> NoTextureAssigned Then Begin
                                 for i:=0 to columnFaceNum - 1 do begin
                                     Vert[0].x:=xStart - columnFace_XZ_U[i+1].x;
                                     Vert[0].y:=map[a, b].CeilingHeight[0];
                                     Vert[0].z:=zStart + columnFace_XZ_U[i+1].z;
                                     Vert[0].u:=columnFace_XZ_U[i+1].u;
                                     Vert[0].v:=(trunc(vert[0].y / QW_BlockSize)+1) * QW_Blocksize - vert[0].y + map[a, b].textureOffsets[QW_TexWall];

                                     Vert[1].x:=xStart - columnFace_XZ_U[i].x;
                                     Vert[1].y:=Map[a, b].CeilingHeight[0];
                                     Vert[1].z:=zStart + columnFace_XZ_U[i].z;
                                     Vert[1].u:=columnFace_XZ_U[i].u;
                                     Vert[1].v:=(trunc(vert[1].y / QW_BlockSize)+1) * QW_Blocksize - vert[1].y + map[a, b].textureOffsets[QW_TexWall];;

                                     Vert[2].x:=xStart - columnFace_XZ_U[i].x;
                                     Vert[2].y:=Map[a, b].FloorHeight;
                                     Vert[2].z:=zStart + columnFace_XZ_U[i].z;
                                     Vert[2].u:=columnFace_XZ_U[i].u;
                                     Vert[2].v:=vert[0].v + (vert[0].y - vert[2].y);

                                     Vert[3].x:=xStart - columnFace_XZ_U[i+1].x;
                                     Vert[3].y:=Map[a, b].FloorHeight;
                                     Vert[3].z:=zStart + columnFace_XZ_U[i+1].z;
                                     Vert[3].u:=columnFace_XZ_U[i+1].u;
                                     Vert[3].v:=vert[1].v + (vert[1].y - vert[3].y);;

                                     AddPolygon(Vert, Map[a, b].Textures[QW_TexWall], 4, a, b, 255);
                                     Inc(CompiledMap[a, b].PolygonNum);
                                 end;
                              end;
                           End;
                   End;
                end;

                Dec(xStart, QW_BlockSize);
            End;
            Inc(zStart, QW_BlockSize);

            // Anzeige aktualisieren
            Dummy:=FrmCompile.MmoCompile.Lines[FrmCompile.MmoCompile.Lines.Count-1];
            Dummy:=Dummy+'.';
            FrmCompile.MmoCompile.Lines[FrmCompile.MmoCompile.Lines.Count-1]:=Dummy;
        End;

        // Info ausgeben
        FrmCompile.Add('');
        FrmCompile.Add('Eckpunkte: '+Stri(VertexNum));
        FrmCompile.Add('Polygone: '+Stri(PolygonNum));
        FrmCompile.Add('Textur-Koord.: '+Stri(TexCoordNum));
        FrmCompile.Add('Indezies: '+Stri(IndexNum));

        // Datei erstellen
        FrmCompile.Add('');
        FrmCompile.Add('Erstelle '+OutputFile+'...');
        IOResult;
        System.Assign(f, OutputFile);
        ReWrite(f, 1);
        If IOResult <> 0 Then Begin
           Screen.Cursor:=crDefault;
           FrmCompile.Add('Fehler beim Erstellen von '+OutputFile+'!');
           Exit;
        End;

        // Header schreiben
        For i:=0 To 3 Do Header.Str[i]:=QW_QCMStr[i+1];
        Header.StartX:=PlayerStartPos.x;
        Header.StartY:=PlayerStartPos.y;
        Header.TextureNum:=TextureNum;
        Header.IndexNum:=IndexNum;
        Header.VertexNum:=VertexNum;
        Header.PolygonNum:=PolygonNum;
        Header.TexCoordNum:=TexCoordNum;
        BlockWrite(f, Header, SizeOf(Header));

        // Map-Daten schreiben
        BlockWrite(f, CompiledMap, SizeOf(CompiledMap));

        // Texturen schreiben
        For i:=0 To TextureNum - 1 Do Begin
            BlockWrite(f, Textures[i].AnimationType, SizeOf(Textures[i].AnimationType));
            BlockWrite(f, Textures[i].SubTextureNum, SizeOf(Textures[i].SubTextureNum));
            For m:=0 To Textures[i].SubTextureNum - 1 Do
                BlockWrite(f, Textures[i].SubTextures[m].FileName, SizeOf(Textures[i].SubTextures[m].Filename));
        End;

        // Vertex-Indecies schreiben
        BlockWrite(f, VertexIndecies, SizeOf(Word)*IndexNum);

        // Textur-Koordinaten-Indecies schreiben
        BlockWrite(f, TexCoordIndecies, SizeOf(Word)*IndexNum);

        // Vertecies schreiben
        BlockWrite(f, Vertecies, SizeOf(TVertex)*VertexNum);

        // Polygone schreiben
        BlockWrite(f, Polygons, SizeOf(TPolygon)*PolygonNum);

        // Textur-Koordinaten schreiben
        BlockWrite(f, TexCoords, SizeOf(TTexCoord)*TexCoordNum);

        // Datei schlieen
        System.Close(f);
        If IOResult <> 0 Then Begin
           Screen.Cursor:=crDefault;
           FrmCompile.Add('Fehler beim Schreiben von '+OutputFile+'!');
           Exit;
        End;

        // Ende!
        FrmCompile.Add('');
        FrmCompile.Add('Compilierung abgeschlossen.');

     Finally
        Screen.Cursor:=crDefault;
     End;

End;

end.
