unit test;

interface

uses
    Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, DirectDraw, Direct3D, StdCtrls, HiColor2, global, mmsystem, DXGUtils;

const
  displayWidth = 1280;
  displayHeight = 800;
  displayOffsetX: integer = displayWidth div 2;
  displayOffsetY: integer = displayHeight div 2;
  EyeDistance = displayWidth / 2;
  maxTmpTransPolygons = 999;
  viewPointHeightOverGround = QW_BlockSize - QW_BlockSize / 4;
  MAX_LIGHTSOURCES = 100;
  MAX_VIEW_RAYS = 1280;
//  MAX_VIEW_RAYS = 300000;

type
   TNormal = Packed Record
      x, y, z: Single;
   End;

  tViewPoint = record
     x, y, z: single;
     rotX, rotY, rotZ: single;
     sinX, cosX: single;
     sinY, cosY: single;
     mapX, mapY: integer;
  end;

  tTextureDesc = record
     AnimationType: Integer;
     SubTextureNum: Integer;
     SubTextures: Array[0..QW_MaxSubTextures] Of string[12];
  end;

  TMapBlock = Packed Record
     BlockType: Byte;
     Opacity: Byte;
     Flags: Word;
     FloorHeight: Single;
     MinCeilingHeight: Single;
     WaterHeight: Single;
     vertexData, vertexDataTrans: pointer;
  End;

  tTextureData = Packed Record
     texture: integer;
     vertexNum: integer;
  end;

  tVertexDataHeader = packed Record
     polygonNum: integer;
  end;

  tMapPolygon = packed record
     texture: integer;
     polygon: array[0..2] of TD3DTLVERTEX;
     normal: TNormal;
  end;

  tVertexData = Packed Record
     header: tVertexDataHeader;
     polygons: array[0..255] of tMapPolygon;
  end;

  tTransPolygon = packed record
     texture: integer;
     polygon: array[0..2] of TD3DTLVERTEX;
     normal: TNormal;
  end;

  tVertexDataTransHeader = packed record
     polygonNum: integer;
  end;

  tVertexDataTrans = packed record
     header: tVertexDataTransHeader;
     polygons: array[0..255] of tTransPolygon;
  end;

  tTexture = Packed Record
     activeSubTexture: IDirectDrawSurface7;
     activeSubTextureIndex: integer;
     AnimationType: Integer;
     SubTextureInc: integer;
     SubTextureNum: Integer;
     subTextures: array[0..QW_MaxSubTextures] of IDirectDrawSurface7;
  end;

  tTmpTransPolyIndex = packed record
     averageDepth: single;
     dataPtr: pointer;
  end;

  tLightSource = packed record
    x, y, z: single;
    r, g, b: single;
    radius: single;
  end;

  TfrmTest = class(TForm)
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    // verwendete Interfaces
    dd_Interface: IDirectDraw7;
    dd_Primary, dd_Back, dd_zbuf: IDirectDrawSurface7;
    d3d_Interface: IDirect3D7;
    d3d_Device: IDirect3DDevice7;
    //Variablen
    initialized: boolean;
    progPath: string;
    mapFilename: string;
    viewPoint: tViewPoint;
    map: Packed Array[0..QW_MapWidth-1, 0..QW_MapHeight-1] Of TMapBlock;
    Vertecies: Packed Array[0..QW_MaxVertex] Of TVertex;
    Polygons: Packed Array[0..QW_MaxPolygon] Of TPolygon;
    TexCoords: Packed Array[0..QW_MaxTexCoord] Of TTexCoord;
    TexCoordIndecies: Packed Array[0..QW_MaxIndex] Of Word;
    VertexIndecies: Packed Array[0..QW_MaxIndex] Of Word;
    VertexNum, PolygonNum, TexCoordNum, IndexNum: Integer;
    textureDesc: Packed Array[0..QW_MaxTextures] of tTextureDesc;
    textureNum, activeTextureIndex: integer;
    textures: array[0..QW_MaxTextures] of tTexture;
    tmpTransPoly: array[0..maxTmpTransPolygons] of tTransPolygon;
    tmpTransPolyIndecies: array[0..maxTmpTransPolygons] of tTmpTransPolyIndex;
    tmpTransPolyNum: integer;
    fps: Integer;
    targetfps: single;
    speedfactor: single;
    tickspersecond: int64;
    currentticks: int64;
    framedelay: int64;
    textureTicks: integer;
    lightSources: array[0..MAX_LIGHTSOURCES] of tLightSource;
    lightSourcesNum: integer;
    showInfo: boolean;
    polygonCounter: integer;
    // Funktionen
    function initDD(hwnd: Hwnd): boolean;
    function initD3D: boolean;
    procedure wipe;
    procedure drawOnIdle(Sender: TObject; var Done: Boolean);
    function LoadTextureFromPCX(filename: string; var texsurf: IDirectDrawSurface7): Boolean;
    function LoadTextureFromPCXMipMap(filename: string; var texsurf: IDirectDrawSurface7): Boolean;
    function loadMap(filename: string): boolean;
    function loadTextures: boolean;
    procedure applyTexture(textureIndex: integer);
    procedure renderBlock(x, y: integer);
    procedure renderTransPolygons;
    procedure initFrameCounter(fps: single);
    procedure calculateSpeedFactor;
    procedure setStartingViewPoint(startX, startY: integer);
    procedure calculateNextViewPoint;
    procedure animateTextures;
    procedure computeVertexLighting(var vert: TD3DTLVERTEX; var normal: TNormal);
  end;


var
  frmTest: TfrmTest;
  zbuf_flag: boolean;
  zbuf_format: integer;
  res: boolean;

implementation

{$R *.DFM}

procedure tFrmTest.initFrameCounter;
begin
     targetfps := fps;
     QueryPerformanceCounter(framedelay);
     QueryPerformanceFrequency(tickspersecond);
end;

procedure tFrmTest.calculateSpeedFactor;
begin
     if (tickspersecond = 0) or (targetfps = 0) then exit;
     QueryPerformanceCounter(currentTicks);
     speedfactor := (currentTicks-framedelay) / (tickspersecond / targetfps);
     if speedfactor <= 0 then speedfactor := 1;
     fps := round(targetfps / speedfactor);
     framedelay := currentticks;
end;

procedure TFrmTest.setStartingViewPoint;
begin
     with viewPoint do begin
          x := (QW_MapWidth * QW_BlockSize) - ((startX+1) * QW_BlockSize) + (QW_BlockSize / 2);
          z := (startY+1) * QW_BlockSize - (QW_BlockSize / 2);
          y := map[startX, startY].FloorHeight + viewPointHeightOverGround;
          rotX := 0;
          rotY := 0;
          rotZ := 0;
          sinX := sin(rotX);
          cosY := cos(rotX);
          sinY := sin(rotY);
          cosY := cos(rotY);
          mapX := startX;
          mapY := startY;
     end;
end;

procedure TFrmTest.calculateNextViewPoint;
var
   Keyboardstate: TKeyboardstate;
begin
     GetKeyboardState(Keyboardstate);

     with viewPoint do begin
          If (Keyboardstate[VK_LEFT] And 128) > 0 Then Begin
               rotY := rotY + (pi/80) * speedFactor;
          End;
          If (Keyboardstate[VK_RIGHT] And 128) > 0 Then Begin
               rotY := rotY - (pi/80) * speedFactor;
          End;

          If (Keyboardstate[VK_UP] And 128) > 0 Then Begin
               x := x + (cos(rotY + (pi/2)) * 6 * speedFactor);
               z := z + (sin(rotY + (pi/2)) * 6 * speedFactor);
          End;
          If (Keyboardstate[VK_DOWN] And 128) > 0 Then Begin
               x := x - (cos(rotY + (pi/2)) * 6 * speedFactor);
               z := z - (sin(rotY + (pi/2)) * 6 * speedFactor);
          End;

          If (Keyboardstate[Ord('A')] And 128) > 0 Then Begin
             viewpoint.rotX := viewpoint.rotX - (pi / 80) * speedFactor;
             if viewpoint.rotX < -(pi/2) then viewpoint.rotX := -(pi/2);
          end;
          If (Keyboardstate[Ord('Y')] And 128) > 0 Then Begin
             viewpoint.rotX := viewpoint.rotX + (pi / 80) * speedFactor;
             if viewpoint.rotX > (pi/2) then viewpoint.rotX := (pi/2);
          end;

          If (Keyboardstate[VK_SPACE] And 128) > 0 Then Begin
             viewpoint.rotX := 0;
          end;

          lightSources[0].x := viewPoint.x;
          lightSources[0].y := viewPoint.y;
          lightSources[0].z := viewPoint.z;

          with lightSources[0] do begin
               r := 0.75;
               g := 0.75;
               b := 0.75;
               radius := 384;
          end;

          sinX := sin(rotX);
          cosX := cos(rotX);
          sinY := sin(rotY);
          cosY := cos(rotY);

          mapX:=trunc((QW_Blocksize*QW_MapWidth-viewpoint.x) / QW_Blocksize);
          mapY:=trunc(viewpoint.z / QW_Blocksize);

          if (mapX >= 0) and (mapX < QW_MapWidth) and (mapY >= 0) and (mapY < QW_MapWidth) then
             viewpoint.y := map[mapX, mapY].FloorHeight + viewpointHeightOverGround
          else
             viewpoint.y := viewpointHeightOverGround;

     end;
end;

Function Stri(Zahl: Integer): String;
Var
   Tmp: String;
Begin
   Str(Zahl,Tmp);
   Stri:=Tmp;
End;

procedure tFrmTest.applyTexture;
begin
     if activeTextureIndex <> textureIndex then begin
        activeTextureIndex := textureIndex;
        d3d_device.SetTexture(0, textures[activeTextureIndex].activeSubTexture);
     end;
end;

procedure tFrmTest.computeVertexLighting(var vert: TD3DTLVERTEX; var normal: TNormal);
var
   distance, intensity: single;
   _r, _g, _b: single;
   l: integer;
   a,b,c: single;
   unitVector: record
      x, y, z: single;
   end;
   unify: single;
   dotproduct: single;
begin
      _r := 0;
      _g := 0;
      _b := 0;

      for l := 0 to lightSourcesNum-1 do begin
        a := (vert.sx - lightSources[l].x);
        b := (vert.sy - lightSources[l].y);
        c := (vert.sz - lightSources[l].z);

        distance := sqrt(a*a + b*b + c*c);

        if (distance < lightSources[l].radius) and (distance <> 0) then begin

            unify := 1 / distance;
            with unitVector do begin
                 x := a * unify;
                 y := b * unify;
                 z := c * unify;
            end;

            dotProduct := ((normal.x * (-unitVector.x)) + (normal.y * (-unitVector.y)) + (normal.z * (-unitVector.z)));

            if (dotProduct > 0) then begin
                intensity := (lightSources[l].radius - distance);
                intensity := (intensity / lightSources[l].radius) * dotproduct;

                _r := _r + intensity * lightSources[l].r;
                _g := _g + intensity * lightSources[l].g;
                _b := _b + intensity * lightSources[l].b;

                if _r > 1 then _r := 1;
                if _g > 1 then _g := 1;
                if _b > 1 then _b := 1;
            end;

        end;

      end;

      vert.color := D3DRGB(_r, _g, _b);
end;

procedure tFrmTest.renderBlock;
var
   i, m, l: integer;
   start: integer;
   vertNum: integer;
   a, b, c, a2, b2, c2: double;
   dummy, dummy2: tTransPolygon;
   averageDepth: single;
   verts: packed array[0..2] of TD3DTLVERTEX;
   pansen:boolean;
begin

     if map[x, y].vertexData <> NIL then begin
        for i:=0 to tVertexData(map[x, y].vertexData^).header.polygonNum - 1 do begin
              applyTexture(tVertexData(map[x, y].vertexData^).polygons[i].texture);

            {  pansen := false;
              for m:=0 to 2 do begin
                  verts[m]:=tVertexData(map[x, y].vertexData^).polygons[i].polygon[m];

                  //computeVertexLighting(verts[m], tVertexData(map[x, y].vertexData^).polygons[i].normal);

                  verts[m].sx := verts[m].sx - viewpoint.x;
                  verts[m].sy := -(verts[m].sy - viewpoint.y);
                  verts[m].sz := verts[m].sz - viewpoint.z;

                  // Drehung um Y-Achse
                  a:=(verts[m].sx*viewPoint.cosY)+(verts[m].sz*viewPoint.sinY);
                  b:=verts[m].sy;
                  c:=-(verts[m].sx*viewPoint.sinY)+(verts[m].sz*viewPoint.cosY);

                  // Drehung um X-Achse
                  a2:=a;
                  b2:=(b*viewpoint.cosX)-(c*viewpoint.sinX);
                  c2:=(b*viewpoint.sinX)+(c*viewpoint.cosX);

                  // 2D-Projektion
                  if  c2 = 0 then c2:=0.00001;

                  verts[m].sx := ((EyeDistance * a2) / c2) + displayOffsetX;
                  verts[m].sy := ((EyeDistance * b2) / c2) + displayOffsetY;

                  verts[m].sz := c2 / (QW_MapWidth * QW_BlockSize);
                  verts[m].rhw := 1 / verts[m].sz;

                  if (verts[m].sz <= 0) then pansen := true;
              end;

              if not pansen then d3d_device.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX, verts, 3, 0);
              inc(polygonCounter); }



              for m:=0 to 2 do begin
                  verts[m]:=tVertexData(map[x, y].vertexData^).polygons[i].polygon[m];

                  computeVertexLighting(verts[m], tVertexData(map[x, y].vertexData^).polygons[i].normal);

              end;

              d3d_device.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_LVERTEX, verts, 3, 0);
              inc(polygonCounter);



        end;
     end;

 {   if map[x, y].vertexDataTrans <> NIL then begin
        for i:=0 to tVertexDataTrans(map[x, y].vertexDataTrans^).header.polygonNum - 1 do begin
            averageDepth:=0;
            dummy:=tVertexDataTrans(map[x, y].vertexDataTrans^).polygons[i];

            for m:=0 to 2 do begin

                computeVertexLighting(dummy.polygon[m], tVertexData(map[x, y].vertexData^).polygons[i].normal);

                dummy.polygon[m].sx := dummy.polygon[m].sx - viewpoint.x;
                dummy.polygon[m].sy := -(dummy.polygon[m].sy - viewpoint.y);
                dummy.polygon[m].sz := dummy.polygon[m].sz - viewpoint.z;

                // Drehung um Y-Achse
                a:=(dummy.polygon[m].sx*viewPoint.cosY)+(dummy.polygon[m].sz*viewPoint.sinY);
                b:=dummy.polygon[m].sy;
                c:=-(dummy.polygon[m].sx*viewPoint.sinY)+(dummy.polygon[m].sz*viewPoint.cosY);

                // Drehung um X-Achse
                a2:=a;
                b2:=(b*viewpoint.cosX)-(c*viewpoint.sinX);
                c2:=(b*viewpoint.sinX)+(c*viewpoint.cosX);

                if  c2 = 0 then c2:=0.00001;

                dummy.polygon[m].sx:=((EyeDistance * a2) / c2)+displayOffsetX;
                dummy.polygon[m].sy:=((EyeDistance * b2) / c2)+displayOffsetY;

                dummy.polygon[m].sz:=c2 / (QW_MapWidth * QW_BlockSize);
                dummy.polygon[m].rhw:=1 / dummy.polygon[m].sz;

                averageDepth := averageDepth + dummy.polygon[m].sz;
            end;
            averageDepth := averageDepth / 3;

            tmpTransPoly[tmpTransPolyNum]:=dummy;
            tmpTransPolyIndecies[tmpTransPolyNum].averageDepth := averageDepth;
            tmpTransPolyIndecies[tmpTransPolyNum].dataPtr := @tmpTransPoly[tmpTransPolyNum];
            inc(tmpTransPolyNum);
            inc(polygonCounter);
        end;
     end;
 }
    if map[x, y].vertexDataTrans <> NIL then begin
        for i:=0 to tVertexDataTrans(map[x, y].vertexDataTrans^).header.polygonNum - 1 do begin
            averageDepth:=0;
            dummy:=tVertexDataTrans(map[x, y].vertexDataTrans^).polygons[i];
            dummy2:=dummy;

            for m:=0 to 2 do begin

//                computeVertexLighting(dummy.polygon[m], tVertexData(map[x, y].vertexData^).polygons[i].normal);
                computeVertexLighting(dummy2.polygon[m], tVertexData(map[x, y].vertexData^).polygons[i].normal);

                dummy.polygon[m].sx := dummy.polygon[m].sx - viewpoint.x;
                dummy.polygon[m].sy := -(dummy.polygon[m].sy - viewpoint.y);
                dummy.polygon[m].sz := dummy.polygon[m].sz - viewpoint.z;

                // Drehung um Y-Achse
                a:=(dummy.polygon[m].sx*viewPoint.cosY)+(dummy.polygon[m].sz*viewPoint.sinY);
                b:=dummy.polygon[m].sy;
                c:=-(dummy.polygon[m].sx*viewPoint.sinY)+(dummy.polygon[m].sz*viewPoint.cosY);

                // Drehung um X-Achse
                a2:=a;
                b2:=(b*viewpoint.cosX)-(c*viewpoint.sinX);
                c2:=(b*viewpoint.sinX)+(c*viewpoint.cosX);

                if  c2 = 0 then c2:=0.00001;

                dummy.polygon[m].sx:=((EyeDistance * a2) / c2)+displayOffsetX;
                dummy.polygon[m].sy:=((EyeDistance * b2) / c2)+displayOffsetY;

                dummy.polygon[m].sz:=c2 / (QW_MapWidth * QW_BlockSize);
                dummy.polygon[m].rhw:=1 / dummy.polygon[m].sz;

                averageDepth := averageDepth + dummy.polygon[m].sz;
            end;
            averageDepth := averageDepth / 3;

//            tmpTransPoly[tmpTransPolyNum]:=dummy;
            tmpTransPoly[tmpTransPolyNum]:=dummy2;
            tmpTransPolyIndecies[tmpTransPolyNum].averageDepth := averageDepth;
            tmpTransPolyIndecies[tmpTransPolyNum].dataPtr := @tmpTransPoly[tmpTransPolyNum];
            inc(tmpTransPolyNum);
            inc(polygonCounter);
        end;
     end;


end;

procedure TfrmTest.renderTransPolygons;
var
   i: integer;
   dummy: tTmpTransPolyIndex;
   sorted: boolean;
begin
     // Polygone nach mittlerer Tiefe sortieren
     repeat
           sorted := TRUE;
           for i:=0 to tmpTransPolyNum-2 do begin
               if tmpTransPolyIndecies[i].averageDepth > tmpTransPolyIndecies[i+1].averageDepth then begin
                  dummy := tmpTransPolyIndecies[i];
                  tmpTransPolyIndecies[i] := tmpTransPolyIndecies[i+1];
                  tmpTransPolyIndecies[i+1] := dummy;
                  sorted := FALSE;
               end;
           end;
     until sorted;

     // zeichnen

     d3d_device.SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, Cardinal(TRUE));
     d3d_device.SetRenderState(D3DRENDERSTATE_CULLMODE, Cardinal(D3DCULL_NONE));
     for i:=tmpTransPolyNum-1 downTo 0 do begin
         applyTexture(tTransPolygon(tmpTransPolyIndecies[i].dataPtr^).texture);
         //d3d_device.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_TLVERTEX, tTransPolygon(tmpTransPolyIndecies[i].dataPtr^).polygon[0] , 3, 0);
         d3d_device.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_LVERTEX, tTransPolygon(tmpTransPolyIndecies[i].dataPtr^).polygon[0] , 3, 0);
     end;
     d3d_device.SetRenderState(D3DRENDERSTATE_CULLMODE, Cardinal(D3DCULL_CCW));
     d3d_device.SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, Cardinal(FALSE));
end;

procedure TfrmTest.animateTextures;
var
  i, m: integer;
  tmp: integer;
begin
     tmp := gettickcount;
     if tmp - textureTicks > QW_animTicks then begin
        textureTicks := tmp;
     end else exit;
     for i:=0 to textureNum-1 do begin
         if textures[i].SubTextureNum > 1 then begin

             if textures[i].AnimationType = QW_animRepeat then begin

                if textures[i].activeSubTextureIndex < (textures[i].SubTextureNum-1) then begin
                   inc(textures[i].activeSubTextureIndex);
                end else begin
                   textures[i].activeSubTextureIndex := 0;
                end;

             end else begin

                inc(textures[i].activeSubTextureIndex, textures[i].SubTextureInc);
                if textures[i].activeSubTextureIndex < 0 then begin
                   textures[i].SubTextureInc := 1;
                   textures[i].activeSubTextureIndex := 1;
                end;

                if textures[i].activeSubTextureIndex > (textures[i].SubTextureNum-1) then begin
                   textures[i].SubTextureInc := -1;
                   textures[i].activeSubTextureIndex := textures[i].SubTextureNum-2;
                end;

             end;

            textures[i].activeSubTexture := textures[i].subTextures[textures[i].activeSubTextureIndex];
         end;
     end;
end;

procedure TfrmTest.drawOnIdle;
var
   dc: HDC;
   str: String;
   x, y, i: integer;
   Angle, AngleInc: double;
   PosX, PosZ: Integer;
   Matrix: Array[0..QW_MapWidth-1, 0..QW_MapHeight-1] Of Boolean;
   Sinus, Cosinus: double;
   IncX, IncZ, MapX, MapZ, NewZ: Integer;
   NewX: double;
   Slope: double;
   addFac: single;
   matrix_x, matrix_y, matrix_z: TD3DMatrix;
   matrix_view: TD3DMatrix;
   trans, proj: TD3DMatrix;
   
function testXY(mapX, mapY: integer): boolean;
begin
     mapX := QW_MapWidth - mapX -1;
     result := FALSE;
     if (mapX < 0) or (mapX >= QW_MapWidth) or (mapY < 0) or (mapY >= QW_MapHeight) then exit;
     if map[mapX, mapY].blockType = QW_BlockWall then exit;
     result := TRUE;
end;

procedure queryRenderBlock(mapX, mapY: integer);
begin
     mapX := QW_MapWidth - mapX-1;
     if (mapX < 0) or (mapX >= QW_MapWidth) or (mapY < 0) or (mapY >= QW_MapHeight) then exit;
     if (not matrix[mapX, mapY]) then begin
        matrix[mapX, mapY] := TRUE;
        renderBlock(mapX, mapY);
     end;
end;


begin
     done := FALSE;

     if initialized then begin

        If (dd_Primary.IsLost = DDERR_SURFACELOST) Then dd_Primary._Restore;
        If (dd_Back.IsLost = DDERR_SURFACELOST) Then dd_Back._Restore;
        If (dd_zbuf.IsLost = DDERR_SURFACELOST) Then dd_zbuf._Restore;

        d3d_device.SetTexture(0, textures[activeTextureIndex].activeSubTexture);

        d3d_device.Clear(0, NIL, D3DCLEAR_ZBUFFER or D3DCLEAR_TARGET, 0, 1.0, 0);

        // Projektionsmatrix
        SetProjectionMatrix(proj, pi / 2, displayHeight / displayWidth, 1, 99999.0);
        d3d_device.SetTransform(D3DTRANSFORMSTATE_PROJECTION, proj);

        // Welt-Matrix
        SetIdentityMatrix(matrix_view);
        d3d_device.SetTransform(D3DTRANSFORMSTATE_WORLD, matrix_view);

        // Ansicht-Matrix
        SetRotateXMatrix(matrix_x, viewPoint.rotX);
        SetRotateYMatrix(matrix_y, viewPoint.rotY);
        SetRotateZMatrix(matrix_z, viewPoint.rotZ);
        SetTranslateMatrix(trans, -viewPoint.x, -viewPoint.y, -viewPoint.z);
        matrix_view := MatrixMul(trans, matrix_view);
  //      matrix_view := MatrixMul(matrix_z, matrix_view);
        matrix_view := MatrixMul(matrix_y, matrix_view);
        matrix_view := MatrixMul(matrix_x, matrix_view);
        d3d_device.SetTransform(D3DTRANSFORMSTATE_VIEW, matrix_view);

        if succeeded(d3d_device.BeginScene) then begin

           addFac := (((pi * 2) - (pi / 2)) *abs(sin(viewPoint.rotX)));

           FillChar(Matrix, SizeOf(Matrix), #0);
           AngleInc:=((pi/1.5 + addFac) / MAX_VIEW_RAYS);
           Angle :=(viewPoint.roty + (pi/6) - (addFac / 2));

           polygonCounter := 0;
           tmpTransPolyNum := 0;

           For i:=0 To (MAX_VIEW_RAYS - 1) Do Begin

               Sinus:=sin(Angle);
               Cosinus:=cos(Angle);

               MapX:=trunc(viewPoint.x / QW_BlockSize);
               MapZ:=viewPoint.mapY;

               If Cosinus < 0 Then IncX:=-1 Else IncX:=1;
               If Sinus < 0 Then IncZ:=-1 Else IncZ:=1;

               If Cosinus = 0 Then Begin
                  Repeat
                        queryRenderBlock(MapX, MapZ);
                        Inc(MapZ, IncZ);
                  Until not testXY(mapX, mapZ);
                  queryRenderBlock(MapX, MapZ);
               End Else Begin

                  Slope:=Sinus / Cosinus;

                  If IncX > 0 Then begin
                     NewX:=QW_BlockSize - (viewpoint.x - MapX*QW_BlockSize);
                  end Else begin
                     NewX:= -(viewpoint.x - MapX*QW_BlockSize);
                  end;

                  Repeat
                        NewZ:=trunc(((Slope * NewX) + viewPoint.z) / QW_BlockSize);

                        While (NewZ <> MapZ) And testXY(mapX, mapZ) Do Begin
                              queryRenderBlock(mapX, mapZ);
                              inc(MapZ, IncZ);
                        End;

                        if testXY(mapX, mapZ) then begin
                           NewX:=NewX + (QW_BlockSize * IncX);
                           queryRenderBlock(mapX, mapZ);
                           inc(MapX, IncX);
                        end;

                  Until not testXY(mapX, mapZ);

                  queryRenderBlock(mapX, mapZ);
               End;

           Angle := (Angle + AngleInc);
           End;

           if tmpTransPolyNum > 0 then renderTransPolygons;

           d3d_device.endscene;
        end;

        dd_primary.Flip(NIL, DDFLIP_WAIT);

        calculateSpeedFactor;
        calculateNextViewPoint;

        animateTextures;

        if showInfo = true then begin
            dd_Primary.GetDC(dc);
            str:=Stri(fps)+' fps';
            Textout(dc, 0, 0, PChar(str), Length(str));
            str:=Stri(round(viewpoint.roty / pi * 360) mod 360)+'';
            Textout(dc, 0, 20, PChar(str), Length(str));
            str := 'POLYGONS: '+stri(polygonCounter);
            Textout(dc, 0, 40, PChar(str), Length(str));
            str := 'LIGHTS: '+stri(lightSourcesNum);
            Textout(dc, 0, 60, PChar(str), Length(str));
            dd_Primary.ReleaseDC(dc);
        end;
     end;
end;

function TfrmTest.initDD;
var
   ddResult: HResult;
   ddSurfDesc: TDDSURFACEDESC2;
   ddsCaps: TDDSCAPS2;
begin
     result := FALSE;

     ddResult := DirectDrawCreateEx(NIL, dd_interface, IDirectDraw7, NIL);
     if ddResult <> DD_OK then exit;

     ddResult := dd_interface.SetCooperativeLevel(hwnd, DDSCL_EXCLUSIVE Or DDSCL_FULLSCREEN or DDSCL_ALLOWREBOOT);
     if ddResult <> DD_OK then exit;

     ddResult := dd_interface.SetDisplayMode(displayWidth, displayHeight, 16, 0, 0);
     if ddResult <> DD_OK then exit;

     zeroMemory(@ddSurfDesc, sizeOf(ddSurfDesc));
     with ddSurfDesc do begin
          dwSize := sizeOf(ddSurfDesc);
          dwFlags := DDSD_CAPS Or DDSD_BACKBUFFERCOUNT;
          ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or DDSCAPS_FLIP or DDSCAPS_3DDEVICE
                            or DDSCAPS_COMPLEX;
          dwBackBufferCount := 1;
     end;

     ddResult := dd_interface.CreateSurface(ddSurfDesc, dd_Primary, NIL);
     if ddResult <> DD_OK then exit;

     zeroMemory(@ddsCaps, sizeOf(ddsCaps));
     ddsCaps.dwCaps := DDSCAPS_BACKBUFFER;

     ddResult := dd_Primary.GetAttachedSurface(ddsCaps, dd_Back);
     if ddResult <> DD_OK then exit;

     result:=TRUE;
end;

function zbufferCallBack(var lpDDPixFmt: TDDPixelFormat; lpContext: Pointer): HResult; stdcall;
begin
     if (lpContext = NIL) then begin
        result := D3DENUMRET_CANCEL;
        exit;
     end;

     if ((lpDDPixFmt.dwFlags = TDDPixelFormat(lpContext^).dwFlags) and
        (lpDDPixFmt.dwSize = sizeOf(TDDPIXELFORMAT)) and
        (lpDDPixFmt.dwZBufferBitDepth >= 16)) then begin
            TDDPixelFormat(lpContext^):=lpDDPixFmt;
            zbuf_flag := TRUE;
            Result := D3DENUMRET_CANCEL;
            Exit;
     End;

     Result := D3DENUMRET_OK;
end;

function TfrmTest.initD3D;
var
   view: TD3DVIEWPORT7;
   zbuf: TDDPIXELFORMAT;
   ddsd: TDDSURFACEDESC2;
   res: HRESULT;
   fStart, fEnd{, fDensity}: single;
   matrix: TD3DMatrix;
begin
     result := FALSE;

     if failed(dd_interface.QueryInterface(IID_IDirect3D7, d3d_interface)) then exit;

     zbuf_flag := FALSE;
     zbuf_format := 0;

     zeroMemory(@zbuf, sizeOf(zbuf));
     zbuf.dwSize := sizeOf(zbuf);
     zbuf.dwFlags := DDPF_ZBUFFER;
     d3d_interface.EnumZBufferFormats(IID_IDirect3DTnLHalDevice, zbufferCallBack, @zbuf);
     if zbuf_flag then
        zbuf_format := 1
     else begin
        d3d_interface.EnumZBufferFormats(IID_IDirect3DHalDevice, zbufferCallBack, @zbuf);
        if zbuf_flag then
           zbuf_format := 2
        else begin
           d3d_interface.EnumZBufferFormats(IID_IDirect3DMMXDevice, zbufferCallBack, @zbuf);
           if zbuf_flag then
              zbuf_format := 3
           else begin
              d3d_interface.EnumZBufferFormats(IID_IDirect3DRGBDevice, zbufferCallBack, @zbuf);
              if zbuf_flag then
                 zbuf_format := 4;
           end;
        end;
     end;

     if zbuf_format > 0 then begin
        zeroMemory(@ddsd, sizeOf(ddsd));
        ddsd.dwSize := sizeOf(ddsd);
        ddsd.dwFlags := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT or DDSD_PIXELFORMAT;

        if (zbuf_format < 3) then
           ddsd.ddsCaps.dwCaps := DDSCAPS_ZBUFFER or DDSCAPS_VIDEOMEMORY
        else
           ddsd.ddsCaps.dwCaps := DDSCAPS_ZBUFFER or DDSCAPS_SYSTEMMEMORY;

        ddsd.dwWidth := displayWidth;
        ddsd.dwHeight := displayHeight;
        ddsd.ddpfPixelFormat.dwSize := sizeOf(TDDPIXELFORMAT);
        ddsd.ddpfPixelFormat.dwFlags := DDPF_ZBUFFER;
        ddsd.ddpfPixelFormat := zbuf;

        res := dd_interface.createSurface(ddsd, dd_zbuf, NIL);
        if res <> DD_OK then exit;

        res := dd_back.AddAttachedSurface(dd_zbuf);
        if res <> DD_OK then exit;

     end;

     if d3d_interface.CreateDevice(IID_IDirect3DTnLHALDevice, dd_back, d3d_device) <> DD_OK then
        if d3d_interface.CreateDevice(IID_IDirect3DHALDevice, dd_back, d3d_device) <> DD_OK then
           if d3d_interface.CreateDevice(IID_IDirect3DMMXDevice, dd_back, d3d_device) <> DD_OK then
              if d3d_interface.CreateDevice(IID_IDirect3DRGBDevice, dd_back, d3d_device) <> DD_OK then
                 exit;


     with view do begin
          dwX := 0;
          dwY := 0;
          dwWidth := displayWidth;
          dwHeight := displayHeight;
          dvMinZ := 0.0;
          dvMaxZ := 1.0;
     end;
     if d3d_device.setViewPort(view) <> DD_OK then exit;

//     d3d_device.setrenderstate(D3DRENDERSTATE_ZENABLE, 1);
     d3d_device.setrenderstate(D3DRENDERSTATE_LIGHTING, 0);
//     d3d_device.setrenderstate(D3DRENDERSTATE_CULLMODE, Cardinal(D3DCULL_NONE));
     d3d_device.SetTextureStageState(0, D3DTSS_MIPFILTER, cardinal(D3DTFP_LINEAR));
     d3d_device.SetTextureStageState(0, D3DTSS_MINFILTER, Cardinal(D3DTFN_ANISOTROPIC));
     d3d_device.SetTextureStageState(0, D3DTSS_MAGFILTER, Cardinal(D3DTFG_ANISOTROPIC));
//     d3d_device.setrenderstate(D3DRENDERSTATE_TEXTUREMIN, cardinal(D3DFILTER_MIPLINEAR));
//     d3d_device.setrenderstate(D3DRENDERSTATE_TEXTUREMAG, cardinal(D3DFILTER_MIPLINEAR));

//     LoadTextureFromPCX(progPath+QW_TexturePath+'unreal2.pcx', tex);
     fStart := 0.0;
     fEnd   := 0.15;
//     fDensity := 0.01;

     // Enable fog blending.
{     d3d_device.SetRenderState(D3DRENDERSTATE_FOGENABLE, CARDINAL(TRUE));

     // Set the fog color.
     d3d_device.SetRenderState(D3DRENDERSTATE_FOGCOLOR, D3DRGB(0, 0, 0));

     d3d_device.SetRenderState(D3DRENDERSTATE_FOGTABLEMODE, Cardinal(D3DFOG_LINEAR));
     d3d_device.SetRenderState(D3DRENDERSTATE_FOGSTART, Cardinal((@fStart)^));
     d3d_device.SetRenderState(D3DRENDERSTATE_FOGEND,   Cardinal((@fEnd)^));}

     d3d_device.SetRenderState(D3DRENDERSTATE_ALPHABLENDENABLE, Cardinal(FALSE));
     d3d_device.SetRenderState(D3DRENDERSTATE_SRCBLEND, Cardinal(D3DBLEND_DESTCOLOR));
     d3d_device.SetRenderState(D3DRENDERSTATE_DESTBLEND, Cardinal(D3DBLEND_SRCCOLOR));

     //     d3d_device.SetRenderState(D3DRENDERSTATE_STIPPLEDALPHA, Cardinal(True));
     {    if(fUseRange)
        g_lpDevice->SetRenderState(
                       D3DRENDERSTATE_RANGEFOGENABLE,
                       TRUE); }

{     d3d_device.SetRenderState(D3DRENDERSTATE_FOGTABLEMODE, Cardinal(D3DFOG_EXP));
     d3d_device.SetRenderState(D3DRENDERSTATE_FOGDENSITY, Cardinal((@fDensity)^));}


     result := TRUE;
end;

procedure TfrmTest.wipe;
begin
     // von initD3D allokiert
     d3d_Device := NIL;
     d3d_Interface := NIL;
     dd_zbuf := NIL;

     // von initDD allokiert
     dd_Back := NIL;
     dd_Primary := NIL;
     dd_Interface := NIL;
end;

procedure TfrmTest.FormShow(Sender: TObject);
begin
     cursor:=crNone;

     if (not initDD(application.handle)) then begin
        exit;
     end;

     if (not initD3D) then begin
        exit;
     end;

     res:=loadmap(mapFilename);
//     if not res then exit;
//     res:=TRUE;

     initFrameCounter(100);
     initialized := TRUE;
end;

procedure TfrmTest.FormClose(Sender: TObject; var Action: TCloseAction);
{var
   sack: Int64;}
begin
     wipe;
     initialized := FALSE;
//     res := queryPerformanceCounter(sack);
//     if not res then messagedlg('scheisse', mtInformation, [mbok], 0);
//messagedlg(stri(sizeof(td3dtlvertex)), mtInformation, [mbok], 0);
{     if map[0, 0].vertexData <> nil then messagedlg(stri(round(tvertexdata(map[0,0].vertexdata^).vertecies[5].sx))+', '+
        stri(round(tvertexdata(map[0,0].vertexdata^).vertecies[5].sy))+', '+
        stri(round(tvertexdata(map[0,0].vertexdata^).vertecies[5].sz)), mtInformation, [mbok], 0);}
end;

procedure TfrmTest.FormCreate(Sender: TObject);
begin
     progPath:=extractFilePath(ParamStr(0));
     mapFilename:=progPath+QW_MapPath+'penis.qcm';
     initialized := FALSE;
     Application.OnIdle := drawOnIdle;
     viewPoint.rotY:=0;
end;

Function TextureEnumCallBack(var lpDDPixFmt: TDDPixelFormat; lpContext: Pointer): HResult; stdcall;
Begin
     If (lpDDPixFmt.dwFlags And (DDPF_LUMINANCE Or DDPF_BUMPLUMINANCE Or DDPF_BUMPDUDV)) <> 0 Then Begin
        Result:=DDENUMRET_OK;
        Exit;
     End;

     If lpDDPixFmt.dwFourCC <> 0 Then Begin
        Result:=DDENUMRET_OK;
        Exit;
     End;

     If (lpDDPixFmt.dwFlags And DDPF_ALPHAPIXELS) <> 0 Then Begin
        Result:=DDENUMRET_OK;
        Exit;
     End;

     If lpDDPixFmt.dwRGBBitCount <> 16 Then Begin
        Result:=DDENUMRET_OK;
        Exit;
     End;

     TDDPixelFormat(lpContext^):=lpDDPixFmt;
     Result:=DDENUMRET_CANCEL;
End;

procedure createMipMap(srcSprite, destSprite: HC_Sprite; srcWidth, destWidth: integer);
var
   mulFac, mulFacQuadrat: integer;
   x, y, m, n: integer;
   destX, destY: integer;
   r, g, b: byte;
   r_dest, g_dest, b_dest: integer;
begin
     mulFac := srcWidth div destWidth;
     mulFacQuadrat := mulFac * mulFac;
     y := 0;

     for destY := 0 to destWidth - 1 do begin

         x := 0;

         for destX := 0 to destWidth - 1 do begin

             r_dest := 0;
             g_dest := 0;
             b_dest := 0;

             for n := y to y + mulFac - 1 do begin
                 for m := x to x + mulFac - 1 do begin
                     hc_decodecolor(hc_getpixelsprite(m, n, srcSprite), r, g, b);

                     inc(r_dest, r shl 3);
                     inc(g_dest, g shl 3);
                     inc(b_dest, b shl 3);
                 end;

             end;

             hc_putpixelsprite(destX, destY, hc_encodecolor((r_dest div mulFacQuadrat) shr 3, (g_dest div mulFacQuadrat) shr 3, (b_dest div mulFacQuadrat) shr 3), destSprite);

             inc(x, mulFac);

         end;

         inc(y, mulFac);

     end;
end;

Function TFrmTest.LoadTextureFromPCXMipMap;
Var
   ddsd: TDDSURFACEDESC2;
   pddsRender, nextSurface, tmpSurf: IDirectDrawSurface7;
   pDD: IDirectDraw7;
   sprite: HC_Sprite;
   dummy: TDDSURFACEDESC2;
   i, mipmapCount, tmpWidth: integer;
   mipmaps: array[0..31] of HC_Sprite;
   x, y: integer;
   col1, col2, col3, col4: word;
   r, g, b: byte;
   r_dest, g_dest, b_dest: integer;
Begin
     Result:=False;

     FillChar(ddsd, SizeOf(ddsd), #0);
     With ddsd Do Begin
          dwSize:=SizeOf(ddsd);

          d3d_device.EnumTextureFormats(TextureEnumCallBack, @ddpfPixelFormat);
          If ddpfPixelFormat.dwRGBBitCount <> 16 Then Exit;

          HC_ExtractRGBBitCount(ddpfPixelFormat);
          HC_Error := HC_NoError;
          HC_LoadPCX8Bit(Filename, sprite);
          If HC_Error <> HC_NoError Then Exit;

          dwWidth:=sprite.Width;
          dwHeight:=sprite.Height;

          mipmapCount := 1;
          tmpWidth := sprite.Width;
          repeat
                tmpWidth := tmpWidth div 2;
                inc(mipmapCount);
          until tmpWidth <= 4;

          dwFlags:=DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH Or DDSD_PIXELFORMAT Or DDSD_TEXTURESTAGE or DDSD_MIPMAPCOUNT;
          ddsCaps.dwCaps:=DDSCAPS_TEXTURE or DDSCAPS_MIPMAP or DDSCAPS_COMPLEX or DDSCAPS_3DDEVICE;
          ddsCaps.dwCaps2:=DDSCAPS2_TEXTUREMANAGE;
          dwTextureStage:=0;
//          mipmapCount := 4;
          dwMipMapCount:=mipmapCount;
     End;

     If Failed(d3d_device.GetRenderTarget(pddsRender)) Then Exit;
     If Failed(pddsRender.GetDDInterface(IUnknown(pDD))) Then Exit;

     If Failed(pDD.CreateSurface(ddsd, texSurf, Nil)) Then Exit;

     FillChar(dummy, SizeOf(dummy), 0);
     dummy.dwSize:=SizeOf(dummy);
     If Failed(texSurf.Lock(Nil, dummy, DDLOCK_WAIT, 0)) Then Exit;
     Move(sprite.Addr^, dummy.lpSurface^, sprite.Width * sprite.Height * 2);
     If Failed(texSurf.UnLock(Nil)) Then Exit;

     nextSurface := texSurf;
     tmpWidth := sprite.Width div 2;

     mipmaps[0] := sprite;

     for i := 0 to mipmapCount - 2 do begin

         hc_createsprite(tmpWidth, tmpWidth, mipmaps[i+1]);
         createMipMap(mipmaps[0], mipmaps[i+1], mipmaps[0].Width, mipmaps[i+1].Width);
{         for y := 0 to tmpWidth - 1 do begin
             for x := 0 to tmpWidth - 1 do begin

                 r_dest := 0;
                 g_dest := 0;
                 b_dest := 0;

                 col1 := hc_getpixelsprite(x * 2, y * 2, mipmaps[i]);
                 col2 := hc_getpixelsprite((x * 2) + 1, y * 2, mipmaps[i]);
                 col3 := hc_getpixelsprite(x * 2, (y * 2) + 1, mipmaps[i]);
                 col4 := hc_getpixelsprite((x * 2) + 1, (y * 2) + 1, mipmaps[i]);

                 hc_decodecolor(col1, r, g, b);
                 r_dest := r_dest + r;
                 g_dest := g_dest + g;
                 b_dest := b_dest + b;

                 hc_decodecolor(col2, r, g, b);
                 r_dest := r_dest + r;
                 g_dest := g_dest + g;
                 b_dest := b_dest + b;

                 hc_decodecolor(col3, r, g, b);
                 r_dest := r_dest + r;
                 g_dest := g_dest + g;
                 b_dest := b_dest + b;

                 hc_decodecolor(col4, r, g, b);
                 r_dest := r_dest + r;
                 g_dest := g_dest + g;
                 b_dest := b_dest + b;

                 r_dest := round(r_dest / 4);
                 g_dest := round(g_dest / 4);
                 b_dest := round(b_dest / 4);

                 hc_putpixelsprite(x, y, hc_encodecolor(r_dest, g_dest, b_dest), mipmaps[i+1]);

             end;
         end;   }

         FillChar(dummy, SizeOf(dummy), 0);
         dummy.dwSize:=SizeOf(dummy);
         dummy.ddsCaps.dwCaps := DDSCAPS_MIPMAP Or DDSCAPS_TEXTURE;

//         tmpSurf._AddRef;
         tmpSurf := nextSurface;
         if failed(tmpSurf.GetAttachedSurface(dummy.ddsCaps, nextSurface)) then break;
//         tmpSurf._Release;
//         nextSurface := tmpSurf;

         FillChar(dummy, SizeOf(dummy), 0);
         dummy.dwSize:=SizeOf(dummy);
         If Failed(nextSurface.Lock(Nil, dummy, DDLOCK_WAIT, 0)) Then break;

         Move(mipmaps[i+1].Addr^, dummy.lpSurface^, tmpWidth * tmpWidth * 2);
{         FillChar(dummy.lpSurface^, tmpWidth * tmpWidth * 2, 255);
         if i = 1 then FillChar(dummy.lpSurface^, tmpWidth * tmpWidth * 2, 200);
         if i = 2 then FillChar(dummy.lpSurface^, tmpWidth * tmpWidth * 2, 90);}
         If Failed(nextSurface.UnLock(Nil)) Then break;

         tmpWidth := tmpWidth div 2;

     end;

     for i := 0 to mipmapCount - 1 do begin
         HC_FreeSprite(mipmaps[i]);
     end;
//     HC_FreeSprite(sprite);

     Result:=True;
End;

Function TFrmTest.LoadTextureFromPCX;
Var
   ddsd: TDDSURFACEDESC2;
   pddsRender: IDirectDrawSurface7;
   pDD: IDirectDraw7;
   sprite: HC_Sprite;
   dummy: TDDSURFACEDESC2;
Begin
     Result:=False;

     FillChar(ddsd, SizeOf(ddsd), #0);
     With ddsd Do Begin
          dwSize:=SizeOf(ddsd);

          d3d_device.EnumTextureFormats(TextureEnumCallBack, @ddpfPixelFormat);
          If ddpfPixelFormat.dwRGBBitCount <> 16 Then Exit;

          HC_ExtractRGBBitCount(ddpfPixelFormat);
          HC_Error := HC_NoError;
          HC_LoadPCX8Bit(Filename, sprite);
          If HC_Error <> HC_NoError Then Exit;

          dwWidth:=sprite.Width;
          dwHeight:=sprite.Height;
          dwFlags:=DDSD_CAPS Or DDSD_HEIGHT Or DDSD_WIDTH Or DDSD_PIXELFORMAT Or DDSD_TEXTURESTAGE;
          ddsCaps.dwCaps:=DDSCAPS_TEXTURE;
          ddsCaps.dwCaps2:=DDSCAPS2_TEXTUREMANAGE;
          dwTextureStage:=0;
     End;

     If Failed(d3d_device.GetRenderTarget(pddsRender)) Then Exit;
     If Failed(pddsRender.GetDDInterface(IUnknown(pDD))) Then Exit;

     If Failed(pDD.CreateSurface(ddsd, texSurf, Nil)) Then Exit;

     FillChar(dummy, SizeOf(dummy), 0);
     dummy.dwSize:=SizeOf(dummy);
     If Failed(texSurf.Lock(Nil, dummy, DDLOCK_WAIT, 0)) Then Exit;
     Move(sprite.Addr^, dummy.lpSurface^, sprite.Width * sprite.Height * 2);
     If Failed(texSurf.UnLock(Nil)) Then Exit;

     HC_FreeSprite(sprite);

     Result:=True; 
End;

function tFrmTest.loadTextures;
var
   i, m: integer;
begin
     result := FALSE;
     activeTextureIndex := 0;
     for i:=0 to textureNum-1 do begin
         textures[i].AnimationType := textureDesc[i].AnimationType;
         textures[i].SubTextureInc := 1;
         textures[i].SubTextureNum := textureDesc[i].SubTextureNum;
         for m:=0 to textures[i].SubTextureNum-1 do begin
             if not loadTextureFromPCXMipMap(progPath+QW_TexturePath+textureDesc[i].subTextures[m],
                                       textures[i].subTextures[m]) then exit;
         end;
         textures[i].activeSubTexture := textures[i].subTextures[0];
         textures[i].activeSubTextureIndex := 0;
     end;
     result := TRUE;
end;

function tFrmTest.loadMap;
var
   f: File;
   Header: TQCMHeader;
   CompiledMap: Packed Array[0..QW_MapWidth-1, 0..QW_MapHeight-1] Of TBlock;
   dummy: shortstring;
   x, y, i, m: integer;

procedure transferVertex(var d3d_vertex: TD3DTLVERTEX; vertex: TVertex; texCoord: TTexCoord);
begin
     with d3d_vertex, vertex, texCoord do begin
          sx := x;
          sy := y;
          sz := z;
          tu := u / QW_Blocksize;
          tv := v / QW_Blocksize;
          color := D3DRGB(r / 255, g / 255, b / 255);
          specular := color;
     end;
end;

function computeNormal(vec: array of TD3DTLVERTEX): TNormal;
var
   normal: tnormal;
   planeVec1, planeVec2: record
      x, y, z: single;
   end;
   length: single;
   unify: single;
begin
     with planeVec1 do begin
          x := vec[1].sx - vec[0].sx;
          y := vec[1].sy - vec[0].sy;
          z := vec[1].sz - vec[0].sz;
     end;
     with planeVec2 do begin
          x := vec[2].sx - vec[0].sx;
          y := vec[2].sy - vec[0].sy;
          z := vec[2].sz - vec[0].sz;
     end;
     normal.x := (planeVec1.y * planeVec2.z) - (planeVec1.z * planeVec2.y);
     normal.y := (planeVec1.z * planeVec2.x) - (planeVec1.x * planeVec2.z);
     normal.z := (planeVec1.x * planeVec2.y) - (planeVec1.y * planeVec2.x);

     length := sqrt(sqr(normal.x)+sqr(normal.y)+sqr(normal.z));
     if length <> 0 then begin
        unify := 1 / length;
        normal.x := normal.x * unify;
        normal.y := normal.y * unify;
        normal.z := normal.z * unify;
     end;
     result := normal;
end;

function createVertexData(firstPolygon, polygonNum: integer): pointer;
var
   polyIndecies: array[0..255] of integer;
   polyIndexNum: integer;
   mapPolygons: array[0..255] of tMapPolygon;
   mapPolygonNum: integer;
   i: integer;
   h: tVertexDataHeader;
   ptr: pointer;
   requiredMem: integer;
begin
     result := NIL;
     if polygonNum = 0 then exit;

     // wir sind nur an transparenten Polygonen interessiert
     polyIndexNum := 0;
     for i := 0 to polygonNum-1 do begin
         if polygons[i+firstPolygon].IsUsed = 0 then begin
            polyIndecies[polyIndexNum] := firstPolygon+i;
            inc(polyIndexNum);
         end;
     end;
     if polyIndexNum = 0 then exit;

     // geladene Daten in "brauchbare" Daten konvertieren
     mapPolygonNum := 0;
     for i := 0 to polyIndexNum-1 do begin
         if polygons[polyIndecies[i]].IndexNum = 3 then begin
            halt;
         end else if polygons[polyIndecies[i]].IndexNum = 4 then begin

            mapPolygons[mapPolygonNum].texture := polygons[polyIndecies[i]].Texture;
            transferVertex(mapPolygons[mapPolygonNum].polygon[0], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex]]);

            transferVertex(mapPolygons[mapPolygonNum].polygon[1], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+1]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+1]]);

            transferVertex(mapPolygons[mapPolygonNum].polygon[2], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+3]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+3]]);
            mapPolygons[mapPolygonNum].normal := computeNormal(mapPolygons[mapPolygonNum].polygon);
            inc(mapPolygonNum);


            mapPolygons[mapPolygonNum].texture := polygons[polyIndecies[i]].Texture;
            transferVertex(mapPolygons[mapPolygonNum].polygon[0], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+1]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+1]]);

            transferVertex(mapPolygons[mapPolygonNum].polygon[1], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+2]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+2]]);

            transferVertex(mapPolygons[mapPolygonNum].polygon[2], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+3]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+3]]);
            mapPolygons[mapPolygonNum].normal := computeNormal(mapPolygons[mapPolygonNum].polygon);
            inc(mapPolygonNum);
         end;
     end;
     h.PolygonNum := mapPolygonNum;

     // speicher belegen, Daten kopieren, Zeiger zurckgeben
     requiredMem := sizeOf(tVertexDataHeader) + (mapPolygonNum * sizeOf(TMapPolygon));
     getMem(ptr, requiredMem);
     with tVertexData(ptr^) do begin
          tVertexData(ptr^).header := h;
          for i:=0 to mapPolygonNum-1 do tVertexData(ptr^).polygons[i] := mapPolygons[i];
     end;
     result := ptr;

end;

procedure transferVertexTrans(var d3d_vertex: TD3DTLVERTEX; vertex: TVertex; texCoord: TTexCoord; opacity: byte);
begin
     with d3d_vertex, vertex, texCoord do begin
          sx := x;
          sy := y;
          sz := z;
          tu := u / QW_Blocksize;
          tv := v / QW_Blocksize;
          color := D3DRGBA(r / 255, g / 255, b / 255, 0.5);
          specular := color;
     end;
end;

function createVertexDataTrans(firstPolygon, polygonNum: integer): pointer;
var
   polyIndecies: array[0..255] of integer;
   polyIndexNum: integer;
   transPolygons: array[0..255] of tTransPolygon;
   h: tVertexDataTransHeader;
   transPolygonNum: integer;
   i: integer;
   ptr: pointer;
   requiredMem: integer;
begin
     result := NIL;
     if polygonNum = 0 then exit;

     // wir sind nur an transparenten Polygonen interessiert
     polyIndexNum := 0;
     for i := 0 to polygonNum-1 do begin
         if polygons[i+firstPolygon].IsUsed <> 0 then begin
            polyIndecies[polyIndexNum] := firstPolygon+i;
            inc(polyIndexNum);
         end;
     end;
     if polyIndexNum = 0 then exit;

     transPolygonNum := 0;
     for i := 0 to polyIndexNum-1 do begin
         if polygons[polyIndecies[i]].IndexNum = 3 then begin
         end else if polygons[polyIndecies[i]].IndexNum = 4 then begin

            transPolygons[transPolygonNum].texture := polygons[polyIndecies[i]].Texture;
            transferVertexTrans(transPolygons[transPolygonNum].polygon[0], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex]], polygons[polyIndecies[i]].opacity);

            transferVertexTrans(transPolygons[transPolygonNum].polygon[1], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+1]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+1]], polygons[polyIndecies[i]].opacity);

            transferVertexTrans(transPolygons[transPolygonNum].polygon[2], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+3]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+3]], polygons[polyIndecies[i]].opacity);
            inc(transPolygonNum);


            transPolygons[transPolygonNum].texture := polygons[polyIndecies[i]].Texture;
            transferVertexTrans(transPolygons[transPolygonNum].polygon[0], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+1]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+1]], polygons[polyIndecies[i]].opacity);

            transferVertexTrans(transPolygons[transPolygonNum].polygon[1], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+2]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+2]], polygons[polyIndecies[i]].opacity);

            transferVertexTrans(transPolygons[transPolygonNum].polygon[2], vertecies[vertexIndecies[polygons[polyIndecies[i]].firstIndex+3]],
                                           texCoords[texCoordIndecies[polygons[polyIndecies[i]].firstIndex+3]], polygons[polyIndecies[i]].opacity);
            inc(transPolygonNum);
         end;
     end;
     h.polygonNum := transPolygonNum;

     // speicher belegen, Daten kopieren, Zeiger zurckgeben
     requiredMem := sizeOf(tVertexDataTransHeader) + (transPolygonNum * sizeOf(tTransPolygon));
     getMem(ptr, requiredMem);
     tVertexDataTrans(ptr^).header := h;
     for i:=0 to transPolygonNum-1 do tVertexDataTrans(ptr^).polygons[i] := transPolygons[i];
     result := ptr;
end;

begin
     result := FALSE;

     if not fileExists(filename) then exit;

     try
        // Datei ffnen
        assignFile(f, filename);
        reset(f, 1);

        // Header einlesen
        zeroMemory(@Header, sizeOf(Header));
        blockRead(f, Header, sizeOf(Header));

        for i:=0 to 3 do dummy[i+1]:=header.str[i];
        dummy[0]:=chr(4);
        if dummy <> QW_QCMStr then exit;

        // Map-Daten lesen
        BlockRead(f, CompiledMap, SizeOf(CompiledMap));

        // Texturen (Dateinamen) lesen
        For i:=0 To header.TextureNum - 1 Do Begin
            blockread(f, TextureDesc[i].AnimationType, SizeOf(TextureDesc[i].AnimationType));
            blockread(f, textureDesc[i].SubTextureNum, SizeOf(textureDesc[i].SubTextureNum));
            For m:=0 To textureDesc[i].SubTextureNum - 1 Do
                blockread(f, textureDesc[i].SubTextures[m], SizeOf(textureDesc[i].SubTextures[m]));
        End;
        textureNum := header.textureNum;
//        indexNum:=header.IndexNum;

        // Vertex-Indecies lesen
        blockread(f, VertexIndecies, SizeOf(Word)*header.IndexNum);

        // Textur-Koordinaten-Indecies lesen
        blockread(f, TexCoordIndecies, SizeOf(Word)*header.IndexNum);

        // Vertecies lesen
        blockread(f, Vertecies, SizeOf(TVertex)*header.VertexNum);

        // Polygone lesen
        blockread(f, Polygons, SizeOf(TPolygon)*header.PolygonNum);

        // Textur-Koordinaten lesen
        blockread(f, TexCoords, SizeOf(TTexCoord)*header.TexCoordNum);

        closeFile(f);
     except
        on EInOutError do begin
           {$I-} closeFile(f); {$I+}
           IOResult;
           exit;
        end;
     end;

     for y:=0 to QW_MapHeight-1 do begin
         for x:=0 to QW_MapWidth-1 do begin
            map[x, y].BlockType:=compiledMap[x, y].BlockType;
            map[x, y].Opacity:=compiledMap[x, y].Opacity;
            map[x, y].Flags:=compiledMap[x, y].Flags;
            map[x, y].FloorHeight:=compiledMap[x, y].FloorHeight;
            map[x, y].MinCeilingHeight:=compiledMap[x, y].MinCeilingHeight;
            map[x, y].WaterHeight:=compiledMap[x, y].WaterHeight;
            map[x, y].vertexData := createVertexData(compiledMap[x, y].firstPolygon, compiledMap[x, y].polygonNum);
            map[x, y].vertexDataTrans := createVertexDataTrans(compiledMap[x, y].firstPolygon, compiledMap[x, y].polygonNum);
         end;
     end;

     setStartingViewPoint(header.StartX, header.StartY);

     // Texturen einladen
     result := loadTextures;
     textureTicks := gettickcount;

     // Lichtquellen laden

     // VORLUFIG *********
     lightSourcesNum := 1;
     with lightSources[0] do begin
        x := 500;
        y := 64;
        z := 500;
        r := 0.5;
        g := 0.5;
        b := 0.5;
        radius := 1280;
     end;

     showInfo := false;

end;

procedure TfrmTest.FormKeyPress(Sender: TObject; var Key: Char);
begin
     if key = #27 then close;
     if key = 'q' then begin
        if lightSourcesNum < MAX_LIGHTSOURCES then begin
          with lightSources[lightSourcesNum] do begin
            x := viewpoint.x;
            y := viewpoint.y;
            z := viewpoint.z;
            r := 1.5;
            g := 1.5;
            b := 1.75;
            radius := 512;
          end;
          inc(lightSourcesNUm);
        end;
     end;

     if key = 'w' then begin
        if lightSourcesNum < MAX_LIGHTSOURCES then begin
          with lightSources[lightSourcesNum] do begin
            x := viewpoint.x;
            y := viewpoint.y;
            z := viewpoint.z;
            r := 1;
            g := 0;
            b := 0;
            radius := 256;
          end;
          inc(lightSourcesNUm);
        end;
     end;

     if key = 'e' then begin
        if lightSourcesNum < MAX_LIGHTSOURCES then begin
          with lightSources[lightSourcesNum] do begin
            x := viewpoint.x;
            y := viewpoint.y;
            z := viewpoint.z;
            r := 0;
            g := 1;
            b := 0;
            radius := 256;
          end;
          inc(lightSourcesNUm);
        end;
     end;


     if key = 'r' then begin
        if lightSourcesNum < MAX_LIGHTSOURCES then begin
          with lightSources[lightSourcesNum] do begin
            x := viewpoint.x;
            y := viewpoint.y;
            z := viewpoint.z;
            r := 0;
            g := 0;
            b := 1;
            radius := 256;
          end;
          inc(lightSourcesNUm);
        end;
     end;

     if key = 'd' then begin
        dec(lightSourcesNUm);
        if lightSourcesNUm < 0 then lightSourcesNum := 0;
     end;

     if (key = 'i') or (key = 'I') then begin
        showInfo := NOT showInfo;
     end;
end;

end.

